@@ -209,19 +209,19 @@ static BOOL AVIFILE_FormatsEqual(PAVISTREAM avi1, PAVISTREAM avi2)
     return FALSE;
 
   /* sizes match, now get formats and compare them */
-  fmt1 = HeapAlloc(GetProcessHeap(), 0, size1);
+  fmt1 = malloc(size1);
   if (fmt1 == NULL)
     return FALSE;
   if (SUCCEEDED(AVIStreamReadFormat(avi1, start1, fmt1, &size1))) {
-    fmt2 = HeapAlloc(GetProcessHeap(), 0, size1);
+    fmt2 = malloc(size1);
     if (fmt2 != NULL) {
       if (SUCCEEDED(AVIStreamReadFormat(avi2, start2, fmt2, &size1)))
         status = (memcmp(fmt1, fmt2, size1) == 0);
     }
   }
 
-  HeapFree(GetProcessHeap(), 0, fmt2);
-  HeapFree(GetProcessHeap(), 0, fmt1);
+  free(fmt2);
+  free(fmt1);
 
   return status;
 }
@@ -278,10 +278,10 @@ static ULONG   WINAPI IAVIEditStream_fnRelease(IAVIEditStream*iface)
         if (This->pStreams[i].pStream != NULL)
           IAVIStream_Release(This->pStreams[i].pStream);
       }
-      HeapFree(GetProcessHeap(), 0, This->pStreams);
+      free(This->pStreams);
     }
 
-    HeapFree(GetProcessHeap(), 0, This);
+    free(This);
   }
   return ref;
 }
@@ -343,8 +343,7 @@ static HRESULT WINAPI IAVIEditStream_fnCut(IAVIEditStream*iface,LONG*plStart,
     } else {
       /* splitting */
       if (This->nStreams + 1 >= This->nTableSize) {
-        This->pStreams = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, This->pStreams,
-                                     (This->nTableSize + 32) * sizeof(EditStreamTable));
+        This->pStreams = _recalloc(This->pStreams, This->nTableSize + 32, sizeof(EditStreamTable));
         if (This->pStreams == NULL)
           return AVIERR_MEMORY;
         This->nTableSize += 32;
@@ -530,7 +529,7 @@ static HRESULT WINAPI IAVIEditStream_fnPaste(IAVIEditStream*iface,LONG*plStart,
   if (This->nStreams + nStreams + 1 > This->nTableSize) {
     n = This->nStreams + nStreams + 33;
 
-    This->pStreams = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, This->pStreams, n * sizeof(EditStreamTable));
+    This->pStreams = _recalloc(This->pStreams, n, sizeof(EditStreamTable));
     if (This->pStreams == NULL)
       return AVIERR_MEMORY;
     This->nTableSize = n;
@@ -619,8 +618,7 @@ static HRESULT WINAPI IAVIEditStream_fnClone(IAVIEditStream*iface,
   if (pEdit == NULL)
     return AVIERR_MEMORY;
   if (This->nStreams > pEdit->nTableSize) {
-    pEdit->pStreams = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, pEdit->pStreams,
-                                  This->nStreams * sizeof(EditStreamTable));
+    pEdit->pStreams = _recalloc(pEdit->pStreams, This->nStreams, sizeof(EditStreamTable));
     if (pEdit->pStreams == NULL)
       return AVIERR_MEMORY;
     pEdit->nTableSize = This->nStreams;
@@ -702,7 +700,7 @@ static HRESULT WINAPI IEditAVIStream_fnCreate(IAVIStream*iface,
     return AVIERR_ERROR;
 
   if (This->pStreams == NULL) {
-    This->pStreams = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, 256 * sizeof(EditStreamTable));
+    This->pStreams = calloc(256, sizeof(EditStreamTable));
     if (This->pStreams == NULL)
       return AVIERR_MEMORY;
     This->nTableSize = 256;
@@ -1010,7 +1008,7 @@ static IAVIEditStreamImpl *AVIFILE_CreateEditStream(IAVIStream *pstream)
 {
   IAVIEditStreamImpl *pedit = NULL;
 
-  pedit = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(IAVIEditStreamImpl));
+  pedit = calloc(1, sizeof(IAVIEditStreamImpl));
   if (pedit == NULL)
     return NULL;
 
