@@ -76,18 +76,22 @@ static INT    (WINAPI *pFindMRUData)(HANDLE, LPCVOID, DWORD, LPINT);
 static INT    (WINAPI *pAddMRUData)(HANDLE, LPCVOID, DWORD);
 static HANDLE (WINAPI *pCreateMRUListW)(MRUINFOW*);
 
-static void InitPointers(void)
+static void init_functions(void)
 {
-    pCreateMRUListA = (void*)GetProcAddress(hComctl32,(LPCSTR)151);
-    pFreeMRUList    = (void*)GetProcAddress(hComctl32,(LPCSTR)152);
-    pAddMRUStringA  = (void*)GetProcAddress(hComctl32,(LPCSTR)153);
-    pEnumMRUListA   = (void*)GetProcAddress(hComctl32,(LPCSTR)154);
-    pCreateMRUListLazyA = (void*)GetProcAddress(hComctl32,(LPCSTR)157);
-    pAddMRUData     = (void*)GetProcAddress(hComctl32,(LPCSTR)167);
-    pFindMRUData    = (void*)GetProcAddress(hComctl32,(LPCSTR)169);
-    pCreateMRUListW = (void*)GetProcAddress(hComctl32,(LPCSTR)400);
-    pEnumMRUListW   = (void*)GetProcAddress(hComctl32,(LPCSTR)403);
-    pCreateMRUListLazyW = (void*)GetProcAddress(hComctl32,(LPCSTR)404);
+    hComctl32 = LoadLibraryA("comctl32.dll");
+
+#define X2(f, ord) p##f = (void*)GetProcAddress(hComctl32, (const char *)ord);
+    X2(CreateMRUListA, 151);
+    X2(FreeMRUList, 152);
+    X2(AddMRUStringA, 153);
+    X2(EnumMRUListA, 154);
+    X2(CreateMRUListLazyA, 157);
+    X2(AddMRUData, 167);
+    X2(FindMRUData, 169);
+    X2(CreateMRUListW, 400);
+    X2(EnumMRUListW, 403);
+    X2(CreateMRUListLazyW, 404);
+#undef X2
 }
 
 /* Based on RegDeleteTreeW from dlls/advapi32/registry.c */
@@ -113,10 +117,10 @@ static LSTATUS mru_RegDeleteTreeA(HKEY hKey, LPCSTR lpszSubKey)
     dwMaxSubkeyLen++;
     dwMaxValueLen++;
     dwMaxLen = max(dwMaxSubkeyLen, dwMaxValueLen);
-    if (dwMaxLen > sizeof(szNameBuf)/sizeof(CHAR))
+    if (dwMaxLen > ARRAY_SIZE(szNameBuf))
     {
         /* Name too big: alloc a buffer for it */
-        if (!(lpszName = HeapAlloc( GetProcessHeap(), 0, dwMaxLen*sizeof(CHAR))))
+        if (!(lpszName = malloc(dwMaxLen * sizeof(CHAR))))
         {
             ret = ERROR_NOT_ENOUGH_MEMORY;
             goto cleanup;
@@ -151,7 +155,7 @@ static LSTATUS mru_RegDeleteTreeA(HKEY hKey, LPCSTR lpszSubKey)
 cleanup:
     /* Free buffer if allocated */
     if (lpszName != szNameBuf)
-        HeapFree( GetProcessHeap(), 0, lpszName);
+        free(lpszName);
     if(lpszSubKey)
         RegCloseKey(hSubKey);
     return ret;
@@ -195,7 +199,7 @@ static void check_reg_entries(const char *mrulist, const char**items)
     buff[0] = '\0';
     ret = RegQueryValueExA(hKey, "MRUList", NULL, &type, (LPBYTE)buff, &size);
 
-    ok(!ret && buff[0], "Checking MRU: got %d from RegQueryValueExW\n", ret);
+    ok(!ret && buff[0], "Checking MRU: got %ld from RegQueryValueExW\n", ret);
     if(ret || !buff[0]) return;
 
     ok(strcmp(buff, mrulist) == 0, "Checking MRU: Expected list %s, got %s\n",
@@ -212,7 +216,7 @@ static void check_reg_entries(const char *mrulist, const char**items)
         buff[0] = '\0';
         ret = RegQueryValueExA(hKey, name, NULL, &type, (LPBYTE)buff, &size);
         ok(!ret && buff[0],
-           "Checking MRU item %d ('%c'): got %d from RegQueryValueExW\n",
+           "Checking MRU item %d ('%c'): got %ld from RegQueryValueExW\n",
            i, mrulist[i], ret);
         if(ret || !buff[0]) return;
         ok(!strcmp(buff, items[mrulist[i]-'a']),
@@ -257,7 +261,7 @@ static void test_MRUListA(void)
     SetLastError(0);
     hMRU = pCreateMRUListA(&infoA);
     ok (!hMRU && !GetLastError(),
-        "CreateMRUListA(too small) expected NULL,0 got %p,%d\n",
+        "CreateMRUListA(too small) expected NULL,0 got %p,%ld\n",
         hMRU, GetLastError());
 
     /* size too big */
@@ -271,7 +275,7 @@ static void test_MRUListA(void)
     SetLastError(0);
     hMRU = pCreateMRUListA(&infoA);
     ok (!hMRU && !GetLastError(),
-        "CreateMRUListA(too big) expected NULL,0 got %p,%d\n",
+        "CreateMRUListA(too big) expected NULL,0 got %p,%ld\n",
         hMRU, GetLastError());
 
     /* NULL hKey */
@@ -285,7 +289,7 @@ static void test_MRUListA(void)
     SetLastError(0);
     hMRU = pCreateMRUListA(&infoA);
     ok (!hMRU && !GetLastError(),
-        "CreateMRUListA(NULL key) expected NULL,0 got %p,%d\n",
+        "CreateMRUListA(NULL key) expected NULL,0 got %p,%ld\n",
         hMRU, GetLastError());
 
     /* NULL subkey name */
@@ -299,7 +303,7 @@ static void test_MRUListA(void)
     SetLastError(0);
     hMRU = pCreateMRUListA(&infoA);
     ok (!hMRU && !GetLastError(),
-        "CreateMRUListA(NULL name) expected NULL,0 got %p,%d\n",
+        "CreateMRUListA(NULL name) expected NULL,0 got %p,%ld\n",
         hMRU, GetLastError());
 
     /* Create a string MRU */
@@ -316,7 +320,7 @@ static void test_MRUListA(void)
 
     hMRU = pCreateMRUListA(&infoA);
     ok(hMRU && !GetLastError(),
-       "CreateMRUListA(string) expected non-NULL,0 got %p,%d\n",
+       "CreateMRUListA(string) expected non-NULL,0 got %p,%ld\n",
        hMRU, GetLastError());
 
     if (hMRU)
@@ -331,39 +335,35 @@ static void test_MRUListA(void)
         SetLastError(0);
         iRet = pAddMRUStringA(NULL, checks[0]);
         ok(iRet == -1 && !GetLastError(),
-           "AddMRUStringA(NULL list) expected -1,0 got %d,%d\n",
+           "AddMRUStringA(NULL list) expected -1,0 got %d,%ld\n",
            iRet, GetLastError());
 
         /* Add (NULL string) */
-        if (0)
-        {
-	/* Some native versions crash when passed NULL or fail to SetLastError()  */
         SetLastError(0);
         iRet = pAddMRUStringA(hMRU, NULL);
-        ok(iRet == 0 && GetLastError() == ERROR_INVALID_PARAMETER,
-           "AddMRUStringA(NULL str) expected 0,ERROR_INVALID_PARAMETER got %d,%d\n",
+        todo_wine ok(iRet == 0 && !GetLastError(),
+           "AddMRUStringA(NULL str) expected 0,0 got %d,%ld\n",
            iRet, GetLastError());
-        }
 
         /* Add 3 strings. Check the registry is correct after each add */
         SetLastError(0);
         iRet = pAddMRUStringA(hMRU, checks[0]);
         ok(iRet == 0 && !GetLastError(),
-           "AddMRUStringA(1) expected 0,0 got %d,%d\n",
+           "AddMRUStringA(1) expected 0,0 got %d,%ld\n",
            iRet, GetLastError());
         check_reg_entries("a", checks);
 
         SetLastError(0);
         iRet = pAddMRUStringA(hMRU, checks[1]);
         ok(iRet == 1 && !GetLastError(),
-           "AddMRUStringA(2) expected 1,0 got %d,%d\n",
+           "AddMRUStringA(2) expected 1,0 got %d,%ld\n",
            iRet, GetLastError());
         check_reg_entries("ba", checks);
 
         SetLastError(0);
         iRet = pAddMRUStringA(hMRU, checks[2]);
         ok(iRet == 2 && !GetLastError(),
-           "AddMRUStringA(2) expected 2,0 got %d,%d\n",
+           "AddMRUStringA(2) expected 2,0 got %d,%ld\n",
            iRet, GetLastError());
         check_reg_entries("cba", checks);
 
@@ -373,7 +373,7 @@ static void test_MRUListA(void)
         SetLastError(0);
         iRet = pAddMRUStringA(hMRU, checks[1]);
         ok(iRet == 1 && !GetLastError(),
-           "AddMRUStringA(re-add 1) expected 1,0 got %d,%d\n",
+           "AddMRUStringA(re-add 1) expected 1,0 got %d,%ld\n",
            iRet, GetLastError());
         check_reg_entries("bca", checks);
 
@@ -381,7 +381,7 @@ static void test_MRUListA(void)
         SetLastError(0);
         iRet = pAddMRUStringA(hMRU, checks[3]);
         ok(iRet == 0 && !GetLastError(),
-           "AddMRUStringA(add new) expected 0,0 got %d,%d\n",
+           "AddMRUStringA(add new) expected 0,0 got %d,%ld\n",
            iRet, GetLastError());
         checks[0] = checks[3];
         check_reg_entries("abc", checks);
@@ -445,7 +445,7 @@ static void test_MRUListA(void)
         pFreeMRUList(hMRU);
     }
 
-    /* FreeMRUList(NULL) crashes on Win98 OSR0 */
+    pFreeMRUList(NULL); /* should not crash */
 }
 
 typedef struct {
@@ -475,7 +475,7 @@ static void test_CreateMRUListLazyA(void)
         return;
     }
 
-    for (i = 0; i < sizeof(create_lazyA)/sizeof(create_lazya_t); i++)
+    for (i = 0; i < ARRAY_SIZE(create_lazyA); i++)
     {
         const create_lazya_t *ptr = &create_lazyA[i];
         HANDLE hMRU;
@@ -499,13 +499,9 @@ static void test_EnumMRUList(void)
         return;
     }
 
-    /* NULL handle */
-    if (0)
-    {
-        /* crashes on NT4, passed on Win2k, XP, 2k3, Vista, 2k8 */
-        pEnumMRUListA(NULL, 0, NULL, 0);
-        pEnumMRUListW(NULL, 0, NULL, 0);
-    }
+    /* NULL handle - should not crash */
+    pEnumMRUListA(NULL, 0, NULL, 0);
+    pEnumMRUListW(NULL, 0, NULL, 0);
 }
 
 static void test_FindMRUData(void)
@@ -540,7 +536,6 @@ static void test_AddMRUData(void)
 
 static void test_CreateMRUListW(void)
 {
-    static const WCHAR mrutestW[] = {'M','R','U','T','e','s','t',0};
     MRUINFOW infoW;
     void *named;
     HKEY hKey;
@@ -563,7 +558,7 @@ static void test_CreateMRUListW(void)
     infoW.cbSize = sizeof(infoW);
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -575,7 +570,7 @@ static void test_CreateMRUListW(void)
     infoW.cbSize = sizeof(infoW) - 1;
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -587,7 +582,7 @@ static void test_CreateMRUListW(void)
     infoW.cbSize = sizeof(infoW) + 1;
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -599,7 +594,7 @@ static void test_CreateMRUListW(void)
     infoW.cbSize = 0;
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -611,7 +606,7 @@ static void test_CreateMRUListW(void)
     infoW.cbSize = sizeof(infoW);
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = NULL;
     infoW.lpfnCompare = NULL;
 
@@ -623,7 +618,6 @@ static void test_CreateMRUListW(void)
 
 static void test_CreateMRUListLazyW(void)
 {
-    static const WCHAR mrutestW[] = {'M','R','U','T','e','s','t',0};
     MRUINFOW infoW;
     void *named;
     HKEY hKey;
@@ -645,7 +639,7 @@ static void test_CreateMRUListLazyW(void)
     infoW.cbSize = sizeof(infoW);
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -657,7 +651,7 @@ static void test_CreateMRUListLazyW(void)
     infoW.cbSize = sizeof(infoW) - 1;
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -669,7 +663,7 @@ static void test_CreateMRUListLazyW(void)
     infoW.cbSize = sizeof(infoW) + 1;
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -681,7 +675,7 @@ static void test_CreateMRUListLazyW(void)
     infoW.cbSize = 0;
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = hKey;
     infoW.lpfnCompare = NULL;
 
@@ -693,7 +687,7 @@ static void test_CreateMRUListLazyW(void)
     infoW.cbSize = sizeof(infoW);
     infoW.uMax = 1;
     infoW.fFlags = 0;
-    infoW.lpszSubKey = mrutestW;
+    infoW.lpszSubKey = L"MRUTest";
     infoW.hKey = NULL;
     infoW.lpfnCompare = NULL;
 
@@ -705,13 +699,11 @@ static void test_CreateMRUListLazyW(void)
 
 START_TEST(mru)
 {
-    hComctl32 = GetModuleHandleA("comctl32.dll");
-
     delete_reg_entries();
     if (!create_reg_entries())
         return;
 
-    InitPointers();
+    init_functions();
 
     test_MRUListA();
     test_CreateMRUListLazyA();
