@@ -17,9 +17,6 @@
  *
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include "d3d10_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d10);
@@ -68,6 +65,44 @@ struct d3d10_stateblock
     BOOL predicate_value;
 };
 
+#define WINE_D3D10_TO_STR(x) case x: return #x
+
+static const char *debug_d3d10_device_state_types(D3D10_DEVICE_STATE_TYPES t)
+{
+    switch (t)
+    {
+        WINE_D3D10_TO_STR(D3D10_DST_SO_BUFFERS);
+        WINE_D3D10_TO_STR(D3D10_DST_OM_RENDER_TARGETS);
+        WINE_D3D10_TO_STR(D3D10_DST_OM_DEPTH_STENCIL_STATE);
+        WINE_D3D10_TO_STR(D3D10_DST_OM_BLEND_STATE);
+        WINE_D3D10_TO_STR(D3D10_DST_VS);
+        WINE_D3D10_TO_STR(D3D10_DST_VS_SAMPLERS);
+        WINE_D3D10_TO_STR(D3D10_DST_VS_SHADER_RESOURCES);
+        WINE_D3D10_TO_STR(D3D10_DST_VS_CONSTANT_BUFFERS);
+        WINE_D3D10_TO_STR(D3D10_DST_GS);
+        WINE_D3D10_TO_STR(D3D10_DST_GS_SAMPLERS);
+        WINE_D3D10_TO_STR(D3D10_DST_GS_SHADER_RESOURCES);
+        WINE_D3D10_TO_STR(D3D10_DST_GS_CONSTANT_BUFFERS);
+        WINE_D3D10_TO_STR(D3D10_DST_PS);
+        WINE_D3D10_TO_STR(D3D10_DST_PS_SAMPLERS);
+        WINE_D3D10_TO_STR(D3D10_DST_PS_SHADER_RESOURCES);
+        WINE_D3D10_TO_STR(D3D10_DST_PS_CONSTANT_BUFFERS);
+        WINE_D3D10_TO_STR(D3D10_DST_IA_VERTEX_BUFFERS);
+        WINE_D3D10_TO_STR(D3D10_DST_IA_INDEX_BUFFER);
+        WINE_D3D10_TO_STR(D3D10_DST_IA_INPUT_LAYOUT);
+        WINE_D3D10_TO_STR(D3D10_DST_IA_PRIMITIVE_TOPOLOGY);
+        WINE_D3D10_TO_STR(D3D10_DST_RS_VIEWPORTS);
+        WINE_D3D10_TO_STR(D3D10_DST_RS_SCISSOR_RECTS);
+        WINE_D3D10_TO_STR(D3D10_DST_RS_RASTERIZER_STATE);
+        WINE_D3D10_TO_STR(D3D10_DST_PREDICATION);
+        default:
+            FIXME("Unrecognised D3D10_DEVICE_STATE_TYPES %#x.\n", t);
+            return "unrecognised";
+    }
+}
+
+#undef WINE_D3D10_TO_STR
+
 static inline struct d3d10_stateblock *impl_from_ID3D10StateBlock(ID3D10StateBlock *iface)
 {
     return CONTAINING_RECORD(iface, struct d3d10_stateblock, ID3D10StateBlock_iface);
@@ -259,7 +294,7 @@ static ULONG STDMETHODCALLTYPE d3d10_stateblock_AddRef(ID3D10StateBlock *iface)
     struct d3d10_stateblock *stateblock = impl_from_ID3D10StateBlock(iface);
     ULONG refcount = InterlockedIncrement(&stateblock->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", stateblock, refcount);
+    TRACE("%p increasing refcount to %lu.\n", stateblock, refcount);
 
     return refcount;
 }
@@ -269,13 +304,13 @@ static ULONG STDMETHODCALLTYPE d3d10_stateblock_Release(ID3D10StateBlock *iface)
     struct d3d10_stateblock *stateblock = impl_from_ID3D10StateBlock(iface);
     ULONG refcount = InterlockedDecrement(&stateblock->refcount);
 
-    TRACE("%p decreasing refcount to %u.\n", stateblock, refcount);
+    TRACE("%p decreasing refcount to %lu.\n", stateblock, refcount);
 
     if (!refcount)
     {
         stateblock_cleanup(stateblock);
         ID3D10Device_Release(stateblock->device);
-        HeapFree(GetProcessHeap(), 0, stateblock);
+        free(stateblock);
     }
 
     return refcount;
@@ -522,8 +557,7 @@ HRESULT WINAPI D3D10CreateStateBlock(ID3D10Device *device,
 
     TRACE("device %p, mask %p, stateblock %p.\n", device, mask, stateblock);
 
-    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
-    if (!object)
+    if (!(object = calloc(1, sizeof(*object))))
     {
         ERR("Failed to allocate D3D10 stateblock object memory.\n");
         return E_OUTOFMEMORY;
@@ -662,9 +696,9 @@ HRESULT WINAPI D3D10StateBlockMaskDisableCapture(D3D10_STATE_BLOCK_MASK *mask,
             return stateblock_mask_clear_bits(&mask->SOBuffers, 1, start_idx, count);
         case D3D10_DST_OM_RENDER_TARGETS:
             return stateblock_mask_clear_bits(&mask->OMRenderTargets, 1, start_idx, count);
-        case D3D10_DST_DEPTH_STENCIL_STATE:
+        case D3D10_DST_OM_DEPTH_STENCIL_STATE:
             return stateblock_mask_clear_bits(&mask->OMDepthStencilState, 1, start_idx, count);
-        case D3D10_DST_BLEND_STATE:
+        case D3D10_DST_OM_BLEND_STATE:
             return stateblock_mask_clear_bits(&mask->OMBlendState, 1, start_idx, count);
         case D3D10_DST_VS:
             return stateblock_mask_clear_bits(&mask->VS, 1, start_idx, count);
@@ -749,9 +783,9 @@ HRESULT WINAPI D3D10StateBlockMaskEnableCapture(D3D10_STATE_BLOCK_MASK *mask,
             return stateblock_mask_set_bits(&mask->SOBuffers, 1, start_idx, count);
         case D3D10_DST_OM_RENDER_TARGETS:
             return stateblock_mask_set_bits(&mask->OMRenderTargets, 1, start_idx, count);
-        case D3D10_DST_DEPTH_STENCIL_STATE:
+        case D3D10_DST_OM_DEPTH_STENCIL_STATE:
             return stateblock_mask_set_bits(&mask->OMDepthStencilState, 1, start_idx, count);
-        case D3D10_DST_BLEND_STATE:
+        case D3D10_DST_OM_BLEND_STATE:
             return stateblock_mask_set_bits(&mask->OMBlendState, 1, start_idx, count);
         case D3D10_DST_VS:
             return stateblock_mask_set_bits(&mask->VS, 1, start_idx, count);
@@ -824,9 +858,9 @@ BOOL WINAPI D3D10StateBlockMaskGetSetting(D3D10_STATE_BLOCK_MASK *mask,
             return stateblock_mask_get_bit(&mask->SOBuffers, 1, idx);
         case D3D10_DST_OM_RENDER_TARGETS:
             return stateblock_mask_get_bit(&mask->OMRenderTargets, 1, idx);
-        case D3D10_DST_DEPTH_STENCIL_STATE:
+        case D3D10_DST_OM_DEPTH_STENCIL_STATE:
             return stateblock_mask_get_bit(&mask->OMDepthStencilState, 1, idx);
-        case D3D10_DST_BLEND_STATE:
+        case D3D10_DST_OM_BLEND_STATE:
             return stateblock_mask_get_bit(&mask->OMBlendState, 1, idx);
         case D3D10_DST_VS:
             return stateblock_mask_get_bit(&mask->VS, 1, idx);
