@@ -26,37 +26,41 @@
 #include <assert.h>
 #include <stdarg.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #define COBJMACROS
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "d3d8.h"
 #include "wine/wined3d.h"
 
 #define D3DPRESENTFLAGS_MASK 0x00000fffu
 
 #define D3D8_MAX_VERTEX_SHADER_CONSTANTF 256
+#define D3D8_MAX_PIXEL_SHADER_CONSTANTF 8
 #define D3D8_MAX_STREAMS 16
 
+#define D3DFMT_RESZ MAKEFOURCC('R','E','S','Z')
+#define D3D8_RESZ_CODE 0x7fa05000
+
 /* CreateVertexShader can return > 0xFFFF */
 #define VS_HIGHESTFIXEDFXF 0xF0000000
 
-extern const struct wined3d_parent_ops d3d8_null_wined3d_parent_ops DECLSPEC_HIDDEN;
+extern const struct wined3d_parent_ops d3d8_null_wined3d_parent_ops;
 
-void d3dcaps_from_wined3dcaps(D3DCAPS8 *caps, const struct wined3d_caps *wined3d_caps) DECLSPEC_HIDDEN;
+void d3dcaps_from_wined3dcaps(D3DCAPS8 *caps, const struct wined3d_caps *wined3d_caps,
+        unsigned int adapter_ordinal);
 
 struct d3d8
 {
     IDirect3D8 IDirect3D8_iface;
     LONG refcount;
     struct wined3d *wined3d;
+    struct wined3d_output **wined3d_outputs;
+    unsigned int wined3d_output_count;
 };
 
-BOOL d3d8_init(struct d3d8 *d3d8) DECLSPEC_HIDDEN;
+BOOL d3d8_init(struct d3d8 *d3d8);
 
 /*****************************************************************************
  * IDirect3DDevice8 implementation structure
@@ -107,20 +111,16 @@ struct d3d8_device
     struct wined3d_device_parent device_parent;
     LONG                    ref;
     struct wined3d_device  *wined3d_device;
-    IDirect3D8             *d3d_parent;
+    struct wined3d_device_context *immediate_context;
+    unsigned int            adapter_ordinal;
+    struct d3d8            *d3d_parent;
     struct                  d3d8_handle_table handle_table;
 
     /* FVF management */
     struct FvfToDecl       *decls;
     UINT                    numConvertedDecls, declArraySize;
 
-    /* User data draws */
-    struct wined3d_buffer *vertex_buffer;
-    UINT                   vertex_buffer_size;
-    UINT                   vertex_buffer_pos;
-    struct wined3d_buffer *index_buffer;
-    UINT                   index_buffer_size;
-    UINT                   index_buffer_pos;
+    struct wined3d_streaming_buffer vertex_buffer, index_buffer;
 
     LONG device_state;
     DWORD sysmem_vb : 16; /* D3D8_MAX_STREAMS */
@@ -128,15 +128,18 @@ struct d3d8_device
     DWORD in_destruction : 1;
     DWORD padding : 14;
 
+    unsigned int max_user_clip_planes, vs_uniform_count;
+
     /* The d3d8 API supports only one implicit swapchain (no D3DCREATE_ADAPTERGROUP_DEVICE,
      * no GetSwapchain, GetBackBuffer doesn't accept a swapchain number). */
     struct wined3d_swapchain *implicit_swapchain;
 
     struct wined3d_stateblock *recording, *state, *update_state;
+    const struct wined3d_stateblock_state *stateblock_state;
 };
 
 HRESULT device_init(struct d3d8_device *device, struct d3d8 *parent, struct wined3d *wined3d, UINT adapter,
-        D3DDEVTYPE device_type, HWND focus_window, DWORD flags, D3DPRESENT_PARAMETERS *parameters) DECLSPEC_HIDDEN;
+        D3DDEVTYPE device_type, HWND focus_window, DWORD flags, D3DPRESENT_PARAMETERS *parameters);
 
 static inline struct d3d8_device *impl_from_IDirect3DDevice8(IDirect3DDevice8 *iface)
 {
@@ -149,13 +152,13 @@ struct d3d8_resource
     struct wined3d_private_store private_store;
 };
 
-void d3d8_resource_cleanup(struct d3d8_resource *resource) DECLSPEC_HIDDEN;
-HRESULT d3d8_resource_free_private_data(struct d3d8_resource *resource, const GUID *guid) DECLSPEC_HIDDEN;
+void d3d8_resource_cleanup(struct d3d8_resource *resource);
+HRESULT d3d8_resource_free_private_data(struct d3d8_resource *resource, const GUID *guid);
 HRESULT d3d8_resource_get_private_data(struct d3d8_resource *resource, const GUID *guid,
-        void *data, DWORD *data_size) DECLSPEC_HIDDEN;
-void d3d8_resource_init(struct d3d8_resource *resource) DECLSPEC_HIDDEN;
+        void *data, DWORD *data_size);
+void d3d8_resource_init(struct d3d8_resource *resource);
 HRESULT d3d8_resource_set_private_data(struct d3d8_resource *resource, const GUID *guid,
-        const void *data, DWORD data_size, DWORD flags) DECLSPEC_HIDDEN;
+        const void *data, DWORD data_size, DWORD flags);
 
 struct d3d8_volume
 {
@@ -167,19 +170,20 @@ struct d3d8_volume
 };
 
 void volume_init(struct d3d8_volume *volume, struct wined3d_texture *wined3d_texture,
-        unsigned int sub_resource_idx, const struct wined3d_parent_ops **parent_ops) DECLSPEC_HIDDEN;
+        unsigned int sub_resource_idx, const struct wined3d_parent_ops **parent_ops);
 
 struct d3d8_swapchain
 {
     IDirect3DSwapChain8 IDirect3DSwapChain8_iface;
     LONG refcount;
     struct wined3d_swapchain *wined3d_swapchain;
+    struct wined3d_swapchain_state_parent state_parent;
     IDirect3DDevice8 *parent_device;
     unsigned int swap_interval;
 };
 
 HRESULT d3d8_swapchain_create(struct d3d8_device *device, struct wined3d_swapchain_desc *desc,
-        unsigned int swap_interval, struct d3d8_swapchain **swapchain) DECLSPEC_HIDDEN;
+        unsigned int swap_interval, struct d3d8_swapchain **swapchain);
 
 struct d3d8_surface
 {
@@ -192,15 +196,16 @@ struct d3d8_surface
     IDirect3DDevice8 *parent_device;
     IUnknown *container;
     struct d3d8_texture *texture;
+    struct wined3d_swapchain *swapchain;
 };
 
-struct wined3d_rendertarget_view *d3d8_surface_acquire_rendertarget_view(struct d3d8_surface *surface) DECLSPEC_HIDDEN;
-struct d3d8_device *d3d8_surface_get_device(const struct d3d8_surface *surface) DECLSPEC_HIDDEN;
+struct wined3d_rendertarget_view *d3d8_surface_acquire_rendertarget_view(struct d3d8_surface *surface);
+struct d3d8_surface *d3d8_surface_create(struct wined3d_texture *wined3d_texture,
+        unsigned int sub_resource_idx, IUnknown *container);
+struct d3d8_device *d3d8_surface_get_device(const struct d3d8_surface *surface);
 void d3d8_surface_release_rendertarget_view(struct d3d8_surface *surface,
-        struct wined3d_rendertarget_view *rtv) DECLSPEC_HIDDEN;
-void surface_init(struct d3d8_surface *surface, struct wined3d_texture *wined3d_texture, unsigned int sub_resource_idx,
-        const struct wined3d_parent_ops **parent_ops) DECLSPEC_HIDDEN;
-struct d3d8_surface *unsafe_impl_from_IDirect3DSurface8(IDirect3DSurface8 *iface) DECLSPEC_HIDDEN;
+        struct wined3d_rendertarget_view *rtv);
+struct d3d8_surface *unsafe_impl_from_IDirect3DSurface8(IDirect3DSurface8 *iface);
 
 struct d3d8_vertexbuffer
 {
@@ -210,11 +215,12 @@ struct d3d8_vertexbuffer
     IDirect3DDevice8 *parent_device;
     struct wined3d_buffer *draw_buffer;
     DWORD fvf, usage;
+    bool discarded;
 };
 
 HRESULT vertexbuffer_init(struct d3d8_vertexbuffer *buffer, struct d3d8_device *device,
-        UINT size, DWORD usage, DWORD fvf, D3DPOOL pool) DECLSPEC_HIDDEN;
-struct d3d8_vertexbuffer *unsafe_impl_from_IDirect3DVertexBuffer8(IDirect3DVertexBuffer8 *iface) DECLSPEC_HIDDEN;
+        UINT size, DWORD usage, DWORD fvf, D3DPOOL pool);
+struct d3d8_vertexbuffer *unsafe_impl_from_IDirect3DVertexBuffer8(IDirect3DVertexBuffer8 *iface);
 
 struct d3d8_indexbuffer
 {
@@ -222,31 +228,38 @@ struct d3d8_indexbuffer
     struct d3d8_resource resource;
     struct wined3d_buffer *wined3d_buffer;
     IDirect3DDevice8 *parent_device;
-    struct wined3d_buffer *draw_buffer;
     enum wined3d_format_id format;
     DWORD usage;
+    bool sysmem;
+    bool discarded;
 };
 
 HRESULT indexbuffer_init(struct d3d8_indexbuffer *buffer, struct d3d8_device *device,
-        UINT size, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
-struct d3d8_indexbuffer *unsafe_impl_from_IDirect3DIndexBuffer8(IDirect3DIndexBuffer8 *iface) DECLSPEC_HIDDEN;
+        UINT size, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+struct d3d8_indexbuffer *unsafe_impl_from_IDirect3DIndexBuffer8(IDirect3DIndexBuffer8 *iface);
 
 struct d3d8_texture
 {
     IDirect3DBaseTexture8 IDirect3DBaseTexture8_iface;
     struct d3d8_resource resource;
-    struct wined3d_texture *wined3d_texture;
-    IDirect3DDevice8 *parent_device;
+    struct wined3d_texture *wined3d_texture, *draw_texture;
+    struct d3d8_device *parent_device;
     struct list rtv_list;
 };
 
-HRESULT cubetexture_init(struct d3d8_texture *texture, struct d3d8_device *device,
-        UINT edge_length, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
-HRESULT texture_init(struct d3d8_texture *texture, struct d3d8_device *device,
-        UINT width, UINT height, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
-HRESULT volumetexture_init(struct d3d8_texture *texture, struct d3d8_device *device,
-        UINT width, UINT height, UINT depth, UINT levels, DWORD usage, D3DFORMAT format, D3DPOOL pool) DECLSPEC_HIDDEN;
-struct d3d8_texture *unsafe_impl_from_IDirect3DBaseTexture8(IDirect3DBaseTexture8 *iface) DECLSPEC_HIDDEN;
+HRESULT d3d8_texture_2d_init(struct d3d8_texture *texture, struct d3d8_device *device, unsigned int width,
+        unsigned int height, unsigned int level_count, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+HRESULT d3d8_texture_3d_init(struct d3d8_texture *texture, struct d3d8_device *device, unsigned int width,
+        unsigned int height, unsigned int depth, unsigned int levels, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+HRESULT d3d8_texture_cube_init(struct d3d8_texture *texture, struct d3d8_device *device,
+        unsigned int edge_length, unsigned int level_count, DWORD usage, D3DFORMAT format, D3DPOOL pool);
+
+static inline struct wined3d_texture *d3d8_texture_get_draw_texture(struct d3d8_texture *texture)
+{
+    return texture->draw_texture ? texture->draw_texture : texture->wined3d_texture;
+}
+
+struct d3d8_texture *unsafe_impl_from_IDirect3DBaseTexture8(IDirect3DBaseTexture8 *iface);
 
 struct d3d8_vertex_declaration
 {
@@ -257,11 +270,11 @@ struct d3d8_vertex_declaration
     DWORD shader_handle;
 };
 
-void d3d8_vertex_declaration_destroy(struct d3d8_vertex_declaration *declaration) DECLSPEC_HIDDEN;
+void d3d8_vertex_declaration_destroy(struct d3d8_vertex_declaration *declaration);
 HRESULT d3d8_vertex_declaration_init(struct d3d8_vertex_declaration *declaration,
-        struct d3d8_device *device, const DWORD *elements, DWORD shader_handle) DECLSPEC_HIDDEN;
+        struct d3d8_device *device, const DWORD *elements, DWORD shader_handle);
 HRESULT d3d8_vertex_declaration_init_fvf(struct d3d8_vertex_declaration *declaration,
-        struct d3d8_device *device, DWORD fvf) DECLSPEC_HIDDEN;
+        struct d3d8_device *device, DWORD fvf);
 
 struct d3d8_vertex_shader
 {
@@ -269,9 +282,9 @@ struct d3d8_vertex_shader
   struct wined3d_shader *wined3d_shader;
 };
 
-void d3d8_vertex_shader_destroy(struct d3d8_vertex_shader *shader) DECLSPEC_HIDDEN;
+void d3d8_vertex_shader_destroy(struct d3d8_vertex_shader *shader);
 HRESULT d3d8_vertex_shader_init(struct d3d8_vertex_shader *shader, struct d3d8_device *device,
-        const DWORD *declaration, const DWORD *byte_code, DWORD shader_handle, DWORD usage) DECLSPEC_HIDDEN;
+        const DWORD *declaration, const DWORD *byte_code, DWORD shader_handle, DWORD usage);
 
 struct d3d8_pixel_shader
 {
@@ -279,15 +292,15 @@ struct d3d8_pixel_shader
     struct wined3d_shader *wined3d_shader;
 };
 
-void d3d8_pixel_shader_destroy(struct d3d8_pixel_shader *shader) DECLSPEC_HIDDEN;
+void d3d8_pixel_shader_destroy(struct d3d8_pixel_shader *shader);
 HRESULT d3d8_pixel_shader_init(struct d3d8_pixel_shader *shader, struct d3d8_device *device,
-        const DWORD *byte_code, DWORD shader_handle) DECLSPEC_HIDDEN;
+        const DWORD *byte_code, DWORD shader_handle);
 
-D3DFORMAT d3dformat_from_wined3dformat(enum wined3d_format_id format) DECLSPEC_HIDDEN;
-enum wined3d_format_id wined3dformat_from_d3dformat(D3DFORMAT format) DECLSPEC_HIDDEN;
-unsigned int wined3dmapflags_from_d3dmapflags(unsigned int flags, unsigned int usage) DECLSPEC_HIDDEN;
-void load_local_constants(const DWORD *d3d8_elements, struct wined3d_shader *wined3d_vertex_shader) DECLSPEC_HIDDEN;
-size_t parse_token(const DWORD *pToken) DECLSPEC_HIDDEN;
+D3DFORMAT d3dformat_from_wined3dformat(enum wined3d_format_id format);
+enum wined3d_format_id wined3dformat_from_d3dformat(D3DFORMAT format);
+unsigned int wined3dmapflags_from_d3dmapflags(unsigned int flags, unsigned int usage);
+void load_local_constants(const DWORD *d3d8_elements, struct wined3d_shader *wined3d_vertex_shader);
+size_t parse_token(const DWORD *pToken);
 
 static inline DWORD d3dusage_from_wined3dusage(unsigned int wined3d_usage, unsigned int bind_flags)
 {
@@ -301,6 +314,9 @@ static inline DWORD d3dusage_from_wined3dusage(unsigned int wined3d_usage, unsig
 
 static inline D3DPOOL d3dpool_from_wined3daccess(unsigned int access, unsigned int usage)
 {
+    if (usage & WINED3DUSAGE_MANAGED)
+        return D3DPOOL_MANAGED;
+
     switch (access & (WINED3D_RESOURCE_ACCESS_GPU | WINED3D_RESOURCE_ACCESS_CPU))
     {
         default:
@@ -346,6 +362,16 @@ static inline unsigned int wined3daccess_from_d3dpool(D3DPOOL pool, unsigned int
     return access;
 }
 
+static inline unsigned int wined3d_usage_from_d3d(D3DPOOL pool, DWORD usage)
+{
+    usage &= WINED3DUSAGE_MASK;
+    if (pool == D3DPOOL_SCRATCH)
+        usage |= WINED3DUSAGE_SCRATCH;
+    else if (pool == D3DPOOL_MANAGED)
+        usage |= WINED3DUSAGE_MANAGED;
+    return usage | WINED3DUSAGE_VIDMEM_ACCOUNTING;
+}
+
 static inline unsigned int wined3d_bind_flags_from_d3d8_usage(DWORD usage)
 {
     unsigned int bind_flags = 0;
@@ -358,4 +384,14 @@ static inline unsigned int wined3d_bind_flags_from_d3d8_usage(DWORD usage)
     return bind_flags;
 }
 
+static inline D3DMULTISAMPLE_TYPE d3dmultisample_type_from_wined3d(enum wined3d_multisample_type type)
+{
+    return (D3DMULTISAMPLE_TYPE)type;
+}
+
+static inline enum wined3d_device_type wined3d_device_type_from_d3d(D3DDEVTYPE type)
+{
+    return (enum wined3d_device_type)type;
+}
+
 #endif /* __WINE_D3DX8_PRIVATE_H */
