@@ -1,5 +1,5 @@
 /*
- * Copyright 2020 Ziqing Hui for CodeWeavers
+ * Copyright 2016 Andrey Gusev
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,14 +16,14 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "wine/debug.h"
-
 #define COBJMACROS
 
-#include "d3d10_1.h"
-#include "d3dx10.h"
+#include "d3dx11.h"
+#include "d3dcompiler.h"
 #include "dxhelpers.h"
-#include <assert.h>
+#include "assert.h"
+
+#include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3dx);
 
@@ -112,7 +112,6 @@ static DXGI_FORMAT dxgi_format_from_d3dx_pixel_format_id(enum d3dx_pixel_format_
         case D3DX_PIXEL_FORMAT_R10G10B10A2_UNORM:       return DXGI_FORMAT_R10G10B10A2_UNORM;
         case D3DX_PIXEL_FORMAT_R16G16B16A16_UNORM:      return DXGI_FORMAT_R16G16B16A16_UNORM;
         case D3DX_PIXEL_FORMAT_R8_UNORM:                return DXGI_FORMAT_R8_UNORM;
-        case D3DX_PIXEL_FORMAT_R8_SNORM:                return DXGI_FORMAT_R8_SNORM;
         case D3DX_PIXEL_FORMAT_R8G8_UNORM:              return DXGI_FORMAT_R8G8_UNORM;
         case D3DX_PIXEL_FORMAT_R16_UNORM:               return DXGI_FORMAT_R16_UNORM;
         case D3DX_PIXEL_FORMAT_R16G16_UNORM:            return DXGI_FORMAT_R16G16_UNORM;
@@ -140,7 +139,7 @@ static DXGI_FORMAT dxgi_format_from_d3dx_pixel_format_id(enum d3dx_pixel_format_
 
         /*
          * These have DXGI_FORMAT equivalents, but are explicitly unsupported on
-         * d3dx10.
+         * d3dx10+.
          */
         case D3DX_PIXEL_FORMAT_B5G6R5_UNORM:
         case D3DX_PIXEL_FORMAT_B5G5R5A1_UNORM:
@@ -153,25 +152,25 @@ static DXGI_FORMAT dxgi_format_from_d3dx_pixel_format_id(enum d3dx_pixel_format_
     }
 }
 
-static D3DX10_IMAGE_FILE_FORMAT d3dx10_image_file_format_from_d3dx_image_file_format(enum d3dx_image_file_format iff)
+static D3DX11_IMAGE_FILE_FORMAT d3dx11_image_file_format_from_d3dx_image_file_format(enum d3dx_image_file_format iff)
 {
     switch (iff)
     {
-        case D3DX_IMAGE_FILE_FORMAT_BMP: return D3DX10_IFF_BMP;
-        case D3DX_IMAGE_FILE_FORMAT_JPG: return D3DX10_IFF_JPG;
-        case D3DX_IMAGE_FILE_FORMAT_PNG: return D3DX10_IFF_PNG;
-        case D3DX_IMAGE_FILE_FORMAT_DDS: return D3DX10_IFF_DDS;
-        case D3DX_IMAGE_FILE_FORMAT_TIFF: return D3DX10_IFF_TIFF;
-        case D3DX_IMAGE_FILE_FORMAT_GIF: return D3DX10_IFF_GIF;
-        case D3DX_IMAGE_FILE_FORMAT_WMP: return D3DX10_IFF_WMP;
-        case D3DX_IMAGE_FILE_FORMAT_DDS_DXT10: return D3DX10_IFF_DDS;
+        case D3DX_IMAGE_FILE_FORMAT_BMP: return D3DX11_IFF_BMP;
+        case D3DX_IMAGE_FILE_FORMAT_JPG: return D3DX11_IFF_JPG;
+        case D3DX_IMAGE_FILE_FORMAT_PNG: return D3DX11_IFF_PNG;
+        case D3DX_IMAGE_FILE_FORMAT_DDS: return D3DX11_IFF_DDS;
+        case D3DX_IMAGE_FILE_FORMAT_TIFF: return D3DX11_IFF_TIFF;
+        case D3DX_IMAGE_FILE_FORMAT_GIF: return D3DX11_IFF_GIF;
+        case D3DX_IMAGE_FILE_FORMAT_WMP: return D3DX11_IFF_WMP;
+        case D3DX_IMAGE_FILE_FORMAT_DDS_DXT10: return D3DX11_IFF_DDS;
         default:
-            FIXME("No D3DX10_IMAGE_FILE_FORMAT for d3dx_image_file_format %d.\n", iff);
-            return D3DX10_IFF_FORCE_DWORD;
+            FIXME("No D3DX11_IMAGE_FILE_FORMAT for d3dx_image_file_format %d.\n", iff);
+            return D3DX11_IFF_FORCE_DWORD;
     }
 }
 
-HRESULT WINAPI D3DX10GetImageInfoFromFileA(const char *src_file, ID3DX10ThreadPump *pump, D3DX10_IMAGE_INFO *info,
+HRESULT WINAPI D3DX11GetImageInfoFromFileA(const char *src_file, ID3DX11ThreadPump *pump, D3DX11_IMAGE_INFO *info,
         HRESULT *result)
 {
     WCHAR *buffer;
@@ -192,14 +191,14 @@ HRESULT WINAPI D3DX10GetImageInfoFromFileA(const char *src_file, ID3DX10ThreadPu
         return E_OUTOFMEMORY;
 
     MultiByteToWideChar(CP_ACP, 0, src_file, -1, buffer, str_len);
-    hr = D3DX10GetImageInfoFromFileW(buffer, pump, info, result);
+    hr = D3DX11GetImageInfoFromFileW(buffer, pump, info, result);
 
     free(buffer);
 
     return hr;
 }
 
-HRESULT WINAPI D3DX10GetImageInfoFromFileW(const WCHAR *src_file, ID3DX10ThreadPump *pump, D3DX10_IMAGE_INFO *info,
+HRESULT WINAPI D3DX11GetImageInfoFromFileW(const WCHAR *src_file, ID3DX11ThreadPump *pump, D3DX11_IMAGE_INFO *info,
         HRESULT *result)
 {
     void *buffer = NULL;
@@ -212,25 +211,7 @@ HRESULT WINAPI D3DX10GetImageInfoFromFileW(const WCHAR *src_file, ID3DX10ThreadP
         return E_FAIL;
 
     if (pump)
-    {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncFileLoaderW(src_file, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureInfoProcessor(info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, result, NULL);
-        if (FAILED(hr))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
-    }
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
 
     if (SUCCEEDED((hr = load_file(src_file, &buffer, &size))))
     {
@@ -242,8 +223,8 @@ HRESULT WINAPI D3DX10GetImageInfoFromFileW(const WCHAR *src_file, ID3DX10ThreadP
     return hr;
 }
 
-HRESULT WINAPI D3DX10GetImageInfoFromResourceA(HMODULE module, const char *resource, ID3DX10ThreadPump *pump,
-        D3DX10_IMAGE_INFO *info, HRESULT *result)
+HRESULT WINAPI D3DX11GetImageInfoFromResourceA(HMODULE module, const char *resource, ID3DX11ThreadPump *pump,
+        D3DX11_IMAGE_INFO *info, HRESULT *result)
 {
     uint32_t size;
     void *buffer;
@@ -253,24 +234,7 @@ HRESULT WINAPI D3DX10GetImageInfoFromResourceA(HMODULE module, const char *resou
             module, debugstr_a(resource), pump, info, result);
 
     if (pump)
-    {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncResourceLoaderA(module, resource, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureInfoProcessor(info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        if (FAILED((hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, result, NULL))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
-    }
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
 
     if (FAILED((hr = d3dx_load_resource_a(module, resource, &buffer, &size))))
         return hr;
@@ -280,8 +244,8 @@ HRESULT WINAPI D3DX10GetImageInfoFromResourceA(HMODULE module, const char *resou
     return hr;
 }
 
-HRESULT WINAPI D3DX10GetImageInfoFromResourceW(HMODULE module, const WCHAR *resource, ID3DX10ThreadPump *pump,
-        D3DX10_IMAGE_INFO *info, HRESULT *result)
+HRESULT WINAPI D3DX11GetImageInfoFromResourceW(HMODULE module, const WCHAR *resource, ID3DX11ThreadPump *pump,
+        D3DX11_IMAGE_INFO *info, HRESULT *result)
 {
     uint32_t size;
     void *buffer;
@@ -291,24 +255,7 @@ HRESULT WINAPI D3DX10GetImageInfoFromResourceW(HMODULE module, const WCHAR *reso
             module, debugstr_w(resource), pump, info, result);
 
     if (pump)
-    {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncResourceLoaderW(module, resource, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureInfoProcessor(info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        if (FAILED((hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, result, NULL))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
-    }
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
 
     if (FAILED((hr = d3dx_load_resource_w(module, resource, &buffer, &size))))
         return hr;
@@ -318,9 +265,9 @@ HRESULT WINAPI D3DX10GetImageInfoFromResourceW(HMODULE module, const WCHAR *reso
     return hr;
 }
 
-static HRESULT d3dx10_image_info_from_d3dx_image(D3DX10_IMAGE_INFO *info, struct d3dx_image *image)
+static HRESULT d3dx11_image_info_from_d3dx_image(D3DX11_IMAGE_INFO *info, struct d3dx_image *image)
 {
-    D3DX10_IMAGE_FILE_FORMAT iff = d3dx10_image_file_format_from_d3dx_image_file_format(image->image_file_format);
+    D3DX11_IMAGE_FILE_FORMAT iff = d3dx11_image_file_format_from_d3dx_image_file_format(image->image_file_format);
     DXGI_FORMAT format;
 
     memset(info, 0, sizeof(*info));
@@ -335,7 +282,7 @@ static HRESULT d3dx10_image_info_from_d3dx_image(D3DX10_IMAGE_INFO *info, struct
             break;
 
         default:
-            if (iff == D3DX10_IFF_FORCE_DWORD)
+            if (iff == D3DX11_IFF_FORCE_DWORD)
                 return E_FAIL;
 
             /* All other image file formats use the default format. */
@@ -352,16 +299,16 @@ static HRESULT d3dx10_image_info_from_d3dx_image(D3DX10_IMAGE_INFO *info, struct
     switch (image->resource_type)
     {
         case D3DX_RESOURCE_TYPE_TEXTURE_2D:
-            info->ResourceDimension = D3D10_RESOURCE_DIMENSION_TEXTURE2D;
+            info->ResourceDimension = D3D11_RESOURCE_DIMENSION_TEXTURE2D;
             break;
 
         case D3DX_RESOURCE_TYPE_CUBE_TEXTURE:
-            info->ResourceDimension = D3D10_RESOURCE_DIMENSION_TEXTURE2D;
-            info->MiscFlags |= D3D10_RESOURCE_MISC_TEXTURECUBE;
+            info->ResourceDimension = D3D11_RESOURCE_DIMENSION_TEXTURE2D;
+            info->MiscFlags |= D3D11_RESOURCE_MISC_TEXTURECUBE;
             break;
 
         case D3DX_RESOURCE_TYPE_TEXTURE_3D:
-            info->ResourceDimension = D3D10_RESOURCE_DIMENSION_TEXTURE3D;
+            info->ResourceDimension = D3D11_RESOURCE_DIMENSION_TEXTURE3D;
             break;
 
         default:
@@ -379,7 +326,7 @@ static HRESULT d3dx10_image_info_from_d3dx_image(D3DX10_IMAGE_INFO *info, struct
     return S_OK;
 }
 
-HRESULT get_image_info(const void *data, SIZE_T size, D3DX10_IMAGE_INFO *img_info)
+HRESULT get_image_info(const void *data, SIZE_T size, D3DX11_IMAGE_INFO *img_info)
 {
     struct d3dx_image image;
     HRESULT hr;
@@ -389,7 +336,7 @@ HRESULT get_image_info(const void *data, SIZE_T size, D3DX10_IMAGE_INFO *img_inf
 
     hr = d3dx_image_init(data, size, &image, 0, D3DX_IMAGE_INFO_ONLY | D3DX_IMAGE_SUPPORT_DXT10);
     if (SUCCEEDED(hr))
-        hr = d3dx10_image_info_from_d3dx_image(img_info, &image);
+        hr = d3dx11_image_info_from_d3dx_image(img_info, &image);
 
     if (hr != S_OK)
     {
@@ -399,219 +346,7 @@ HRESULT get_image_info(const void *data, SIZE_T size, D3DX10_IMAGE_INFO *img_inf
     return S_OK;
 }
 
-HRESULT WINAPI D3DX10GetImageInfoFromMemory(const void *src_data, SIZE_T src_data_size, ID3DX10ThreadPump *pump,
-        D3DX10_IMAGE_INFO *img_info, HRESULT *result)
-{
-    HRESULT hr;
-
-    TRACE("src_data %p, src_data_size %Iu, pump %p, img_info %p, hresult %p.\n",
-            src_data, src_data_size, pump, img_info, result);
-
-    if (!src_data)
-        return E_FAIL;
-
-    if (pump)
-    {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncMemoryLoader(src_data, src_data_size, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureInfoProcessor(img_info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        if (FAILED((hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, result, NULL))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
-    }
-
-    hr = get_image_info(src_data, src_data_size, img_info);
-    if (result)
-        *result = hr;
-    return hr;
-}
-
-static HRESULT create_texture(ID3D10Device *device, const void *data, SIZE_T size,
-        D3DX10_IMAGE_LOAD_INFO *load_info, ID3D10Resource **texture)
-{
-    D3D10_SUBRESOURCE_DATA *resource_data;
-    D3DX10_IMAGE_LOAD_INFO load_info_copy;
-    D3DX10_IMAGE_INFO img_info;
-    HRESULT hr;
-
-    init_load_info(load_info, &load_info_copy);
-    if (!load_info_copy.pSrcInfo)
-        load_info_copy.pSrcInfo = &img_info;
-
-    if (FAILED((hr = load_texture_data(data, size, &load_info_copy, &resource_data))))
-        return hr;
-    hr = create_d3d_texture(device, &load_info_copy, resource_data, texture);
-    free(resource_data);
-    return hr;
-}
-
-HRESULT WINAPI D3DX10CreateTextureFromFileA(ID3D10Device *device, const char *src_file,
-        D3DX10_IMAGE_LOAD_INFO *load_info, ID3DX10ThreadPump *pump, ID3D10Resource **texture, HRESULT *hresult)
-{
-    WCHAR *buffer;
-    int str_len;
-    HRESULT hr;
-
-    TRACE("device %p, src_file %s, load_info %p, pump %p, texture %p, hresult %p.\n",
-            device, debugstr_a(src_file), load_info, pump, texture, hresult);
-
-    if (!device)
-        return E_INVALIDARG;
-    if (!src_file)
-        return E_FAIL;
-
-    if (!(str_len = MultiByteToWideChar(CP_ACP, 0, src_file, -1, NULL, 0)))
-        return HRESULT_FROM_WIN32(GetLastError());
-
-    if (!(buffer = malloc(str_len * sizeof(*buffer))))
-        return E_OUTOFMEMORY;
-
-    MultiByteToWideChar(CP_ACP, 0, src_file, -1, buffer, str_len);
-    hr = D3DX10CreateTextureFromFileW(device, buffer, load_info, pump, texture, hresult);
-
-    free(buffer);
-
-    return hr;
-}
-
-HRESULT WINAPI D3DX10CreateTextureFromFileW(ID3D10Device *device, const WCHAR *src_file,
-        D3DX10_IMAGE_LOAD_INFO *load_info, ID3DX10ThreadPump *pump, ID3D10Resource **texture, HRESULT *hresult)
-{
-    void *buffer = NULL;
-    DWORD size = 0;
-    HRESULT hr;
-
-    TRACE("device %p, src_file %s, load_info %p, pump %p, texture %p, hresult %p.\n",
-            device, debugstr_w(src_file), load_info, pump, texture, hresult);
-
-    if (!device)
-        return E_INVALIDARG;
-    if (!src_file)
-        return E_FAIL;
-
-    if (pump)
-    {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncFileLoaderW(src_file, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureProcessor(device, load_info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        if (FAILED((hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, hresult, (void **)texture))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
-    }
-
-    if (SUCCEEDED((hr = load_file(src_file, &buffer, &size))))
-    {
-        hr = create_texture(device, buffer, size, load_info, texture);
-        free(buffer);
-    }
-    if (hresult)
-        *hresult = hr;
-    return hr;
-}
-
-HRESULT WINAPI D3DX10CreateTextureFromResourceA(ID3D10Device *device, HMODULE module, const char *resource,
-        D3DX10_IMAGE_LOAD_INFO *load_info, ID3DX10ThreadPump *pump, ID3D10Resource **texture, HRESULT *hresult)
-{
-    uint32_t size;
-    void *buffer;
-    HRESULT hr;
-
-    TRACE("device %p, module %p, resource %s, load_info %p, pump %p, texture %p, hresult %p.\n",
-            device, module, debugstr_a(resource), load_info, pump, texture, hresult);
-
-    if (!device)
-        return E_INVALIDARG;
-
-    if (pump)
-    {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncResourceLoaderA(module, resource, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureProcessor(device, load_info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        if (FAILED((hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, hresult, (void **)texture))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
-    }
-
-    if (FAILED((hr = d3dx_load_resource_a(module, resource, &buffer, &size))))
-        return hr;
-    hr = create_texture(device, buffer, size, load_info, texture);
-    if (hresult)
-        *hresult = hr;
-    return hr;
-}
-
-HRESULT WINAPI D3DX10CreateTextureFromResourceW(ID3D10Device *device, HMODULE module, const WCHAR *resource,
-        D3DX10_IMAGE_LOAD_INFO *load_info, ID3DX10ThreadPump *pump, ID3D10Resource **texture, HRESULT *hresult)
-{
-    uint32_t size;
-    void *buffer;
-    HRESULT hr;
-
-    TRACE("device %p, module %p, resource %s, load_info %p, pump %p, texture %p, hresult %p.\n",
-            device, module, debugstr_w(resource), load_info, pump, texture, hresult);
-
-    if (!device)
-        return E_INVALIDARG;
-
-    if (pump)
-    {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncResourceLoaderW(module, resource, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureProcessor(device, load_info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        if (FAILED((hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, hresult, (void **)texture))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
-    }
-
-    if (FAILED((hr = d3dx_load_resource_w(module, resource, &buffer, &size))))
-        return hr;
-    hr = create_texture(device, buffer, size, load_info, texture);
-    if (hresult)
-        *hresult = hr;
-    return hr;
-}
-
-void init_load_info(const D3DX10_IMAGE_LOAD_INFO *load_info, D3DX10_IMAGE_LOAD_INFO *out)
+static void init_load_info(const D3DX11_IMAGE_LOAD_INFO *load_info, D3DX11_IMAGE_LOAD_INFO *out)
 {
     if (load_info)
     {
@@ -619,28 +354,28 @@ void init_load_info(const D3DX10_IMAGE_LOAD_INFO *load_info, D3DX10_IMAGE_LOAD_I
         return;
     }
 
-    out->Width = D3DX10_DEFAULT;
-    out->Height = D3DX10_DEFAULT;
-    out->Depth = D3DX10_DEFAULT;
-    out->FirstMipLevel = D3DX10_DEFAULT;
-    out->MipLevels = D3DX10_DEFAULT;
-    out->Usage = D3DX10_DEFAULT;
-    out->BindFlags = D3DX10_DEFAULT;
-    out->CpuAccessFlags = D3DX10_DEFAULT;
-    out->MiscFlags = D3DX10_DEFAULT;
-    out->Format = D3DX10_DEFAULT;
-    out->Filter = D3DX10_DEFAULT;
-    out->MipFilter = D3DX10_DEFAULT;
+    out->Width = D3DX11_DEFAULT;
+    out->Height = D3DX11_DEFAULT;
+    out->Depth = D3DX11_DEFAULT;
+    out->FirstMipLevel = D3DX11_DEFAULT;
+    out->MipLevels = D3DX11_DEFAULT;
+    out->Usage = D3DX11_DEFAULT;
+    out->BindFlags = D3DX11_DEFAULT;
+    out->CpuAccessFlags = D3DX11_DEFAULT;
+    out->MiscFlags = D3DX11_DEFAULT;
+    out->Format = D3DX11_DEFAULT;
+    out->Filter = D3DX11_DEFAULT;
+    out->MipFilter = D3DX11_DEFAULT;
     out->pSrcInfo = NULL;
 }
 
-HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO *load_info,
-        D3D10_SUBRESOURCE_DATA **resource_data)
+HRESULT load_texture_data(const void *data, SIZE_T size, D3DX11_IMAGE_LOAD_INFO *load_info,
+        D3D11_SUBRESOURCE_DATA **resource_data)
 {
     uint32_t loaded_mip_level_count, max_mip_level_count;
     const struct pixel_format_desc *fmt_desc, *src_desc;
     struct d3dx_subresource_data *sub_rsrcs = NULL;
-    D3DX10_IMAGE_INFO img_info;
+    D3DX11_IMAGE_INFO img_info;
     struct d3dx_image image;
     unsigned int i, j;
     HRESULT hr = S_OK;
@@ -649,8 +384,8 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
         return E_FAIL;
 
     *resource_data = NULL;
-    if (!load_info->Filter || load_info->Filter == D3DX10_DEFAULT)
-        load_info->Filter = D3DX10_FILTER_LINEAR;
+    if (!load_info->Filter || load_info->Filter == D3DX11_DEFAULT)
+        load_info->Filter = D3DX11_FILTER_LINEAR;
     if (FAILED(hr = d3dx_validate_filter(load_info->Filter)))
     {
         WARN("Invalid filter argument %#x.\n", load_info->Filter);
@@ -661,7 +396,7 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
     if (FAILED(hr))
         return E_FAIL;
 
-    hr = d3dx10_image_info_from_d3dx_image(&img_info, &image);
+    hr = d3dx11_image_info_from_d3dx_image(&img_info, &image);
     if (FAILED(hr))
     {
         WARN("Invalid or unsupported image file, hr %#lx.\n", hr);
@@ -669,7 +404,7 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
         goto end;
     }
 
-    if ((!(img_info.MiscFlags & D3D10_RESOURCE_MISC_TEXTURECUBE) || img_info.ArraySize != 6)
+    if ((!(img_info.MiscFlags & D3D11_RESOURCE_MISC_TEXTURECUBE) || img_info.ArraySize != 6)
             && img_info.ArraySize != 1)
     {
         FIXME("img_info.ArraySize = %u not supported.\n", img_info.ArraySize);
@@ -677,10 +412,10 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
         goto end;
     }
 
-    if (load_info->FirstMipLevel == D3DX10_DEFAULT || load_info->FirstMipLevel >= img_info.MipLevels)
+    if (load_info->FirstMipLevel == D3DX11_DEFAULT || load_info->FirstMipLevel >= img_info.MipLevels)
         load_info->FirstMipLevel = 0;
 
-    if (load_info->Format == D3DX10_DEFAULT || load_info->Format == DXGI_FORMAT_FROM_FILE)
+    if (load_info->Format == D3DX11_DEFAULT || load_info->Format == DXGI_FORMAT_FROM_FILE)
         load_info->Format = img_info.Format;
     fmt_desc = get_d3dx_pixel_format_info(d3dx_pixel_format_id_from_dxgi_format(load_info->Format));
     if (fmt_desc->format == D3DX_PIXEL_FORMAT_COUNT)
@@ -691,14 +426,14 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
     }
 
     /* Potentially round up width/height to align with block size. */
-    if (!load_info->Width || load_info->Width == D3DX10_FROM_FILE || load_info->Width == D3DX10_DEFAULT)
+    if (!load_info->Width || load_info->Width == D3DX11_FROM_FILE || load_info->Width == D3DX11_DEFAULT)
         load_info->Width = (img_info.Width + fmt_desc->block_width - 1) & ~(fmt_desc->block_width - 1);
-    if (!load_info->Height || load_info->Height == D3DX10_FROM_FILE || load_info->Height == D3DX10_DEFAULT)
+    if (!load_info->Height || load_info->Height == D3DX11_FROM_FILE || load_info->Height == D3DX11_DEFAULT)
         load_info->Height = (img_info.Height + fmt_desc->block_height - 1) & ~(fmt_desc->block_height - 1);
-    if (!load_info->Depth || load_info->Depth == D3DX10_FROM_FILE || load_info->Depth == D3DX10_DEFAULT)
+    if (!load_info->Depth || load_info->Depth == D3DX11_FROM_FILE || load_info->Depth == D3DX11_DEFAULT)
         load_info->Depth = img_info.Depth;
 
-    if ((load_info->Depth > 1) && (img_info.ResourceDimension != D3D10_RESOURCE_DIMENSION_TEXTURE3D))
+    if ((load_info->Depth > 1) && (img_info.ResourceDimension != D3D11_RESOURCE_DIMENSION_TEXTURE3D))
     {
         WARN("Invalid depth value %u for image with dimension %d.\n", load_info->Depth, img_info.ResourceDimension);
         hr = E_FAIL;
@@ -706,8 +441,8 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
     }
 
     max_mip_level_count = d3dx_get_max_mip_levels_for_size(load_info->Width, load_info->Height, load_info->Depth);
-    if (!load_info->MipLevels || load_info->MipLevels == D3DX10_DEFAULT || load_info->MipLevels == D3DX10_FROM_FILE)
-        load_info->MipLevels = (load_info->MipLevels == D3DX10_FROM_FILE) ? img_info.MipLevels : max_mip_level_count;
+    if (!load_info->MipLevels || load_info->MipLevels == D3DX11_DEFAULT || load_info->MipLevels == D3DX11_FROM_FILE)
+        load_info->MipLevels = (load_info->MipLevels == D3DX11_FROM_FILE) ? img_info.MipLevels : max_mip_level_count;
     load_info->MipLevels = min(max_mip_level_count, load_info->MipLevels);
 
     hr = d3dx_create_subresource_data_for_texture(load_info->Width, load_info->Height, load_info->Depth,
@@ -747,8 +482,8 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
         struct volume base_level_size = { load_info->Width, load_info->Height, load_info->Depth };
         const uint32_t base_level = loaded_mip_level_count - 1;
 
-        if (!load_info->MipFilter || load_info->MipFilter == D3DX10_DEFAULT)
-            load_info->MipFilter = D3DX10_FILTER_LINEAR;
+        if (!load_info->MipFilter || load_info->MipFilter == D3DX11_DEFAULT)
+            load_info->MipFilter = D3DX11_FILTER_LINEAR;
         if (FAILED(hr = d3dx_validate_filter(load_info->MipFilter)))
         {
             WARN("Invalid mip filter argument %#x.\n", load_info->MipFilter);
@@ -785,13 +520,13 @@ HRESULT load_texture_data(const void *data, SIZE_T size, D3DX10_IMAGE_LOAD_INFO
         }
     }
 
-    *resource_data = (D3D10_SUBRESOURCE_DATA *)sub_rsrcs;
+    *resource_data = (D3D11_SUBRESOURCE_DATA *)sub_rsrcs;
     sub_rsrcs = NULL;
 
-    load_info->Usage = (load_info->Usage == D3DX10_DEFAULT) ? D3D10_USAGE_DEFAULT : load_info->Usage;
-    load_info->BindFlags = (load_info->BindFlags == D3DX10_DEFAULT) ? D3D10_BIND_SHADER_RESOURCE : load_info->BindFlags;
-    load_info->CpuAccessFlags = (load_info->CpuAccessFlags == D3DX10_DEFAULT) ? 0 : load_info->CpuAccessFlags;
-    load_info->MiscFlags = (load_info->MiscFlags == D3DX10_DEFAULT) ? 0 : load_info->MiscFlags;
+    load_info->Usage = (load_info->Usage == D3DX11_DEFAULT) ? D3D11_USAGE_DEFAULT : load_info->Usage;
+    load_info->BindFlags = (load_info->BindFlags == D3DX11_DEFAULT) ? D3D11_BIND_SHADER_RESOURCE : load_info->BindFlags;
+    load_info->CpuAccessFlags = (load_info->CpuAccessFlags == D3DX11_DEFAULT) ? 0 : load_info->CpuAccessFlags;
+    load_info->MiscFlags = (load_info->MiscFlags == D3DX11_DEFAULT) ? 0 : load_info->MiscFlags;
     load_info->MiscFlags |= img_info.MiscFlags;
     /*
      * Must be present in order to get resource dimension for texture
@@ -806,18 +541,18 @@ end:
     return hr;
 }
 
-HRESULT create_d3d_texture(ID3D10Device *device, D3DX10_IMAGE_LOAD_INFO *load_info,
-        D3D10_SUBRESOURCE_DATA *resource_data, ID3D10Resource **texture)
+HRESULT create_d3d_texture(ID3D11Device *device, D3DX11_IMAGE_LOAD_INFO *load_info,
+        D3D11_SUBRESOURCE_DATA *resource_data, ID3D11Resource **texture)
 {
     HRESULT hr;
 
     *texture = NULL;
     switch (load_info->pSrcInfo->ResourceDimension)
     {
-        case D3D10_RESOURCE_DIMENSION_TEXTURE2D:
+        case D3D11_RESOURCE_DIMENSION_TEXTURE2D:
         {
-            D3D10_TEXTURE2D_DESC texture_2d_desc = { 0 };
-            ID3D10Texture2D *texture_2d;
+            D3D11_TEXTURE2D_DESC texture_2d_desc = { 0 };
+            ID3D11Texture2D *texture_2d;
 
             texture_2d_desc.Width = load_info->Width;
             texture_2d_desc.Height = load_info->Height;
@@ -830,16 +565,16 @@ HRESULT create_d3d_texture(ID3D10Device *device, D3DX10_IMAGE_LOAD_INFO *load_in
             texture_2d_desc.CPUAccessFlags = load_info->CpuAccessFlags;
             texture_2d_desc.MiscFlags = load_info->MiscFlags;
 
-            if (FAILED(hr = ID3D10Device_CreateTexture2D(device, &texture_2d_desc, resource_data, &texture_2d)))
+            if (FAILED(hr = ID3D11Device_CreateTexture2D(device, &texture_2d_desc, resource_data, &texture_2d)))
                 return hr;
-            *texture = (ID3D10Resource *)texture_2d;
+            *texture = (ID3D11Resource *)texture_2d;
             break;
         }
 
-        case D3D10_RESOURCE_DIMENSION_TEXTURE3D:
+        case D3D11_RESOURCE_DIMENSION_TEXTURE3D:
         {
-            D3D10_TEXTURE3D_DESC texture_3d_desc = { 0 };
-            ID3D10Texture3D *texture_3d;
+            D3D11_TEXTURE3D_DESC texture_3d_desc = { 0 };
+            ID3D11Texture3D *texture_3d;
 
             texture_3d_desc.Width = load_info->Width;
             texture_3d_desc.Height = load_info->Height;
@@ -851,9 +586,9 @@ HRESULT create_d3d_texture(ID3D10Device *device, D3DX10_IMAGE_LOAD_INFO *load_in
             texture_3d_desc.CPUAccessFlags = load_info->CpuAccessFlags;
             texture_3d_desc.MiscFlags = load_info->MiscFlags;
 
-            if (FAILED(hr = ID3D10Device_CreateTexture3D(device, &texture_3d_desc, resource_data, &texture_3d)))
+            if (FAILED(hr = ID3D11Device_CreateTexture3D(device, &texture_3d_desc, resource_data, &texture_3d)))
                 return hr;
-            *texture = (ID3D10Resource *)texture_3d;
+            *texture = (ID3D11Resource *)texture_3d;
             break;
         }
 
@@ -865,40 +600,213 @@ HRESULT create_d3d_texture(ID3D10Device *device, D3DX10_IMAGE_LOAD_INFO *load_in
     return S_OK;
 }
 
-HRESULT WINAPI D3DX10CreateTextureFromMemory(ID3D10Device *device, const void *src_data, SIZE_T src_data_size,
-        D3DX10_IMAGE_LOAD_INFO *load_info, ID3DX10ThreadPump *pump, ID3D10Resource **texture, HRESULT *hresult)
+static HRESULT create_texture(ID3D11Device *device, const void *data, SIZE_T size,
+        D3DX11_IMAGE_LOAD_INFO *load_info, ID3D11Resource **texture)
 {
+    D3D11_SUBRESOURCE_DATA *resource_data;
+    D3DX11_IMAGE_LOAD_INFO load_info_copy;
+    D3DX11_IMAGE_INFO img_info;
     HRESULT hr;
 
-    TRACE("device %p, src_data %p, src_data_size %Iu, load_info %p, pump %p, texture %p, hresult %p.\n",
-            device, src_data, src_data_size, load_info, pump, texture, hresult);
+    init_load_info(load_info, &load_info_copy);
+    if (!load_info_copy.pSrcInfo)
+        load_info_copy.pSrcInfo = &img_info;
+
+    if (FAILED((hr = load_texture_data(data, size, &load_info_copy, &resource_data))))
+        return hr;
+    hr = create_d3d_texture(device, &load_info_copy, resource_data, texture);
+    free(resource_data);
+    return hr;
+}
+
+HRESULT WINAPI D3DX11CreateTextureFromFileA(ID3D11Device *device, const char *src_file,
+        D3DX11_IMAGE_LOAD_INFO *load_info, ID3DX11ThreadPump *pump, ID3D11Resource **texture, HRESULT *hresult)
+{
+    WCHAR *buffer;
+    int str_len;
+    HRESULT hr;
+
+    TRACE("device %p, src_file %s, load_info %p, pump %p, texture %p, hresult %p.\n",
+            device, debugstr_a(src_file), load_info, pump, texture, hresult);
 
     if (!device)
         return E_INVALIDARG;
-    if (!src_data)
+    if (!src_file)
+        return E_FAIL;
+
+    if (!(str_len = MultiByteToWideChar(CP_ACP, 0, src_file, -1, NULL, 0)))
+        return HRESULT_FROM_WIN32(GetLastError());
+
+    if (!(buffer = malloc(str_len * sizeof(*buffer))))
+        return E_OUTOFMEMORY;
+
+    MultiByteToWideChar(CP_ACP, 0, src_file, -1, buffer, str_len);
+    hr = D3DX11CreateTextureFromFileW(device, buffer, load_info, pump, texture, hresult);
+
+    free(buffer);
+
+    return hr;
+}
+
+HRESULT WINAPI D3DX11CreateTextureFromFileW(ID3D11Device *device, const WCHAR *src_file,
+        D3DX11_IMAGE_LOAD_INFO *load_info, ID3DX11ThreadPump *pump, ID3D11Resource **texture, HRESULT *hresult)
+{
+    void *buffer = NULL;
+    DWORD size = 0;
+    HRESULT hr;
+
+    TRACE("device %p, src_file %s, load_info %p, pump %p, texture %p, hresult %p.\n",
+            device, debugstr_w(src_file), load_info, pump, texture, hresult);
+
+    if (!device)
+        return E_INVALIDARG;
+    if (!src_file)
         return E_FAIL;
 
     if (pump)
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
+
+    if (SUCCEEDED((hr = load_file(src_file, &buffer, &size))))
     {
-        ID3DX10DataProcessor *processor;
-        ID3DX10DataLoader *loader;
-
-        if (FAILED((hr = D3DX10CreateAsyncMemoryLoader(src_data, src_data_size, &loader))))
-            return hr;
-        if (FAILED((hr = D3DX10CreateAsyncTextureProcessor(device, load_info, &processor))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            return hr;
-        }
-        if (FAILED((hr = ID3DX10ThreadPump_AddWorkItem(pump, loader, processor, hresult, (void **)texture))))
-        {
-            ID3DX10DataLoader_Destroy(loader);
-            ID3DX10DataProcessor_Destroy(processor);
-        }
-        return hr;
+        hr = create_texture(device, buffer, size, load_info, texture);
+        free(buffer);
     }
+    if (hresult)
+        *hresult = hr;
+    return hr;
+}
 
-    hr = create_texture(device, src_data, src_data_size, load_info, texture);
+HRESULT WINAPI D3DX11CreateTextureFromResourceA(ID3D11Device *device, HMODULE module, const char *resource,
+        D3DX11_IMAGE_LOAD_INFO *load_info, ID3DX11ThreadPump *pump, ID3D11Resource **texture, HRESULT *hresult)
+{
+    uint32_t size;
+    void *buffer;
+    HRESULT hr;
+
+    TRACE("device %p, module %p, resource %s, load_info %p, pump %p, texture %p, hresult %p.\n",
+            device, module, debugstr_a(resource), load_info, pump, texture, hresult);
+
+    if (!device)
+        return E_INVALIDARG;
+
+    if (pump)
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
+
+    if (FAILED((hr = d3dx_load_resource_a(module, resource, &buffer, &size))))
+        return hr;
+    hr = create_texture(device, buffer, size, load_info, texture);
+    if (hresult)
+        *hresult = hr;
+    return hr;
+}
+
+HRESULT WINAPI D3DX11CreateTextureFromResourceW(ID3D11Device *device, HMODULE module, const WCHAR *resource,
+        D3DX11_IMAGE_LOAD_INFO *load_info, ID3DX11ThreadPump *pump, ID3D11Resource **texture, HRESULT *hresult)
+{
+    uint32_t size;
+    void *buffer;
+    HRESULT hr;
+
+    TRACE("device %p, module %p, resource %s, load_info %p, pump %p, texture %p, hresult %p.\n",
+            device, module, debugstr_w(resource), load_info, pump, texture, hresult);
+
+    if (!device)
+        return E_INVALIDARG;
+
+    if (pump)
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
+
+    if (FAILED((hr = d3dx_load_resource_w(module, resource, &buffer, &size))))
+        return hr;
+    hr = create_texture(device, buffer, size, load_info, texture);
+    if (hresult)
+        *hresult = hr;
+    return hr;
+}
+
+HRESULT WINAPI D3DX11CreateShaderResourceViewFromMemory(ID3D11Device *device, const void *data,
+        SIZE_T data_size, D3DX11_IMAGE_LOAD_INFO *load_info, ID3DX11ThreadPump *pump,
+        ID3D11ShaderResourceView **view, HRESULT *hresult)
+{
+    FIXME("device %p, data %p, data_size %Iu, load_info %p, pump %p, view %p, hresult %p stub!\n",
+            device, data, data_size, load_info, pump, view, hresult);
+
+    return E_NOTIMPL;
+}
+
+HRESULT WINAPI D3DX11CreateTextureFromMemory(ID3D11Device *device, const void *data,
+        SIZE_T data_size, D3DX11_IMAGE_LOAD_INFO *load_info, ID3DX11ThreadPump *pump,
+        ID3D11Resource **texture, HRESULT *hresult)
+{
+    HRESULT hr;
+
+    TRACE("device %p, data %p, data_size %Iu, load_info %p, pump %p, texture %p, hresult %p.\n",
+            device, data, data_size, load_info, pump, texture, hresult);
+
+    if (!device)
+        return E_INVALIDARG;
+    if (!data)
+        return E_FAIL;
+
+    if (pump)
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
+
+    hr = create_texture(device, data, data_size, load_info, texture);
+    if (hresult)
+        *hresult = hr;
+    return hr;
+}
+
+HRESULT WINAPI D3DX11SaveTextureToFileW(ID3D11DeviceContext *context, ID3D11Resource *texture,
+        D3DX11_IMAGE_FILE_FORMAT format, const WCHAR *filename)
+{
+    FIXME("context %p, texture %p, format %u, filename %s stub!\n",
+            context, texture, format, debugstr_w(filename));
+
+    return E_NOTIMPL;
+}
+
+HRESULT WINAPI D3DX11SaveTextureToFileA(ID3D11DeviceContext *context, ID3D11Resource *texture,
+        D3DX11_IMAGE_FILE_FORMAT format, const char *filename)
+{
+    FIXME("context %p, texture %p, format %u, filename %s stub!\n",
+            context, texture, format, debugstr_a(filename));
+
+    return E_NOTIMPL;
+}
+
+HRESULT WINAPI D3DX11SaveTextureToMemory(ID3D11DeviceContext *context, ID3D11Resource *texture,
+        D3DX11_IMAGE_FILE_FORMAT format, ID3D10Blob **buffer, UINT flags)
+{
+    FIXME("context %p, texture %p, format %u, buffer %p, flags %#x stub!\n",
+            context, texture, format, buffer, flags);
+
+    return E_NOTIMPL;
+}
+
+HRESULT WINAPI D3DX11LoadTextureFromTexture(ID3D11DeviceContext *context, ID3D11Resource *src_texture,
+        D3DX11_TEXTURE_LOAD_INFO *info, ID3D11Resource *dst_texture)
+{
+    FIXME("context %p, src_texture %p, info %p, dst_texture %p stub!\n",
+            context, src_texture, info, dst_texture);
+
+    return E_NOTIMPL;
+}
+
+HRESULT WINAPI D3DX11GetImageInfoFromMemory(const void *src_data, SIZE_T src_data_size, ID3DX11ThreadPump *pump,
+        D3DX11_IMAGE_INFO *img_info, HRESULT *hresult)
+{
+    HRESULT hr;
+
+    TRACE("src_data %p, src_data_size %Iu, pump %p, img_info %p, hresult %p.\n", src_data, src_data_size, pump,
+            img_info, hresult);
+
+    if (!src_data)
+        return E_FAIL;
+    if (pump)
+        FIXME("D3DX11 thread pump is currently unimplemented.\n");
+
+    hr = get_image_info(src_data, src_data_size, img_info);
     if (hresult)
         *hresult = hr;
     return hr;
