@@ -458,6 +458,16 @@ HRESULT d3dx_create_subresource_data_for_texture(uint32_t width, uint32_t height
         uint32_t mip_levels, uint32_t layer_count, const struct pixel_format_desc *fmt_desc,
         struct d3dx_subresource_data **out_sub_rsrc_data);
 
+/*
+ * File/resource loading helper functions.
+ */
+HRESULT d3dx_load_file(const WCHAR *path, void **data, uint32_t *size);
+HRESULT d3dx_load_resource_init_a(HMODULE module, const char *resource, HRSRC *rsrc);
+HRESULT d3dx_load_resource_init_w(HMODULE module, const WCHAR *resource, HRSRC *rsrc);
+HRESULT d3dx_load_resource(HMODULE module, HRSRC rsrc, void **data, uint32_t *size);
+HRESULT d3dx_load_resource_a(HMODULE module, const char *resource, void **data, uint32_t *size);
+HRESULT d3dx_load_resource_w(HMODULE module, const WCHAR *resource, void **data, uint32_t *size);
+
 /* debug helpers */
 const char *debug_d3dx_image_file_format(enum d3dx_image_file_format format);
 #endif /* __WINE_D3DX_HELPERS_H */
