@@ -698,7 +698,7 @@ static void elf_hash_symtab(struct module* module, struct pool* pool,
         {
         case ELF_STT_FILE:
             if (symname)
-                compiland = symt_new_compiland(module, symname);
+                compiland = symt_new_compiland(module, symt_ptr_to_symref(&module->top->symt), symname);
             else
                 compiland = NULL;
             continue;
@@ -946,7 +946,7 @@ static int elf_new_wine_thunks(struct module* module, const struct hash_table* h
             ULONG64     ref_addr;
             struct location loc;
 
-            symt = symt_find_nearest(module, addr);
+            symt = (struct symt_ht*)SYMT_SYMREF_TO_PTR(symt_find_nearest(module, addr));
             if (symt && !symt_get_address(&symt->symt, &ref_addr))
                 ref_addr = addr;
             if (!symt || addr != ref_addr)
