@@ -29,6 +29,7 @@
 #include "dbghelp_private.h"
 #include "image_private.h"
 #include "psapi.h"
+#include "tlhelp32.h"
 #include "winternl.h"
 #include "wine/debug.h"
 
@@ -1032,20 +1033,12 @@ BOOL module_remove(struct process* pcs, struct module* module)
     /* remove local scope if symbol is from this module */
     if (pcs->localscope_symt)
     {
-        struct symt* locsym = pcs->localscope_symt;
-        if (symt_check_tag(locsym, SymTagInlineSite))
-            locsym = &symt_get_function_from_inlined((struct symt_function*)locsym)->symt;
-        if (symt_check_tag(locsym, SymTagFunction))
+        struct module_pair pair;
+        if (module_init_pair(&pair, pcs->handle, pcs->localscope_pc) &&
+            (module == pair.effective || module == pair.requested))
         {
-            struct symt_compiland *compiland = (struct symt_compiland*)SYMT_SYMREF_TO_PTR(((struct symt_function*)locsym)->container);
-            if (symt_check_tag(&compiland->symt, SymTagCompiland))
-            {
-                if (module == ((struct symt_module*)SYMT_SYMREF_TO_PTR(compiland->container))->module)
-                {
-                    pcs->localscope_pc = 0;
-                    pcs->localscope_symt = NULL;
-                }
-            }
+            pcs->localscope_pc = 0;
+            pcs->localscope_symt = NULL;
         }
     }
     while (module_format_vtable_iterator_next(module, &iter, MODULE_FORMAT_VTABLE_INDEX(remove)))
@@ -1254,23 +1247,6 @@ BOOL  WINAPI EnumerateLoadedModules(HANDLE hProcess,
 }
 #endif
 
-static unsigned int load_and_grow_modules(HANDLE process, HMODULE** hmods, unsigned start, unsigned* alloc, DWORD filter)
-{
-    DWORD needed;
-    BOOL ret;
-
-    while ((ret = EnumProcessModulesEx(process, *hmods + start, (*alloc - start) * sizeof(HMODULE),
-                                       &needed, filter)) &&
-           needed > (*alloc - start) * sizeof(HMODULE))
-    {
-        HMODULE* new = HeapReAlloc(GetProcessHeap(), 0, *hmods, (*alloc) * 2 * sizeof(HMODULE));
-        if (!new) return 0;
-        *hmods = new;
-        *alloc *= 2;
-    }
-    return ret ? needed / sizeof(HMODULE) : 0;
-}
-
 /******************************************************************
  *		EnumerateLoadedModulesW64 (DBGHELP.@)
  *
@@ -1280,15 +1256,16 @@ BOOL  WINAPI EnumerateLoadedModulesW64(HANDLE process,
                                        PVOID user)
 {
     OBJECT_BASIC_INFORMATION obi;
-    HMODULE*            hmods;
-    unsigned            alloc = 256, count, count32, i;
-    USHORT              pcs_machine, native_machine;
-    BOOL                with_32bit_modules;
-    WCHAR               imagenameW[MAX_PATH];
-    MODULEINFO          mi;
-    WCHAR*              sysdir = NULL;
-    WCHAR*              wowdir = NULL;
-    size_t              sysdir_len = 0, wowdir_len = 0;
+    DWORD               snapshot_flags;
+    DWORD               pid = GetProcessId( process );
+    HANDLE              snapshot;
+    MODULEENTRY32W      me;
+
+    if (!pid)
+    {
+        SetLastError(STATUS_INVALID_CID);
+        return FALSE;
+    }
 
     if (process != GetCurrentProcess() &&
         RtlIsCurrentProcess( process ) &&
@@ -1296,90 +1273,28 @@ BOOL  WINAPI EnumerateLoadedModulesW64(HANDLE process,
         obi.GrantedAccess & PROCESS_VM_READ)
     {
         TRACE("same process.\n");
-        process = GetCurrentProcess();
+        pid = 0;
     }
 
-    /* process might not be a handle to a live process */
-    if (!IsWow64Process2(process, &pcs_machine, &native_machine))
-    {
-        SetLastError(STATUS_INVALID_CID);
-        return FALSE;
-    }
-    with_32bit_modules = sizeof(void*) > sizeof(int) &&
-        pcs_machine != IMAGE_FILE_MACHINE_UNKNOWN &&
-        (dbghelp_options & SYMOPT_INCLUDE_32BIT_MODULES);
+    snapshot_flags = TH32CS_SNAPMODULE;
+    if (dbghelp_options & SYMOPT_INCLUDE_32BIT_MODULES)
+        snapshot_flags |= TH32CS_SNAPMODULE32;
 
-    if (!(hmods = HeapAlloc(GetProcessHeap(), 0, alloc * sizeof(hmods[0]))))
+    snapshot = CreateToolhelp32Snapshot( snapshot_flags, pid );
+    if (snapshot == INVALID_HANDLE_VALUE)
         return FALSE;
 
-    /* Note:
-     * - we report modules returned from kernelbase.EnumProcessModulesEx
-     * - appending 32bit modules when possible and requested
-     *
-     * When considering 32bit modules in a wow64 child process, required from
-     * a 64bit process:
-     * - native returns from kernelbase.EnumProcessModulesEx
-     *   redirected paths (that is in system32 directory), while
-     *   dbghelp.EnumerateLoadedModulesWine returns the effective path
-     *   (eg. syswow64 for x86_64).
-     * - (Except for the main module, if gotten from syswow64, where kernelbase
-     *    will return the effective path)
-     * - Wine kernelbase (and ntdll) incorrectly return these modules from
-     *   syswow64 (except for ntdll which is returned from system32).
-     * => for these modules, always perform a system32 => syswow64 path
-     *    conversion (it'll work even if ntdll/kernelbase is fixed).
-     */
-    if ((count = load_and_grow_modules(process, &hmods, 0, &alloc, LIST_MODULES_DEFAULT)) && with_32bit_modules)
+    me.dwSize = sizeof(me);
+    if (Module32FirstW( snapshot, &me ))
     {
-        /* append 32bit modules when required */
-        if ((count32 = load_and_grow_modules(process, &hmods, count, &alloc, LIST_MODULES_32BIT)))
+        do
         {
-            sysdir_len = GetSystemDirectoryW(NULL, 0);
-            wowdir_len = GetSystemWow64Directory2W(NULL, 0, pcs_machine);
-
-            if (!sysdir_len || !wowdir_len ||
-                !(sysdir = HeapAlloc(GetProcessHeap(), 0, (sysdir_len + 1 + wowdir_len + 1) * sizeof(WCHAR))))
-            {
-                HeapFree(GetProcessHeap(), 0, hmods);
-                return FALSE;
-            }
-            wowdir = sysdir + sysdir_len + 1;
-            if (GetSystemDirectoryW(sysdir, sysdir_len) >= sysdir_len)
-                FIXME("shouldn't happen\n");
-            if (GetSystemWow64Directory2W(wowdir, wowdir_len, pcs_machine) >= wowdir_len)
-                FIXME("shouldn't happen\n");
-            wcscat(sysdir, L"\\");
-            wcscat(wowdir, L"\\");
-        }
-    }
-    else count32 = 0;
-
-    for (i = 0; i < count + count32; i++)
-    {
-        if (GetModuleInformation(process, hmods[i], &mi, sizeof(mi)) &&
-            GetModuleFileNameExW(process, hmods[i], imagenameW, ARRAY_SIZE(imagenameW)))
-        {
-            /* rewrite path in system32 into syswow64 for 32bit modules */
-            if (i >= count)
-            {
-                size_t len = wcslen(imagenameW);
-
-                if (!wcsnicmp(imagenameW, sysdir, sysdir_len) &&
-                    (len - sysdir_len + wowdir_len) + 1 <= ARRAY_SIZE(imagenameW))
-                {
-                    memmove(&imagenameW[wowdir_len], &imagenameW[sysdir_len], (len - sysdir_len) * sizeof(WCHAR));
-                    memcpy(imagenameW, wowdir, wowdir_len * sizeof(WCHAR));
-                }
-            }
-            if (!enum_cb(imagenameW, (DWORD_PTR)mi.lpBaseOfDll, mi.SizeOfImage, user))
-                break;
-        }
+            if (!enum_cb( me.szExePath, (DWORD_PTR)me.modBaseAddr, me.modBaseSize, user )) break;
+        } while (Module32NextW( snapshot, &me ));
     }
 
-    HeapFree(GetProcessHeap(), 0, hmods);
-    HeapFree(GetProcessHeap(), 0, sysdir);
-
-    return count != 0;
+    CloseHandle( snapshot );
+    return TRUE;
 }
 
 static void dbghelp_str_WtoA(const WCHAR *src, char *dst, int dst_len)
