@@ -141,9 +141,14 @@ BOOL symt_get_address(const struct symt* type, ULONG64* addr)
         case DataIsGlobal:
         case DataIsFileStatic:
         case DataIsStaticLocal:
-            *addr = ((const struct symt_data*)type)->u.var.offset;
-            break;
-        default: return FALSE;
+            if (((const struct symt_data*)type)->u.var.kind == loc_absolute)
+            {
+                *addr = ((const struct symt_data*)type)->u.var.offset;
+                break;
+            }
+            /* fall through */
+        default:
+            return FALSE;
         }
         break;
     case SymTagBlock:
@@ -1123,6 +1128,30 @@ BOOL symt_get_info(struct module* module, const struct symt* type,
         X(DWORD) = symt_ptr_to_index(module, type);
         break;
 
+    case TI_GET_ADDRESSOFFSET:
+        switch (type->tag)
+        {
+        case SymTagData:
+            switch (((const struct symt_data*)type)->kind)
+            {
+            case DataIsGlobal:
+            case DataIsFileStatic:
+            case DataIsStaticLocal:
+                if (((const struct symt_data*)type)->u.var.kind == loc_tlsrel)
+                {
+                    X(DWORD) = ((const struct symt_data*)type)->u.var.offset;
+                    break;
+                }
+                /* fall through */
+            default:
+                return FALSE;
+            }
+            break;
+        default:
+            return FALSE;
+        }
+        break;
+
         /* FIXME: we don't support properly C++ for now */
     case TI_GET_VIRTUALBASECLASS:
     case TI_GET_VIRTUALTABLESHAPEID:
