@@ -20,7 +20,6 @@
  *
  */
 
-#include "config.h"
 
 #include <stdarg.h>
 
@@ -36,6 +35,10 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(dpnet);
 
+static PFNDPNMESSAGEHANDLER threadpool_msghandler = NULL;
+static DWORD threadpool_flags       = 0;
+static void *threadpool_usercontext = NULL;
+
 static inline IDirectPlay8ThreadPoolImpl *impl_from_IDirectPlay8ThreadPool(IDirectPlay8ThreadPool *iface)
 {
     return CONTAINING_RECORD(iface, IDirectPlay8ThreadPoolImpl, IDirectPlay8ThreadPool_iface);
@@ -43,19 +46,16 @@ static inline IDirectPlay8ThreadPoolImpl *impl_from_IDirectPlay8ThreadPool(IDire
 
 /* IUnknown interface follows */
 static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_QueryInterface(IDirectPlay8ThreadPool *iface,
-        REFIID riid, void **ppobj)
+        REFIID riid, void **ret_iface)
 {
-    IDirectPlay8ThreadPoolImpl *This = impl_from_IDirectPlay8ThreadPool(iface);
-
-    if(IsEqualGUID(riid, &IID_IUnknown) ||
-       IsEqualGUID(riid, &IID_IDirectPlay8ThreadPool))
+    if(IsEqualGUID(riid, &IID_IUnknown) || IsEqualGUID(riid, &IID_IDirectPlay8ThreadPool))
     {
-        IUnknown_AddRef(iface);
-        *ppobj = This;
-        return DPN_OK;
+        IDirectPlay8ThreadPool_AddRef(iface);
+        *ret_iface = iface;
+        return S_OK;
     }
 
-    WARN("(%p)->(%s,%p): not found\n", This, debugstr_guid(riid), ppobj);
+    WARN("(%p)->(%s,%p): not found\n", iface, debugstr_guid(riid), ret_iface);
     return E_NOINTERFACE;
 }
 
@@ -84,17 +84,17 @@ static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_Initialize(IDirectPlay8ThreadPo
 {
     IDirectPlay8ThreadPoolImpl *This = impl_from_IDirectPlay8ThreadPool(iface);
 
-    TRACE("(%p)->(%p,%p,%x)\n", This, pvUserContext, pfn, dwFlags);
+    TRACE("(%p)->(%p,%p,%lx)\n", This, pvUserContext, pfn, dwFlags);
 
     if(!pfn)
         return DPNERR_INVALIDPARAM;
 
-    if(This->msghandler)
+    if(threadpool_msghandler)
         return DPNERR_ALREADYINITIALIZED;
 
-    This->msghandler  = pfn;
-    This->flags       = dwFlags;
-    This->usercontext = pvUserContext;
+    threadpool_msghandler  = pfn;
+    threadpool_flags       = dwFlags;
+    threadpool_usercontext = pvUserContext;
 
     return DPN_OK;
 }
@@ -104,9 +104,12 @@ static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_Close(IDirectPlay8ThreadPool *i
 {
     IDirectPlay8ThreadPoolImpl *This = impl_from_IDirectPlay8ThreadPool(iface);
 
-    FIXME("(%p)->(%x)\n", This, dwFlags);
+    FIXME("(%p)->(%lx)\n", This, dwFlags);
 
-    This->msghandler = NULL;
+    if(!threadpool_msghandler)
+        return DPNERR_UNINITIALIZED;
+
+    threadpool_msghandler = NULL;
 
     return DPN_OK;
 }
@@ -114,7 +117,7 @@ static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_Close(IDirectPlay8ThreadPool *i
 static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_GetThreadCount(IDirectPlay8ThreadPool *iface,
         const DWORD dwProcessorNum, DWORD * const pdwNumThreads, const DWORD dwFlags)
 {
-    FIXME("(%p)->(%x,%p,%x): stub\n", iface, dwProcessorNum, pdwNumThreads, dwFlags);
+    FIXME("(%p)->(%lx,%p,%lx): stub\n", iface, dwProcessorNum, pdwNumThreads, dwFlags);
     *pdwNumThreads = 0;
     return DPN_OK;
 }
@@ -122,7 +125,7 @@ static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_GetThreadCount(IDirectPlay8Thre
 static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_SetThreadCount(IDirectPlay8ThreadPool *iface,
         const DWORD dwProcessorNum, const DWORD dwNumThreads, const DWORD dwFlags)
 {
-    FIXME("(%p)->(%x,%x,%x): stub\n", iface, dwProcessorNum, dwNumThreads, dwFlags);
+    FIXME("(%p)->(%lx,%lx,%lx): stub\n", iface, dwProcessorNum, dwNumThreads, dwFlags);
     return DPN_OK;
 }
 
@@ -132,7 +135,7 @@ static HRESULT WINAPI IDirectPlay8ThreadPoolImpl_DoWork(IDirectPlay8ThreadPool *
     static BOOL Run = FALSE;
 
     if(!Run)
-        FIXME("(%p)->(%x,%x): stub\n", iface, dwAllowedTimeSlice, dwFlags);
+        FIXME("(%p)->(%lx,%lx): stub\n", iface, dwAllowedTimeSlice, dwFlags);
 
     Run = TRUE;
 
