@@ -198,10 +198,14 @@ static HRESULT DSPROPERTY_DescriptionW(
     IPropertyStore *ps;
     PROPVARIANT pv;
     HRESULT hr;
+    WCHAR *id;
 
     TRACE("pPropData=%p,cbPropData=%ld,pcbReturned=%p)\n",
           pPropData,cbPropData,pcbReturned);
 
+    if (cbPropData < sizeof(*ppd))
+        return E_INVALIDARG;
+
     TRACE("DeviceId=%s\n",debugstr_guid(&ppd->DeviceId));
     if ( IsEqualGUID( &ppd->DeviceId , &GUID_NULL) ) {
         /* default device of type specified by ppd->DataFlow */
@@ -245,9 +249,11 @@ static HRESULT DSPROPERTY_DescriptionW(
     }
 
     ppd->Description = wcsdup(pv.pwszVal);
-    ppd->Module = wcsdup(wine_vxd_drv);
+    hr = IMMDevice_GetId(mmdevice, &id);
+    ppd->Module = wcsdup(id);
+    CoTaskMemFree(id);
     ppd->Interface = wcsdup(wInterface);
-    ppd->Type = DIRECTSOUNDDEVICE_TYPE_VXD;
+    ppd->Type = DIRECTSOUNDDEVICE_TYPE_WDM;
 
     PropVariantClear(&pv);
     IPropertyStore_Release(ps);
@@ -261,11 +267,16 @@ static HRESULT DSPROPERTY_DescriptionW(
     return S_OK;
 }
 
-static
-BOOL CALLBACK enum_callback(GUID *guid, const WCHAR *desc, const WCHAR *module,
-        void *user)
+struct enum_callback_ctx
 {
-    PDSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_W_DATA ppd = user;
+    DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_W_DATA *pd;
+    EDataFlow flow;
+    unsigned int id;
+};
+
+static BOOL CALLBACK enum_callback(GUID *guid, const WCHAR *desc, const WCHAR *module, void *user)
+{
+    struct enum_callback_ctx *ctx = user;
     DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTION_W_DATA data;
     DWORD len;
     BOOL ret;
@@ -276,19 +287,19 @@ BOOL CALLBACK enum_callback(GUID *guid, const WCHAR *desc, const WCHAR *module,
     if(!guid)
         return TRUE;
 
+    data.Type = DIRECTSOUNDDEVICE_TYPE_WDM;
+    data.DataFlow = ctx->flow == eRender ? DIRECTSOUNDDEVICE_DATAFLOW_RENDER : DIRECTSOUNDDEVICE_DATAFLOW_CAPTURE;
     data.DeviceId = *guid;
-
-    len = lstrlenW(module) + 1;
-    data.Module = malloc(len * sizeof(WCHAR));
-    memcpy(data.Module, module, len * sizeof(WCHAR));
-
     len = lstrlenW(desc) + 1;
     data.Description = malloc(len * sizeof(WCHAR));
     memcpy(data.Description, desc, len * sizeof(WCHAR));
-
+    len = lstrlenW(module) + 1;
+    data.Module = malloc(len * sizeof(WCHAR));
+    memcpy(data.Module, module, len * sizeof(WCHAR));
     data.Interface = wInterface;
+    data.WaveDeviceId = ctx->id++;
 
-    ret = ppd->Callback(&data, ppd->Context);
+    ret = ctx->pd->Callback(&data, ctx->pd->Context);
 
     free(data.Module);
     free(data.Description);
@@ -296,32 +307,34 @@ BOOL CALLBACK enum_callback(GUID *guid, const WCHAR *desc, const WCHAR *module,
     return ret;
 }
 
-static HRESULT DSPROPERTY_EnumerateW(
-    LPVOID pPropData,
-    ULONG cbPropData,
-    PULONG pcbReturned )
+static HRESULT DSPROPERTY_EnumerateW(LPVOID pd, ULONG size, PULONG out_size)
 {
-    PDSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_W_DATA ppd = pPropData;
+    struct enum_callback_ctx ctx = {pd};
     HRESULT hr;
 
-    TRACE("(pPropData=%p,cbPropData=%ld,pcbReturned=%p)\n",
-          pPropData,cbPropData,pcbReturned);
+    TRACE("pd %p, size %lu, out_size %p\n", pd, size, out_size);
 
-    if (pcbReturned)
-        *pcbReturned = 0;
+    if (out_size)
+        *out_size = sizeof(*ctx.pd);
 
-    if (!ppd || !ppd->Callback)
+    if (!pd || !ctx.pd->Callback)
     {
-        WARN("Invalid ppd %p\n", ppd);
+        WARN("Invalid pd %p\n", pd);
         return E_PROP_ID_UNSUPPORTED;
     }
 
-    hr = enumerate_mmdevices(eRender, DSOUND_renderer_guids,
-            enum_callback, ppd);
+    if (size < sizeof(ctx.pd))
+        return E_INVALIDARG;
 
-    if(hr == S_OK)
-        hr = enumerate_mmdevices(eCapture, DSOUND_capture_guids,
-                enum_callback, ppd);
+    ctx.flow = eRender;
+    hr = enumerate_mmdevices(eRender, DSOUND_renderer_guids, enum_callback, &ctx);
+
+    if (hr == S_OK)
+    {
+        ctx.flow = eCapture;
+        ctx.id = 0;
+        hr = enumerate_mmdevices(eCapture, DSOUND_capture_guids, enum_callback, &ctx);
+    }
 
     return SUCCEEDED(hr) ? DS_OK : hr;
 }
@@ -375,6 +388,8 @@ static BOOL CALLBACK DSPROPERTY_enumWtoA(DSPROPERTY_DIRECTSOUNDDEVICE_DESCRIPTIO
     DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_A_DATA *ppd = data;
     BOOL ret;
 
+    TRACE("descW %p, data %p\n", descW, data);
+
     ret = DSPROPERTY_descWtoA(descW, &descA);
     if (!ret)
         return FALSE;
@@ -392,6 +407,8 @@ static HRESULT DSPROPERTY_EnumerateA(
     DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_A_DATA *ppd = pPropData;
     DSPROPERTY_DIRECTSOUNDDEVICE_ENUMERATE_W_DATA data;
 
+    TRACE("pPropData %p, cbPropData %lu, pcbReturned %p\n", pPropData, cbPropData, pcbReturned);
+
     if (!ppd || !ppd->Callback)
     {
         WARN("Invalid ppd %p\n", ppd);
