@@ -49,11 +49,8 @@ WINE_DEFAULT_DEBUG_CHANNEL(ieframe);
 
 #define WM_UPDATEADDRBAR    (WM_APP+1)
 
-static const WCHAR szIEWinFrame[] = { 'I','E','F','r','a','m','e',0 };
-
 /* Windows uses "Microsoft Internet Explorer" */
-static const WCHAR wszWineInternetExplorer[] =
-        {'W','i','n','e',' ','I','n','t','e','r','n','e','t',' ','E','x','p','l','o','r','e','r',0};
+static const WCHAR wszWineInternetExplorer[] = L"Wine Internet Explorer";
 
 static LONG obj_cnt;
 static DWORD dde_inst;
@@ -116,7 +113,7 @@ static void free_fav_menu_data(HMENU menu)
     int i;
 
     for(i = 0; (url = get_fav_url_from_id(menu, ID_BROWSE_GOTOFAV_FIRST + i)); i++)
-        heap_free( url );
+        free( url );
 }
 
 static int get_menu_item_count(HMENU menu)
@@ -152,7 +149,7 @@ static void add_fav_to_menu(HMENU favmenu, HMENU menu, LPWSTR title, LPCWSTR url
         return;
     }
 
-    urlbuf = heap_alloc((lstrlenW(url) + 1) * sizeof(WCHAR));
+    urlbuf = malloc((wcslen(url) + 1) * sizeof(WCHAR));
 
     if(!urlbuf)
         return;
@@ -170,7 +167,6 @@ static void add_fav_to_menu(HMENU favmenu, HMENU menu, LPWSTR title, LPCWSTR url
 
 static void add_favs_to_menu(HMENU favmenu, HMENU menu, LPCWSTR dir)
 {
-    static const WCHAR search[] = {'*',0};
     WCHAR path[MAX_PATH*2];
     WCHAR* filename;
     HANDLE findhandle;
@@ -180,7 +176,7 @@ static void add_favs_to_menu(HMENU favmenu, HMENU menu, LPCWSTR dir)
     HRESULT res;
 
     lstrcpyW(path, dir);
-    PathAppendW(path, search);
+    PathAppendW(path, L"*");
 
     findhandle = FindFirstFileW(path, &finddata);
 
@@ -194,7 +190,7 @@ static void add_favs_to_menu(HMENU favmenu, HMENU menu, LPCWSTR dir)
 
     if(SUCCEEDED(res))
     {
-        filename = path + lstrlenW(path) - lstrlenW(search);
+        filename = path + lstrlenW(path) - strlen("*");
 
         do
         {
@@ -202,11 +198,9 @@ static void add_favs_to_menu(HMENU favmenu, HMENU menu, LPCWSTR dir)
 
             if(finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
             {
-                static const WCHAR ignore1[] = {'.','.',0};
-                static const WCHAR ignore2[] = {'.',0};
                 MENUITEMINFOW item;
 
-                if(!lstrcmpW(filename, ignore1) || !lstrcmpW(filename, ignore2))
+                if(!lstrcmpW(filename, L"..") || !lstrcmpW(filename, L"."))
                     continue;
 
                 item.cbSize = sizeof(item);
@@ -217,11 +211,10 @@ static void add_favs_to_menu(HMENU favmenu, HMENU menu, LPCWSTR dir)
                 add_favs_to_menu(favmenu, item.hSubMenu, path);
             } else
             {
-                static const WCHAR urlext[] = {'.','u','r','l',0};
                 WCHAR* fileext;
                 WCHAR* url = NULL;
 
-                if(lstrcmpiW(PathFindExtensionW(filename), urlext))
+                if(lstrcmpiW(PathFindExtensionW(filename), L".url"))
                     continue;
 
                 if(FAILED(IPersistFile_Load(urlfile, path, 0)))
@@ -232,7 +225,7 @@ static void add_favs_to_menu(HMENU favmenu, HMENU menu, LPCWSTR dir)
                 if(!url)
                     continue;
 
-                fileext = filename + lstrlenW(filename) - lstrlenW(urlext);
+                fileext = filename + lstrlenW(filename) - lstrlenW(L".url");
                 *fileext = 0;
                 add_fav_to_menu(favmenu, menu, filename, url);
             }
@@ -250,17 +243,12 @@ static void add_favs_to_menu(HMENU favmenu, HMENU menu, LPCWSTR dir)
 
 static void add_tbs_to_menu(HMENU menu)
 {
-    static const WCHAR toolbar_key[] = {'S','o','f','t','w','a','r','e','\\',
-                                        'M','i','c','r','o','s','o','f','t','\\',
-                                        'I','n','t','e','r','n','e','t',' ',
-                                        'E','x','p','l','o','r','e','r','\\',
-                                        'T','o','o','l','b','a','r',0};
+    static const WCHAR toolbar_key[] = L"Software\\Microsoft\\Internet Explorer\\Toolbar";
     HUSKEY toolbar_handle;
 
     if(SHRegOpenUSKeyW(toolbar_key, KEY_READ, NULL, &toolbar_handle, TRUE) == ERROR_SUCCESS)
     {
-        static const WCHAR classes_key[] = {'S','o','f','t','w','a','r','e','\\',
-                                            'C','l','a','s','s','e','s','\\','C','L','S','I','D',0};
+        static const WCHAR classes_key[] = L"Software\\Classes\\CLSID";
         HUSKEY classes_handle;
         WCHAR guid[39];
         DWORD value_len = ARRAY_SIZE(guid);
@@ -471,7 +459,7 @@ static void create_rebar(InternetExplorer *ie)
     add_tb_button(ie, 4, ID_BROWSE_HOME, IDS_TB_HOME);
     add_tb_separator(ie);
     add_tb_button(ie, 5, ID_BROWSE_PRINT, IDS_TB_PRINT);
-    SendMessageW(ie->toolbar_hwnd, TB_SETBUTTONSIZE, 0, MAKELPARAM(65,50));
+    SendMessageW(ie->toolbar_hwnd, TB_SETBUTTONSIZE, 0, MAKELPARAM(70,50));
     SendMessageW(ie->toolbar_hwnd, TB_GETMAXSIZE, 0, (LPARAM)&toolbar_size);
 
     bandinf.cbSize = sizeof(bandinf);
@@ -713,7 +701,7 @@ void register_iewindow_class(void)
                             GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_SHARED);
     wc.hCursor = LoadCursorW(0, (LPWSTR)IDC_ARROW);
     wc.hbrBackground = 0;
-    wc.lpszClassName = szIEWinFrame;
+    wc.lpszClassName = L"IEFrame";
     wc.lpszMenuName = NULL;
 
     RegisterClassExW(&wc);
@@ -721,14 +709,14 @@ void register_iewindow_class(void)
 
 void unregister_iewindow_class(void)
 {
-    UnregisterClassW(szIEWinFrame, ieframe_instance);
+    UnregisterClassW(L"IEFrame", ieframe_instance);
 }
 
 static void create_frame_hwnd(InternetExplorer *This)
 {
     CreateWindowExW(
             WS_EX_WINDOWEDGE,
-            szIEWinFrame, wszWineInternetExplorer,
+            L"IEFrame", wszWineInternetExplorer,
             WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME
                 | WS_MINIMIZEBOX | WS_MAXIMIZEBOX,
             CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
@@ -783,6 +771,10 @@ static void DocHostContainer_on_command_state_change(DocHost *iface, LONG comman
 static void DocHostContainer_set_url(DocHost* iface, const WCHAR *url)
 {
     InternetExplorer *This = impl_from_DocHost(iface);
+    const WCHAR *orig_url = error_url_frag(url);
+
+    if(orig_url)
+        url = orig_url;
 
     This->nohome = FALSE;
     SendMessageW(This->frame_hwnd, WM_UPDATEADDRBAR, 0, (LPARAM)url);
@@ -801,7 +793,7 @@ static HRESULT create_ie(InternetExplorer **ret_obj)
 {
     InternetExplorer *ret;
 
-    ret = heap_alloc_zero(sizeof(InternetExplorer));
+    ret = calloc(1, sizeof(InternetExplorer));
     if(!ret)
         return E_OUTOFMEMORY;
 
@@ -888,7 +880,7 @@ static ULONG WINAPI InternetExplorerManager_Release(IInternetExplorerManager *if
 
     if (ref == 0)
     {
-        HeapFree(GetProcessHeap(), 0, This);
+        free(This);
         released_obj();
     }
 
@@ -917,7 +909,7 @@ HRESULT WINAPI InternetExplorerManager_Create(IClassFactory *iface, IUnknown *pO
 
     TRACE("(%p %s %p)\n", pOuter, debugstr_guid(riid), ppv);
 
-    if (!(ret = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*ret))))
+    if (!(ret = calloc(1, sizeof(*ret))))
         return E_OUTOFMEMORY;
 
     ret->IInternetExplorerManager_iface.lpVtbl = &InternetExplorerManager_vtbl;
@@ -1027,10 +1019,12 @@ static HDDEDATA open_dde_url(WCHAR *dde_url)
     }
 
     hres = IWebBrowser2_Navigate2(&ie->IWebBrowser2_iface, &urlv, NULL, NULL, NULL, NULL);
+
+    SysFreeString(V_BSTR(&urlv));
+    IWebBrowser2_Release(&ie->IWebBrowser2_iface);
+
     if(FAILED(hres))
         return 0;
-
-    IWebBrowser2_Release(&ie->IWebBrowser2_iface);
     return ULongToHandle(DDE_FACK);
 }
 
@@ -1055,19 +1049,19 @@ static HDDEDATA WINAPI dde_proc(UINT type, UINT uFmt, HCONV hConv, HSZ hsz1, HSZ
             break;
         }
 
-        url = heap_alloc(size);
+        url = malloc(size);
         if(!url)
             break;
 
         if(DdeGetData(data, (BYTE*)url, size, 0) != size) {
             ERR("error during read\n");
-            heap_free(url);
+            free(url);
             break;
         }
 
         ret = open_dde_url(url);
 
-        heap_free(url);
+        free(url);
         return ret;
     }
 
@@ -1086,20 +1080,17 @@ static void init_dde(void)
 {
     UINT res;
 
-    static const WCHAR iexploreW[] = {'I','E','x','p','l','o','r','e',0};
-    static const WCHAR openurlW[] = {'W','W','W','_','O','p','e','n','U','R','L',0};
-
     res = DdeInitializeW(&dde_inst, dde_proc, CBF_SKIP_ALLNOTIFICATIONS | CBF_FAIL_ADVISES | CBF_FAIL_POKES, 0);
     if(res != DMLERR_NO_ERROR) {
         WARN("DdeInitialize failed: %u\n", res);
         return;
     }
 
-    ddestr_iexplore = DdeCreateStringHandleW(dde_inst, iexploreW, CP_WINUNICODE);
+    ddestr_iexplore = DdeCreateStringHandleW(dde_inst, L"IExplore", CP_WINUNICODE);
     if(!ddestr_iexplore)
         WARN("Failed to create string handle: %u\n", DdeGetLastError(dde_inst));
 
-    ddestr_openurl = DdeCreateStringHandleW(dde_inst, openurlW, CP_WINUNICODE);
+    ddestr_openurl = DdeCreateStringHandleW(dde_inst, L"WWW_OpenURL", CP_WINUNICODE);
     if(!ddestr_openurl)
         WARN("Failed to create string handle: %u\n", DdeGetLastError(dde_inst));
 
@@ -1133,10 +1124,6 @@ DWORD WINAPI IEWinMain(const WCHAR *cmdline, int nShowWindow)
     ULONG_PTR context_cookie;
     ACTCTXW actctx;
 
-    static const WCHAR embeddingW[] = {'-','e','m','b','e','d','d','i','n','g',0};
-    static const WCHAR nohomeW[] = {'-','n','o','h','o','m','e',0};
-    static const WCHAR startmanagerW[] = {'-','s','t','a','r','t','m','a','n','a','g','e','r',0};
-
     TRACE("%s %d\n", debugstr_w(cmdline), nShowWindow);
 
     CoInitialize(NULL);
@@ -1161,11 +1148,11 @@ DWORD WINAPI IEWinMain(const WCHAR *cmdline, int nShowWindow)
 
         while (cmdline[length] && cmdline[length] != ' ' && cmdline[length] != '\t') length++;
 
-        if (!wcsnicmp(cmdline, embeddingW, length))
+        if (!wcsnicmp(cmdline, L"-embedding", length))
             embedding = TRUE;
-        else if (!wcsnicmp(cmdline, nohomeW, length))
+        else if (!wcsnicmp(cmdline, L"-nohome", length))
             nohome = TRUE;
-        else if (!wcsnicmp(cmdline, startmanagerW, length))
+        else if (!wcsnicmp(cmdline, L"-startmanager", length))
             manager = TRUE;
         else
             break;
