@@ -306,6 +306,38 @@ argumentsTest();
     ok(arguments === 1, "arguments = " + arguments);
 })();
 
+// duplicated argument names are shadowed by the last argument with the same name
+(function() {
+    var args, get_a, set_a;
+
+    (function(a, a, b, c) {
+        get_a = function() { return a; }
+        set_a = function(v) { a = v; }
+        ok(get_a() === 2, "function(a, a, b, c) get_a() = " + get_a());
+        ok(a === 2, "function(a, a, b, c) a = " + a);
+        ok(b === 3, "function(a, a, b, c) b = " + b);
+        ok(c === 4, "function(a, a, b, c) c = " + c);
+        a = 42;
+        ok(arguments[0] === 1, "function(a, a, b, c) arguments[0] = " + arguments[0]);
+        ok(arguments[1] === 42, "function(a, a, b, c) arguments[1] = " + arguments[1]);
+        ok(get_a() === 42, "function(a, a, b, c) get_a() = " + get_a() + " expected 42");
+        args = arguments;
+    })(1, 2, 3, 4);
+
+    ok(get_a() === 42, "function(a, a, b, c) get_a() after detach = " + get_a());
+    set_a(100);
+    ok(get_a() === 100, "function(a, a, b, c) get_a() = " + get_a() + " expected 100");
+    ok(args[0] === 1, "function(a, a, b, c) detached args[0] = " + args[0]);
+    ok(args[1] === 42, "function(a, a, b, c) detached args[1] = " + args[1]);
+
+    (function(a, a) {
+        eval("var a = 7;");
+        ok(a === 7, "function(a, a) a = " + a);
+        ok(arguments[0] === 5, "function(a, a) arguments[0] = " + arguments[0]);
+        ok(arguments[1] === 7, "function(a, a) arguments[1] = " + arguments[1]);
+    })(5, 6);
+})();
+
 (function callAsExprTest() {
     ok(callAsExprTest.arguments === null, "callAsExprTest.arguments = " + callAsExprTest.arguments);
 })(1,2);
@@ -415,6 +447,9 @@ obj1.func = function () {
     ok(arguments.length === 1, "arguments.length is not 1");
     ok(arguments["0"] === true, "arguments[0] is not true");
     ok(typeof(arguments.callee) === "function", "typeof(arguments.calee) = " + typeof(arguments.calee));
+    ok(arguments.caller === null, "arguments.caller = " + arguments.caller);
+    function test_caller() { ok(arguments.caller === foobar.arguments, "nested arguments.caller = " + arguments.caller); }
+    function foobar() { test_caller(); } foobar();
 
     return "test";
 };
@@ -1559,6 +1594,7 @@ inobj.test2 = true;
 
 tmp = 0;
 for(iter in inobj) {
+    forinTestObj.prototype.test4 = true;
     arr[iter] = true;
     tmp++;
 }
@@ -1567,6 +1603,7 @@ ok(tmp === 3, "for..in tmp = " + tmp);
 ok(arr["test1"] === true, "arr[test1] !== true");
 ok(arr["test2"] === true, "arr[test2] !== true");
 ok(arr["test3"] === true, "arr[test3] !== true");
+ok(arr["test4"] !== true, "arr[test4] === true");
 
 ok((delete inobj.test1) === true, "delete inobj.test1 returned false");
 ok(!("test1" in inobj), "test1 is still in inobj after delete");
@@ -1678,6 +1715,18 @@ try {
     }
 })();
 
+(function() {
+    function constr() {}
+    constr.prototype = { prop: 1 };
+    var o = new constr(), r;
+    ok(o.prop === 1, "o.prop = " + o.prop);
+    r = delete constr.prototype.prop;
+    ok(r === true, "delete returned " + r);
+    ok(o.prop === undefined, "o.prop = " + o.prop);
+    r = delete o["prop"];
+    ok(r === true, "delete returned " + r);
+})();
+
 if (false)
     if (true)
         ok(false, "if evaluated");
