@@ -17,7 +17,6 @@
  */
 
 #define COBJMACROS
-#define NONAMELESSUNION
 
 #include <stdarg.h>
 
@@ -25,7 +24,6 @@
 #include "winbase.h"
 #include "winnls.h"
 #include "winreg.h"
-#include "wine/heap.h"
 #include "wine/debug.h"
 #include "wine/list.h"
 
@@ -37,7 +35,7 @@
 #include "audiopolicy.h"
 #include "spatialaudioclient.h"
 
-#include "mmdevapi.h"
+#include "mmdevapi_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mmdevapi);
 
@@ -51,6 +49,36 @@ static UINT32 AudioObjectType_to_index(AudioObjectType type)
     return o - 2;
 }
 
+static const char *debugstr_fmtex(const WAVEFORMATEX *fmt)
+{
+    static char buf[2048];
+
+    if (!fmt)
+    {
+        strcpy(buf, "(null)");
+    }
+    else if(fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE)
+    {
+        const WAVEFORMATEXTENSIBLE *fmtex = (const WAVEFORMATEXTENSIBLE *)fmt;
+        snprintf(buf, sizeof(buf), "tag: 0x%x (%s), ch: %u (mask: 0x%lx), rate: %lu, depth: %u",
+                fmt->wFormatTag, debugstr_guid(&fmtex->SubFormat),
+                fmt->nChannels, fmtex->dwChannelMask, fmt->nSamplesPerSec,
+                fmt->wBitsPerSample);
+    }
+    else
+    {
+        snprintf(buf, sizeof(buf), "tag: 0x%x, ch: %u, rate: %lu, depth: %u",
+                fmt->wFormatTag, fmt->nChannels, fmt->nSamplesPerSec,
+                fmt->wBitsPerSample);
+    }
+    return buf;
+}
+
+static BOOL formats_equal(const WAVEFORMATEX *fmt1, const WAVEFORMATEX *fmt2)
+{
+    return !memcmp(fmt1, fmt2, sizeof(*fmt1)) && !memcmp(fmt1 + 1, fmt2 + 1, fmt1->cbSize);
+}
+
 typedef struct SpatialAudioImpl SpatialAudioImpl;
 typedef struct SpatialAudioStreamImpl SpatialAudioStreamImpl;
 typedef struct SpatialAudioObjectImpl SpatialAudioObjectImpl;
@@ -146,7 +174,7 @@ static ULONG WINAPI SAO_AddRef(ISpatialAudioObject *iface)
 {
     SpatialAudioObjectImpl *This = impl_from_ISpatialAudioObject(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
-    TRACE("(%p) new ref %u\n", This, ref);
+    TRACE("(%p) new ref %lu\n", This, ref);
     return ref;
 }
 
@@ -154,15 +182,15 @@ static ULONG WINAPI SAO_Release(ISpatialAudioObject *iface)
 {
     SpatialAudioObjectImpl *This = impl_from_ISpatialAudioObject(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
-    TRACE("(%p) new ref %u\n", This, ref);
+    TRACE("(%p) new ref %lu\n", This, ref);
     if(!ref){
         EnterCriticalSection(&This->sa_stream->lock);
         list_remove(&This->entry);
         LeaveCriticalSection(&This->sa_stream->lock);
 
         ISpatialAudioObjectRenderStream_Release(&This->sa_stream->ISpatialAudioObjectRenderStream_iface);
-        heap_free(This->buf);
-        heap_free(This);
+        free(This->buf);
+        free(This);
     }
     return ref;
 }
@@ -272,7 +300,7 @@ static ULONG WINAPI SAORS_AddRef(ISpatialAudioObjectRenderStream *iface)
 {
     SpatialAudioStreamImpl *This = impl_from_ISpatialAudioObjectRenderStream(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
-    TRACE("(%p) new ref %u\n", This, ref);
+    TRACE("(%p) new ref %lu\n", This, ref);
     return ref;
 }
 
@@ -280,7 +308,7 @@ static ULONG WINAPI SAORS_Release(ISpatialAudioObjectRenderStream *iface)
 {
     SpatialAudioStreamImpl *This = impl_from_ISpatialAudioObjectRenderStream(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
-    TRACE("(%p) new ref %u\n", This, ref);
+    TRACE("(%p) new ref %lu\n", This, ref);
     if(!ref){
         IAudioClient_Stop(This->client);
         if(This->update_frames != ~0 && This->update_frames > 0)
@@ -289,11 +317,11 @@ static ULONG WINAPI SAORS_Release(ISpatialAudioObjectRenderStream *iface)
         IAudioClient_Release(This->client);
         if(This->params.NotifyObject)
             ISpatialAudioObjectRenderStreamNotify_Release(This->params.NotifyObject);
-        heap_free((void*)This->params.ObjectFormat);
+        free((void*)This->params.ObjectFormat);
         CloseHandle(This->params.EventHandle);
         DeleteCriticalSection(&This->lock);
         ISpatialAudioClient_Release(&This->sa_client->ISpatialAudioClient_iface);
-        heap_free(This);
+        free(This);
     }
     return ref;
 }
@@ -325,7 +353,7 @@ static HRESULT WINAPI SAORS_Start(ISpatialAudioObjectRenderStream *iface)
 
     hr = IAudioClient_Start(This->client);
     if(FAILED(hr)){
-        WARN("IAudioClient::Start failed: %08x\n", hr);
+        WARN("IAudioClient::Start failed: %08lx\n", hr);
         return hr;
     }
 
@@ -341,7 +369,7 @@ static HRESULT WINAPI SAORS_Stop(ISpatialAudioObjectRenderStream *iface)
 
     hr = IAudioClient_Stop(This->client);
     if(FAILED(hr)){
-        WARN("IAudioClient::Stop failed: %08x\n", hr);
+        WARN("IAudioClient::Stop failed: %08lx\n", hr);
         return hr;
     }
 
@@ -377,7 +405,7 @@ static HRESULT WINAPI SAORS_BeginUpdatingAudioObjects(ISpatialAudioObjectRenderS
     if(This->update_frames > 0){
         hr = IAudioRenderClient_GetBuffer(This->render, This->update_frames, (BYTE **)&This->buf);
         if(FAILED(hr)){
-            WARN("GetBuffer failed: %08x\n", hr);
+            WARN("GetBuffer failed: %08lx\n", hr);
             This->update_frames = ~0;
             LeaveCriticalSection(&This->lock);
             return hr;
@@ -441,7 +469,7 @@ static HRESULT WINAPI SAORS_EndUpdatingAudioObjects(ISpatialAudioObjectRenderStr
 
         hr = IAudioRenderClient_ReleaseBuffer(This->render, This->update_frames, 0);
         if(FAILED(hr))
-            WARN("ReleaseBuffer failed: %08x\n", hr);
+            WARN("ReleaseBuffer failed: %08lx\n", hr);
     }
 
     This->update_frames = ~0;
@@ -470,7 +498,7 @@ static HRESULT WINAPI SAORS_ActivateSpatialAudioObject(ISpatialAudioObjectRender
             return SPTLAUDCLNT_E_OBJECT_ALREADY_ACTIVE;
     }
 
-    obj = heap_alloc_zero(sizeof(*obj));
+    obj = calloc(1, sizeof(*obj));
     obj->ISpatialAudioObject_iface.lpVtbl = &ISpatialAudioObject_vtbl;
     obj->ref = 1;
     obj->type = type;
@@ -484,7 +512,7 @@ static HRESULT WINAPI SAORS_ActivateSpatialAudioObject(ISpatialAudioObjectRender
     obj->sa_stream = This;
     SAORS_AddRef(&This->ISpatialAudioObjectRenderStream_iface);
 
-    obj->buf = heap_alloc_zero(This->period_frames * This->sa_client->object_fmtex.Format.nBlockAlign);
+    obj->buf = calloc(This->period_frames, This->sa_client->object_fmtex.Format.nBlockAlign);
 
     EnterCriticalSection(&This->lock);
 
@@ -538,7 +566,7 @@ static ULONG WINAPI SAC_AddRef(ISpatialAudioClient *iface)
 {
     SpatialAudioImpl *This = impl_from_ISpatialAudioClient(iface);
     ULONG ref = InterlockedIncrement(&This->ref);
-    TRACE("(%p) new ref %u\n", This, ref);
+    TRACE("(%p) new ref %lu\n", This, ref);
     return ref;
 }
 
@@ -546,10 +574,10 @@ static ULONG WINAPI SAC_Release(ISpatialAudioClient *iface)
 {
     SpatialAudioImpl *This = impl_from_ISpatialAudioClient(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
-    TRACE("(%p) new ref %u\n", This, ref);
+    TRACE("(%p) new ref %lu\n", This, ref);
     if (!ref) {
         IMMDevice_Release(This->mmdev);
-        heap_free(This);
+        free(This);
     }
     return ref;
 }
@@ -612,9 +640,20 @@ static HRESULT WINAPI SAC_GetMaxFrameCount(ISpatialAudioClient *iface,
 static HRESULT WINAPI SAC_IsAudioObjectFormatSupported(ISpatialAudioClient *iface,
         const WAVEFORMATEX *format)
 {
-    SpatialAudioImpl *This = impl_from_ISpatialAudioClient(iface);
-    FIXME("(%p)->(%p)\n", This, format);
-    return E_NOTIMPL;
+    SpatialAudioImpl *sac = impl_from_ISpatialAudioClient(iface);
+
+    TRACE("sac %p, format %s.\n", sac, debugstr_fmtex(format));
+
+    if (!format)
+        return E_POINTER;
+
+    if (!formats_equal(&sac->object_fmtex.Format, format))
+    {
+        FIXME("Reporting format %s as unsupported.\n", debugstr_fmtex(format));
+        return E_INVALIDARG;
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI SAC_IsSpatialAudioStreamAvailable(ISpatialAudioClient *iface,
@@ -627,28 +666,11 @@ static HRESULT WINAPI SAC_IsSpatialAudioStreamAvailable(ISpatialAudioClient *ifa
 
 static WAVEFORMATEX *clone_fmtex(const WAVEFORMATEX *src)
 {
-    WAVEFORMATEX *r = heap_alloc(sizeof(WAVEFORMATEX) + src->cbSize);
+    WAVEFORMATEX *r = malloc(sizeof(WAVEFORMATEX) + src->cbSize);
     memcpy(r, src, sizeof(WAVEFORMATEX) + src->cbSize);
     return r;
 }
 
-static const char *debugstr_fmtex(const WAVEFORMATEX *fmt)
-{
-    static char buf[2048];
-    if(fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE){
-        const WAVEFORMATEXTENSIBLE *fmtex = (const WAVEFORMATEXTENSIBLE *)fmt;
-        snprintf(buf, sizeof(buf), "tag: 0x%x (%s), ch: %u (mask: 0x%x), rate: %u, depth: %u",
-                fmt->wFormatTag, debugstr_guid(&fmtex->SubFormat),
-                fmt->nChannels, fmtex->dwChannelMask, fmt->nSamplesPerSec,
-                fmt->wBitsPerSample);
-    }else{
-        snprintf(buf, sizeof(buf), "tag: 0x%x, ch: %u, rate: %u, depth: %u",
-                fmt->wFormatTag, fmt->nChannels, fmt->nSamplesPerSec,
-                fmt->wBitsPerSample);
-    }
-    return buf;
-}
-
 static void static_mask_to_channels(AudioObjectType static_mask, WORD *count, DWORD *mask, UINT32 *map)
 {
     UINT32 out_chan = 0, map_idx = 0;
@@ -694,13 +716,13 @@ static HRESULT activate_stream(SpatialAudioStreamImpl *stream)
     hr = IMMDevice_Activate(stream->sa_client->mmdev, &IID_IAudioClient,
             CLSCTX_INPROC_SERVER, NULL, (void**)&stream->client);
     if(FAILED(hr)){
-        WARN("Activate failed: %08x\n", hr);
+        WARN("Activate failed: %08lx\n", hr);
         return hr;
     }
 
     hr = IAudioClient_GetDevicePeriod(stream->client, &period, NULL);
     if(FAILED(hr)){
-        WARN("GetDevicePeriod failed: %08x\n", hr);
+        WARN("GetDevicePeriod failed: %08lx\n", hr);
         IAudioClient_Release(stream->client);
         return hr;
     }
@@ -718,24 +740,24 @@ static HRESULT activate_stream(SpatialAudioStreamImpl *stream)
     stream->stream_fmtex.SubFormat = KSDATAFORMAT_SUBTYPE_IEEE_FLOAT;
 
     hr = IAudioClient_Initialize(stream->client, AUDCLNT_SHAREMODE_SHARED,
-            AUDCLNT_STREAMFLAGS_EVENTCALLBACK | AUDCLNT_STREAMFLAGS_NOPERSIST,
+            AUDCLNT_STREAMFLAGS_EVENTCALLBACK | AUDCLNT_STREAMFLAGS_NOPERSIST | AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM,
             period, 0, &stream->stream_fmtex.Format, NULL);
     if(FAILED(hr)){
-        WARN("Initialize failed: %08x\n", hr);
+        WARN("Initialize failed: %08lx\n", hr);
         IAudioClient_Release(stream->client);
         return hr;
     }
 
     hr = IAudioClient_SetEventHandle(stream->client, stream->params.EventHandle);
     if(FAILED(hr)){
-        WARN("SetEventHandle failed: %08x\n", hr);
+        WARN("SetEventHandle failed: %08lx\n", hr);
         IAudioClient_Release(stream->client);
         return hr;
     }
 
     hr = IAudioClient_GetService(stream->client, &IID_IAudioRenderClient, (void**)&stream->render);
     if(FAILED(hr)){
-        WARN("GetService(AudioRenderClient) failed: %08x\n", hr);
+        WARN("GetService(AudioRenderClient) failed: %08lx\n", hr);
         IAudioClient_Release(stream->client);
         return hr;
     }
@@ -778,13 +800,12 @@ static HRESULT WINAPI SAC_ActivateSpatialAudioStream(ISpatialAudioClient *iface,
             return E_INVALIDARG;
         }
 
-        if(!params->ObjectFormat ||
-                memcmp(params->ObjectFormat, &This->object_fmtex.Format, sizeof(*params->ObjectFormat) + params->ObjectFormat->cbSize)){
+        if(!(params->ObjectFormat && formats_equal(params->ObjectFormat, &This->object_fmtex.Format))) {
             *stream = NULL;
             return AUDCLNT_E_UNSUPPORTED_FORMAT;
         }
 
-        obj = heap_alloc_zero(sizeof(SpatialAudioStreamImpl));
+        obj = calloc(1, sizeof(SpatialAudioStreamImpl));
 
         obj->ISpatialAudioObjectRenderStream_iface.lpVtbl = &ISpatialAudioObjectRenderStream_vtbl;
         obj->ref = 1;
@@ -822,10 +843,10 @@ static HRESULT WINAPI SAC_ActivateSpatialAudioStream(ISpatialAudioClient *iface,
             if(obj->params.NotifyObject)
                 ISpatialAudioObjectRenderStreamNotify_Release(obj->params.NotifyObject);
             DeleteCriticalSection(&obj->lock);
-            heap_free((void*)obj->params.ObjectFormat);
+            free((void*)obj->params.ObjectFormat);
             CloseHandle(obj->params.EventHandle);
             ISpatialAudioClient_Release(&obj->sa_client->ISpatialAudioClient_iface);
-            heap_free(obj);
+            free(obj);
             *stream = NULL;
             return hr;
         }
@@ -914,7 +935,7 @@ HRESULT SpatialAudioClient_Create(IMMDevice *mmdev, ISpatialAudioClient **out)
     WAVEFORMATEX *closest;
     HRESULT hr;
 
-    obj = heap_alloc_zero(sizeof(*obj));
+    obj = calloc(1, sizeof(*obj));
 
     obj->ref = 1;
     obj->ISpatialAudioClient_iface.lpVtbl = &ISpatialAudioClient_vtbl;
@@ -931,8 +952,8 @@ HRESULT SpatialAudioClient_Create(IMMDevice *mmdev, ISpatialAudioClient **out)
     hr = IMMDevice_Activate(mmdev, &IID_IAudioClient,
             CLSCTX_INPROC_SERVER, NULL, (void**)&aclient);
     if(FAILED(hr)){
-        WARN("Activate failed: %08x\n", hr);
-        heap_free(obj);
+        WARN("Activate failed: %08lx\n", hr);
+        free(obj);
         return hr;
     }
 
@@ -944,7 +965,7 @@ HRESULT SpatialAudioClient_Create(IMMDevice *mmdev, ISpatialAudioClient **out)
         if(sizeof(WAVEFORMATEX) + closest->cbSize > sizeof(obj->object_fmtex)){
             ERR("Returned format too large: %s\n", debugstr_fmtex(closest));
             CoTaskMemFree(closest);
-            heap_free(obj);
+            free(obj);
             return AUDCLNT_E_UNSUPPORTED_FORMAT;
         }else if(!((closest->wFormatTag == WAVE_FORMAT_IEEE_FLOAT ||
                     (closest->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
@@ -953,7 +974,7 @@ HRESULT SpatialAudioClient_Create(IMMDevice *mmdev, ISpatialAudioClient **out)
                     closest->wBitsPerSample == 32)){
             ERR("Returned format not 32-bit float: %s\n", debugstr_fmtex(closest));
             CoTaskMemFree(closest);
-            heap_free(obj);
+            free(obj);
             return AUDCLNT_E_UNSUPPORTED_FORMAT;
         }
         WARN("The audio stack doesn't support 48kHz 32bit float. Using the closest match. Audio may be glitchy. %s\n", debugstr_fmtex(closest));
@@ -962,8 +983,8 @@ HRESULT SpatialAudioClient_Create(IMMDevice *mmdev, ISpatialAudioClient **out)
                sizeof(WAVEFORMATEX) + closest->cbSize);
         CoTaskMemFree(closest);
     } else if(hr != S_OK){
-        WARN("Checking supported formats failed: %08x\n", hr);
-        heap_free(obj);
+        WARN("Checking supported formats failed: %08lx\n", hr);
+        free(obj);
         return hr;
     }
 
