@@ -25,7 +25,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
-static int cookie_counter;
+static LONG cookie_counter;
 
 struct advise_sink
 {
@@ -42,14 +42,21 @@ struct system_clock
     IUnknown *outer_unk;
     LONG refcount;
 
-    BOOL thread_created;
-    HANDLE thread, notify_event, stop_event;
+    LONG thread_created;
+    BOOL thread_stopped;
+    HANDLE thread;
     REFERENCE_TIME last_time;
     CRITICAL_SECTION cs;
+    CONDITION_VARIABLE cv;
 
     struct list sinks;
 };
 
+static REFERENCE_TIME get_current_time(void)
+{
+    return (REFERENCE_TIME)timeGetTime() * 10000;
+}
+
 static inline struct system_clock *impl_from_IUnknown(IUnknown *iface)
 {
     return CONTAINING_RECORD(iface, struct system_clock, IUnknown_inner);
@@ -80,7 +87,7 @@ static ULONG WINAPI system_clock_inner_AddRef(IUnknown *iface)
     struct system_clock *clock = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedIncrement(&clock->refcount);
 
-    TRACE("%p increasing refcount to %u.\n", clock, refcount);
+    TRACE("%p increasing refcount to %lu.\n", clock, refcount);
 
     return refcount;
 }
@@ -89,22 +96,31 @@ static ULONG WINAPI system_clock_inner_Release(IUnknown *iface)
 {
     struct system_clock *clock = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedDecrement(&clock->refcount);
+    struct advise_sink *sink, *cursor;
 
-    TRACE("%p decreasing refcount to %u.\n", clock, refcount);
+    TRACE("%p decreasing refcount to %lu.\n", clock, refcount);
 
     if (!refcount)
     {
         if (clock->thread)
         {
-            SetEvent(clock->stop_event);
+            EnterCriticalSection(&clock->cs);
+            clock->thread_stopped = TRUE;
+            LeaveCriticalSection(&clock->cs);
+            WakeConditionVariable(&clock->cv);
             WaitForSingleObject(clock->thread, INFINITE);
             CloseHandle(clock->thread);
-            CloseHandle(clock->notify_event);
-            CloseHandle(clock->stop_event);
         }
+
+        LIST_FOR_EACH_ENTRY_SAFE(sink, cursor, &clock->sinks, struct advise_sink, entry)
+        {
+            list_remove(&sink->entry);
+            free(sink);
+        }
+
         clock->cs.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&clock->cs);
-        heap_free(clock);
+        free(clock);
     }
     return refcount;
 }
@@ -126,9 +142,9 @@ static DWORD WINAPI SystemClockAdviseThread(void *param)
     struct system_clock *clock = param;
     struct advise_sink *sink, *cursor;
     REFERENCE_TIME current_time;
-    HANDLE handles[2] = {clock->stop_event, clock->notify_event};
 
     TRACE("Starting advise thread for clock %p.\n", clock);
+    SetThreadDescription(GetCurrentThread(), L"wine_qz_clock_advise");
 
     for (;;)
     {
@@ -136,7 +152,7 @@ static DWORD WINAPI SystemClockAdviseThread(void *param)
 
         EnterCriticalSection(&clock->cs);
 
-        current_time = GetTickCount64() * 10000;
+        current_time = get_current_time();
 
         LIST_FOR_EACH_ENTRY_SAFE(sink, cursor, &clock->sinks, struct advise_sink, entry)
         {
@@ -152,7 +168,7 @@ static DWORD WINAPI SystemClockAdviseThread(void *param)
                 {
                     SetEvent(sink->handle);
                     list_remove(&sink->entry);
-                    heap_free(sink);
+                    free(sink);
                     continue;
                 }
             }
@@ -160,22 +176,47 @@ static DWORD WINAPI SystemClockAdviseThread(void *param)
             timeout = min(timeout, (sink->due_time - current_time) / 10000);
         }
 
-        LeaveCriticalSection(&clock->cs);
-
-        if (WaitForMultipleObjects(2, handles, FALSE, timeout) == 0)
+        SleepConditionVariableCS(&clock->cv, &clock->cs, timeout);
+        if (clock->thread_stopped)
+        {
+            LeaveCriticalSection(&clock->cs);
             return 0;
+        }
+        LeaveCriticalSection(&clock->cs);
     }
 }
 
-static void notify_thread(struct system_clock *clock)
+static HRESULT add_sink(struct system_clock *clock, DWORD_PTR handle,
+        REFERENCE_TIME due_time, REFERENCE_TIME period, DWORD_PTR *cookie)
 {
+    struct advise_sink *sink;
+
+    if (!handle)
+        return E_INVALIDARG;
+
+    if (!cookie)
+        return E_POINTER;
+
+    if (!(sink = calloc(1, sizeof(*sink))))
+        return E_OUTOFMEMORY;
+
+    sink->handle = (HANDLE)handle;
+    sink->due_time = due_time;
+    sink->period = period;
+    sink->cookie = InterlockedIncrement(&cookie_counter);
+    *cookie = sink->cookie;
+
+    EnterCriticalSection(&clock->cs);
+    list_add_tail(&clock->sinks, &sink->entry);
+    LeaveCriticalSection(&clock->cs);
+
     if (!InterlockedCompareExchange(&clock->thread_created, TRUE, FALSE))
     {
-        clock->notify_event = CreateEventW(NULL, FALSE, FALSE, NULL);
-        clock->stop_event = CreateEventW(NULL, TRUE, FALSE, NULL);
         clock->thread = CreateThread(NULL, 0, SystemClockAdviseThread, clock, 0, NULL);
     }
-    SetEvent(clock->notify_event);
+    WakeConditionVariable(&clock->cv);
+
+    return S_OK;
 }
 
 static HRESULT WINAPI SystemClockImpl_QueryInterface(IReferenceClock *iface, REFIID iid, void **out)
@@ -202,13 +243,11 @@ static HRESULT WINAPI SystemClockImpl_GetTime(IReferenceClock *iface, REFERENCE_
     REFERENCE_TIME ret;
     HRESULT hr;
 
-    TRACE("clock %p, time %p.\n", clock, time);
-
     if (!time) {
         return E_POINTER;
     }
 
-    ret = GetTickCount64() * 10000;
+    ret = get_current_time();
 
     EnterCriticalSection(&clock->cs);
 
@@ -217,6 +256,7 @@ static HRESULT WINAPI SystemClockImpl_GetTime(IReferenceClock *iface, REFERENCE_
 
     LeaveCriticalSection(&clock->cs);
 
+    TRACE("clock %p, time %p, returning %s.\n", clock, time, debugstr_time(ret));
     return hr;
 }
 
@@ -224,72 +264,28 @@ static HRESULT WINAPI SystemClockImpl_AdviseTime(IReferenceClock *iface,
         REFERENCE_TIME base, REFERENCE_TIME offset, HEVENT event, DWORD_PTR *cookie)
 {
     struct system_clock *clock = impl_from_IReferenceClock(iface);
-    struct advise_sink *sink;
 
-    TRACE("clock %p, base %s, offset %s, event %#lx, cookie %p.\n",
-            clock, wine_dbgstr_longlong(base), wine_dbgstr_longlong(offset), event, cookie);
-
-    if (!event)
-        return E_INVALIDARG;
+    TRACE("clock %p, base %s, offset %s, event %#Ix, cookie %p.\n",
+            clock, debugstr_time(base), debugstr_time(offset), event, cookie);
 
     if (base + offset <= 0)
         return E_INVALIDARG;
 
-    if (!cookie)
-        return E_POINTER;
-
-    if (!(sink = heap_alloc_zero(sizeof(*sink))))
-        return E_OUTOFMEMORY;
-
-    sink->handle = (HANDLE)event;
-    sink->due_time = base + offset;
-    sink->period = 0;
-    sink->cookie = InterlockedIncrement(&cookie_counter);
-
-    EnterCriticalSection(&clock->cs);
-    list_add_tail(&clock->sinks, &sink->entry);
-    LeaveCriticalSection(&clock->cs);
-
-    notify_thread(clock);
-
-    *cookie = sink->cookie;
-    return S_OK;
+    return add_sink(clock, event, base + offset, 0, cookie);
 }
 
 static HRESULT WINAPI SystemClockImpl_AdvisePeriodic(IReferenceClock* iface,
         REFERENCE_TIME start, REFERENCE_TIME period, HSEMAPHORE semaphore, DWORD_PTR *cookie)
 {
     struct system_clock *clock = impl_from_IReferenceClock(iface);
-    struct advise_sink *sink;
 
-    TRACE("clock %p, start %s, period %s, semaphore %#lx, cookie %p.\n",
-            clock, wine_dbgstr_longlong(start), wine_dbgstr_longlong(period), semaphore, cookie);
-
-    if (!semaphore)
-        return E_INVALIDARG;
+    TRACE("clock %p, start %s, period %s, semaphore %#Ix, cookie %p.\n",
+            clock, debugstr_time(start), debugstr_time(period), semaphore, cookie);
 
     if (start <= 0 || period <= 0)
         return E_INVALIDARG;
 
-    if (!cookie)
-        return E_POINTER;
-
-    if (!(sink = heap_alloc_zero(sizeof(*sink))))
-        return E_OUTOFMEMORY;
-
-    sink->handle = (HANDLE)semaphore;
-    sink->due_time = start;
-    sink->period = period;
-    sink->cookie = InterlockedIncrement(&cookie_counter);
-
-    EnterCriticalSection(&clock->cs);
-    list_add_tail(&clock->sinks, &sink->entry);
-    LeaveCriticalSection(&clock->cs);
-
-    notify_thread(clock);
-
-    *cookie = sink->cookie;
-    return S_OK;
+    return add_sink(clock, semaphore, start, period, cookie);
 }
 
 static HRESULT WINAPI SystemClockImpl_Unadvise(IReferenceClock *iface, DWORD_PTR cookie)
@@ -297,7 +293,7 @@ static HRESULT WINAPI SystemClockImpl_Unadvise(IReferenceClock *iface, DWORD_PTR
     struct system_clock *clock = impl_from_IReferenceClock(iface);
     struct advise_sink *sink;
 
-    TRACE("clock %p, cookie %#lx.\n", clock, cookie);
+    TRACE("clock %p, cookie %#Ix.\n", clock, cookie);
 
     EnterCriticalSection(&clock->cs);
 
@@ -306,7 +302,7 @@ static HRESULT WINAPI SystemClockImpl_Unadvise(IReferenceClock *iface, DWORD_PTR
         if (sink->cookie == cookie)
         {
             list_remove(&sink->entry);
-            heap_free(sink);
+            free(sink);
             LeaveCriticalSection(&clock->cs);
             return S_OK;
         }
@@ -328,13 +324,13 @@ static const IReferenceClockVtbl SystemClock_vtbl =
     SystemClockImpl_Unadvise
 };
 
-HRESULT QUARTZ_CreateSystemClock(IUnknown *outer, void **out)
+HRESULT system_clock_create(IUnknown *outer, IUnknown **out)
 {
     struct system_clock *object;
-  
+
     TRACE("outer %p, out %p.\n", outer, out);
 
-    if (!(object = heap_alloc_zero(sizeof(*object))))
+    if (!(object = calloc(1, sizeof(*object))))
     {
         *out = NULL;
         return E_OUTOFMEMORY;
@@ -345,7 +341,7 @@ HRESULT QUARTZ_CreateSystemClock(IUnknown *outer, void **out)
     object->outer_unk = outer ? outer : &object->IUnknown_inner;
     object->refcount = 1;
     list_init(&object->sinks);
-    InitializeCriticalSection(&object->cs);
+    InitializeCriticalSectionEx(&object->cs, 0, RTL_CRITICAL_SECTION_FLAG_FORCE_DEBUG_INFO);
     object->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": SystemClockImpl.cs");
 
     TRACE("Created system clock %p.\n", object);
