@@ -219,6 +219,7 @@ static void test_ShowScrollBar(void)
 
 static void test_GetScrollBarInfo(void)
 {
+    int width, height, expected_width, expected_height;
     HWND hMainWnd;
     BOOL ret;
     SCROLLBARINFO sbi;
@@ -278,6 +279,26 @@ static void test_GetScrollBarInfo(void)
     ok(EqualRect(&rect, &sbi.rcScrollBar), "PreviousRect %s != CurrentRect %s\n",
        wine_dbgstr_rect(&rect), wine_dbgstr_rect(&sbi.rcScrollBar));
 
+    /* Test width when WS_VSCROLL is present */
+    ok( GetWindowLongA( hMainWnd, GWL_STYLE ) & WS_VSCROLL , "Expected WS_VSCROLL.\n" );
+    sbi.cbSize = sizeof(sbi);
+    ret = pGetScrollBarInfo( hMainWnd, OBJID_HSCROLL, &sbi );
+    ok( ret, "The GetScrollBarInfo() call should not fail.\n" );
+    width = sbi.rcScrollBar.right - sbi.rcScrollBar.left;
+    GetClientRect( hMainWnd, &rect );
+    expected_width = rect.right - rect.left;
+    ok( width == expected_width, "Expected width %d, got %d.\n", expected_width, width );
+
+    /* Test height when WS_HSCROLL is present */
+    ok( GetWindowLongA( hMainWnd, GWL_STYLE ) & WS_HSCROLL , "Expected WS_HSCROLL.\n" );
+    sbi.cbSize = sizeof(sbi);
+    ret = pGetScrollBarInfo( hMainWnd, OBJID_VSCROLL, &sbi );
+    ok( ret, "The GetScrollBarInfo() call should not fail.\n" );
+    height = sbi.rcScrollBar.bottom - sbi.rcScrollBar.top;
+    GetClientRect( hMainWnd, &rect );
+    expected_height = rect.bottom - rect.top;
+    ok( height == expected_height, "Expected height %d, got %d.\n", expected_height, height );
+
     DestroyWindow(hScroll);
     DestroyWindow(hMainWnd);
 }
@@ -944,9 +965,7 @@ static void test_cross_process(const char* argv0)
     ok(ret, "CreateProcess \"%s\" failed err %lu.\n", path, GetLastError());
 
     /* Messages do not need to be processed to read info. */
-    winetest_wait_child_process(process_info.hProcess);
-    CloseHandle(process_info.hProcess);
-    CloseHandle(process_info.hThread);
+    winetest_wait_child_process(&process_info);
 
     sprintf( path, "%s scroll write_process %p", argv0, hMainWnd );
     ret = CreateProcessA( NULL, path, NULL, NULL, FALSE, 0, NULL, NULL, &startup_info, &process_info );
@@ -963,9 +982,7 @@ static void test_cross_process(const char* argv0)
         }
     }
 
-    winetest_wait_child_process(process_info.hProcess);
-    CloseHandle(process_info.hProcess);
-    CloseHandle(process_info.hThread);
+    winetest_wait_child_process(&process_info);
 
     DestroyWindow( hScroll );
     DestroyWindow( hMainWnd );
