@@ -590,7 +590,7 @@ static struct vulkan_physical_device *get_vulkan_physical_device( struct vulkan_
 {
     GUID uuid;
 
-    if (!get_vulkan_uuid_from_luid( luid, &uuid ))
+    if (!get_gpu_uuid_from_luid( luid, &uuid ))
     {
         WARN( "Failed to find Vulkan device with LUID %08x:%08x.\n", luid->HighPart, luid->LowPart );
         return NULL;
@@ -994,7 +994,7 @@ BOOL get_vulkan_gpus( struct list *gpus )
 
     for (i = 0; i < instance->physical_device_count; ++i)
     {
-        struct vulkan_gpu *gpu;
+        struct gpu_info *gpu;
 
         if (!(gpu = calloc( 1, sizeof(*gpu) ))) break;
         memcpy( &gpu->uuid, devinfo[i].id.deviceUUID, sizeof(gpu->uuid) );
@@ -1015,12 +1015,6 @@ BOOL get_vulkan_gpus( struct list *gpus )
     return TRUE;
 }
 
-void free_vulkan_gpu( struct vulkan_gpu *gpu )
-{
-    free( gpu->name );
-    free( gpu );
-}
-
 /******************************************************************************
  *           NtGdiDdDDIShareObjects    (win32u.@)
  */
