@@ -112,51 +112,51 @@ typedef struct dibdrv_physdev
     BOOL   (* pen_lines)(struct dibdrv_physdev *pdev, int num, POINT *pts, BOOL close, HRGN region);
 } dibdrv_physdev;
 
-extern BOOL     CDECL dibdrv_AlphaBlend( PHYSDEV dst_dev, struct bitblt_coords *dst,
-                                         PHYSDEV src_dev, struct bitblt_coords *src, BLENDFUNCTION blend ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                                  INT start_x, INT start_y, INT end_x, INT end_y ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_ArcTo( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                                    INT start_x, INT start_y, INT end_x, INT end_y ) DECLSPEC_HIDDEN;
-extern DWORD    CDECL dibdrv_BlendImage( PHYSDEV dev, BITMAPINFO *info, const struct gdi_image_bits *bits,
-                                         struct bitblt_coords *src, struct bitblt_coords *dst, BLENDFUNCTION func ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                                    INT start_x, INT start_y, INT end_x, INT end_y ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags,
-                                         const RECT *rect, LPCWSTR str, UINT count, const INT *dx ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_FillPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern DWORD    CDECL dibdrv_GetImage( PHYSDEV dev, BITMAPINFO *info, struct gdi_image_bits *bits,
-                                       struct bitblt_coords *src ) DECLSPEC_HIDDEN;
-extern COLORREF CDECL dibdrv_GetNearestColor( PHYSDEV dev, COLORREF color ) DECLSPEC_HIDDEN;
-extern COLORREF CDECL dibdrv_GetPixel( PHYSDEV dev, INT x, INT y ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_GradientFill( PHYSDEV dev, TRIVERTEX *vert_array, ULONG nvert,
-                                           void *grad_array, ULONG ngrad, ULONG mode ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_LineTo( PHYSDEV dev, INT x, INT y ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_PatBlt( PHYSDEV dev, struct bitblt_coords *dst, DWORD rop ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_PaintRgn( PHYSDEV dev, HRGN hrgn ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_Pie( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                                  INT start_x, INT start_y, INT end_x, INT end_y ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_PolyPolygon( PHYSDEV dev, const POINT *pt, const INT *counts, UINT polygons ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_PolyPolyline( PHYSDEV dev, const POINT* pt, const DWORD* counts,
-                                           DWORD polylines ) DECLSPEC_HIDDEN;
-extern DWORD    CDECL dibdrv_PutImage( PHYSDEV dev, HRGN clip, BITMAPINFO *info,
-                                       const struct gdi_image_bits *bits, struct bitblt_coords *src,
-                                       struct bitblt_coords *dst, DWORD rop ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bottom ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
-                                        INT ellipse_width, INT ellipse_height ) DECLSPEC_HIDDEN;
-extern HBRUSH   CDECL dibdrv_SelectBrush( PHYSDEV dev, HBRUSH hbrush, const struct brush_pattern *pattern ) DECLSPEC_HIDDEN;
-extern HFONT    CDECL dibdrv_SelectFont( PHYSDEV dev, HFONT font, UINT *aa_flags ) DECLSPEC_HIDDEN;
-extern HPEN     CDECL dibdrv_SelectPen( PHYSDEV dev, HPEN hpen, const struct brush_pattern *pattern ) DECLSPEC_HIDDEN;
-extern COLORREF CDECL dibdrv_SetDCBrushColor( PHYSDEV dev, COLORREF color ) DECLSPEC_HIDDEN;
-extern COLORREF CDECL dibdrv_SetDCPenColor( PHYSDEV dev, COLORREF color ) DECLSPEC_HIDDEN;
-extern COLORREF CDECL dibdrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
-                                         PHYSDEV src_dev, struct bitblt_coords *src, DWORD rop ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_StrokeAndFillPath( PHYSDEV dev ) DECLSPEC_HIDDEN;
-extern BOOL     CDECL dibdrv_StrokePath( PHYSDEV dev ) DECLSPEC_HIDDEN;
+extern BOOL     dibdrv_AlphaBlend( PHYSDEV dst_dev, struct bitblt_coords *dst,
+                                   PHYSDEV src_dev, struct bitblt_coords *src, BLENDFUNCTION blend );
+extern BOOL     dibdrv_Arc( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                            INT start_x, INT start_y, INT end_x, INT end_y );
+extern BOOL     dibdrv_ArcTo( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                              INT start_x, INT start_y, INT end_x, INT end_y );
+extern DWORD    dibdrv_BlendImage( PHYSDEV dev, BITMAPINFO *info, const struct gdi_image_bits *bits,
+                                   struct bitblt_coords *src, struct bitblt_coords *dst, BLENDFUNCTION func );
+extern BOOL     dibdrv_Chord( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                              INT start_x, INT start_y, INT end_x, INT end_y );
+extern BOOL     dibdrv_Ellipse( PHYSDEV dev, INT left, INT top, INT right, INT bottom );
+extern BOOL     dibdrv_ExtFloodFill( PHYSDEV dev, INT x, INT y, COLORREF color, UINT type );
+extern BOOL     dibdrv_ExtTextOut( PHYSDEV dev, INT x, INT y, UINT flags,
+                                   const RECT *rect, LPCWSTR str, UINT count, const INT *dx );
+extern BOOL     dibdrv_FillPath( PHYSDEV dev );
+extern DWORD    dibdrv_GetImage( PHYSDEV dev, BITMAPINFO *info, struct gdi_image_bits *bits,
+                                 struct bitblt_coords *src );
+extern COLORREF dibdrv_GetNearestColor( PHYSDEV dev, COLORREF color );
+extern COLORREF dibdrv_GetPixel( PHYSDEV dev, INT x, INT y );
+extern BOOL     dibdrv_GradientFill( PHYSDEV dev, TRIVERTEX *vert_array, ULONG nvert,
+                                     void *grad_array, ULONG ngrad, ULONG mode );
+extern BOOL     dibdrv_LineTo( PHYSDEV dev, INT x, INT y );
+extern BOOL     dibdrv_PatBlt( PHYSDEV dev, struct bitblt_coords *dst, DWORD rop );
+extern BOOL     dibdrv_PaintRgn( PHYSDEV dev, HRGN hrgn );
+extern BOOL     dibdrv_Pie( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                            INT start_x, INT start_y, INT end_x, INT end_y );
+extern BOOL     dibdrv_PolyPolygon( PHYSDEV dev, const POINT *pt, const INT *counts, UINT polygons );
+extern BOOL     dibdrv_PolyPolyline( PHYSDEV dev, const POINT* pt, const DWORD* counts,
+                                     DWORD polylines );
+extern DWORD    dibdrv_PutImage( PHYSDEV dev, HRGN clip, BITMAPINFO *info,
+                                 const struct gdi_image_bits *bits, struct bitblt_coords *src,
+                                 struct bitblt_coords *dst, DWORD rop );
+extern BOOL     dibdrv_Rectangle( PHYSDEV dev, INT left, INT top, INT right, INT bottom );
+extern BOOL     dibdrv_RoundRect( PHYSDEV dev, INT left, INT top, INT right, INT bottom,
+                                  INT ellipse_width, INT ellipse_height );
+extern HBRUSH   dibdrv_SelectBrush( PHYSDEV dev, HBRUSH hbrush, const struct brush_pattern *pattern );
+extern HFONT    dibdrv_SelectFont( PHYSDEV dev, HFONT font, UINT *aa_flags );
+extern HPEN     dibdrv_SelectPen( PHYSDEV dev, HPEN hpen, const struct brush_pattern *pattern );
+extern COLORREF dibdrv_SetDCBrushColor( PHYSDEV dev, COLORREF color );
+extern COLORREF dibdrv_SetDCPenColor( PHYSDEV dev, COLORREF color );
+extern COLORREF dibdrv_SetPixel( PHYSDEV dev, INT x, INT y, COLORREF color );
+extern BOOL     dibdrv_StretchBlt( PHYSDEV dst_dev, struct bitblt_coords *dst,
+                                   PHYSDEV src_dev, struct bitblt_coords *src, DWORD rop );
+extern BOOL     dibdrv_StrokeAndFillPath( PHYSDEV dev );
+extern BOOL     dibdrv_StrokePath( PHYSDEV dev );
 
 static inline dibdrv_physdev *get_dibdrv_pdev( PHYSDEV dev )
 {
@@ -213,15 +213,15 @@ typedef struct primitive_funcs
                                     const dib_info *src_dib, const struct bitblt_coords *src);
 } primitive_funcs;
 
-extern const primitive_funcs funcs_8888 DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_32   DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_24   DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_555  DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_16   DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_8    DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_4    DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_1    DECLSPEC_HIDDEN;
-extern const primitive_funcs funcs_null DECLSPEC_HIDDEN;
+extern const primitive_funcs funcs_8888;
+extern const primitive_funcs funcs_32;
+extern const primitive_funcs funcs_24;
+extern const primitive_funcs funcs_555;
+extern const primitive_funcs funcs_16;
+extern const primitive_funcs funcs_8;
+extern const primitive_funcs funcs_4;
+extern const primitive_funcs funcs_1;
+extern const primitive_funcs funcs_null;
 
 struct rop_codes
 {
@@ -247,23 +247,23 @@ struct clipped_rects
     RECT  buffer[32];
 };
 
-extern void get_rop_codes(INT rop, struct rop_codes *codes) DECLSPEC_HIDDEN;
-extern void reset_dash_origin(dibdrv_physdev *pdev) DECLSPEC_HIDDEN;
-extern void init_dib_info_from_bitmapinfo(dib_info *dib, const BITMAPINFO *info, void *bits) DECLSPEC_HIDDEN;
-extern BOOL init_dib_info_from_bitmapobj(dib_info *dib, BITMAPOBJ *bmp) DECLSPEC_HIDDEN;
-extern void free_dib_info(dib_info *dib) DECLSPEC_HIDDEN;
-extern void free_pattern_brush(dib_brush *brush) DECLSPEC_HIDDEN;
-extern void copy_dib_color_info(dib_info *dst, const dib_info *src) DECLSPEC_HIDDEN;
-extern BOOL convert_dib(dib_info *dst, const dib_info *src) DECLSPEC_HIDDEN;
-extern DWORD get_pixel_color( DC *dc, const dib_info *dib, COLORREF color, BOOL mono_fixup ) DECLSPEC_HIDDEN;
-extern int get_dib_rect( const dib_info *dib, RECT *rc ) DECLSPEC_HIDDEN;
-extern int clip_rect_to_dib( const dib_info *dib, RECT *rc ) DECLSPEC_HIDDEN;
-extern int get_clipped_rects( const dib_info *dib, const RECT *rc, HRGN clip, struct clipped_rects *clip_rects ) DECLSPEC_HIDDEN;
-extern void add_clipped_bounds( dibdrv_physdev *dev, const RECT *rect, HRGN clip ) DECLSPEC_HIDDEN;
+extern void get_rop_codes(INT rop, struct rop_codes *codes);
+extern void reset_dash_origin(dibdrv_physdev *pdev);
+extern void init_dib_info_from_bitmapinfo(dib_info *dib, const BITMAPINFO *info, void *bits);
+extern BOOL init_dib_info_from_bitmapobj(dib_info *dib, BITMAPOBJ *bmp);
+extern void free_dib_info(dib_info *dib);
+extern void free_pattern_brush(dib_brush *brush);
+extern void copy_dib_color_info(dib_info *dst, const dib_info *src);
+extern BOOL convert_dib(dib_info *dst, const dib_info *src);
+extern DWORD get_pixel_color( DC *dc, const dib_info *dib, COLORREF color, BOOL mono_fixup );
+extern int get_dib_rect( const dib_info *dib, RECT *rc );
+extern int clip_rect_to_dib( const dib_info *dib, RECT *rc );
+extern int get_clipped_rects( const dib_info *dib, const RECT *rc, HRGN clip, struct clipped_rects *clip_rects );
+extern void add_clipped_bounds( dibdrv_physdev *dev, const RECT *rect, HRGN clip );
 extern int clip_line(const POINT *start, const POINT *end, const RECT *clip,
-                     const bres_params *params, POINT *pt1, POINT *pt2) DECLSPEC_HIDDEN;
-extern void release_cached_font( struct cached_font *font ) DECLSPEC_HIDDEN;
-extern BOOL fill_with_pixel( DC *dc, dib_info *dib, DWORD pixel, int num, const RECT *rects, INT rop ) DECLSPEC_HIDDEN;
+                     const bres_params *params, POINT *pt1, POINT *pt2);
+extern void release_cached_font( struct cached_font *font );
+extern BOOL fill_with_pixel( DC *dc, dib_info *dib, DWORD pixel, int num, const RECT *rects, INT rop );
 
 static inline void init_clipped_rects( struct clipped_rects *clip_rects )
 {
@@ -289,15 +289,3 @@ static inline const RGBQUAD *get_dib_color_table( const dib_info *dib )
 {
     return dib->color_table ? dib->color_table : get_default_color_table( dib->bit_count );
 }
-
-struct osmesa_funcs
-{
-    void (*get_gl_funcs)( struct opengl_funcs *funcs );
-    struct wgl_context * (*create_context)( HDC hdc, const PIXELFORMATDESCRIPTOR *descr );
-    BOOL (*delete_context)( struct wgl_context *context );
-    PROC (*get_proc_address)( const char *proc );
-    BOOL (*make_current)( struct wgl_context *context, void *bits,
-                          int width, int height, int bpp, int stride );
-};
-
-extern const struct osmesa_funcs *init_opengl_lib(void) DECLSPEC_HIDDEN;
