@@ -2573,7 +2573,7 @@ BOOL WINAPI DllMain( HINSTANCE inst, DWORD reason, void *reserved )
         LdrGetDllHandle( NULL, 0, &ntdll_name, &ntdll );
         dispatcher_ptr = RtlFindExportedRoutineByName( ntdll, "__wine_syscall_dispatcher" );
         __wine_syscall_dispatcher = *dispatcher_ptr;
-        if (!__wine_init_unix_call()) WINE_UNIX_CALL( 0, NULL );
+        __wine_init_unix_call();
         break;
     }
     return TRUE;
