@@ -33,11 +33,16 @@
 #include "windows.foundation.h"
 #define WIDL_using_Windows_UI
 #include "windows.ui.h"
+#define WIDL_using_Windows_UI_Core
+#include "windows.ui.core.h"
 #define WIDL_using_Windows_UI_ViewManagement
 #include "windows.ui.viewmanagement.h"
 
+extern IActivationFactory *accessibilitysettings_factory;
 extern IActivationFactory *uisettings_factory;
+extern IActivationFactory *uiviewsettings_factory;
 extern IActivationFactory *inputpane_factory;
+extern IActivationFactory *corewindow_factory;
 
 #define DEFINE_IINSPECTABLE_( pfx, iface_type, impl_type, impl_from, iface_mem, expr )             \
     static inline impl_type *impl_from( iface_type *iface )                                        \
