@@ -53,9 +53,6 @@ DECL_FUNCPTR( ANativeWindow_release );
  * OpenGL driver
  */
 
-extern pthread_mutex_t drawable_mutex;
-extern void update_gl_drawable( HWND hwnd );
-extern void destroy_gl_drawable( HWND hwnd );
 extern UINT ANDROID_OpenGLInit( UINT version, const struct opengl_funcs *opengl_funcs, const struct opengl_driver_funcs **driver_funcs );
 
 
@@ -97,8 +94,10 @@ extern UINT ANDROID_ShowWindow( HWND hwnd, INT cmd, RECT *rect, UINT swp );
 extern LRESULT ANDROID_WindowMessage( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp );
 extern BOOL ANDROID_WindowPosChanging( HWND hwnd, UINT swp_flags, BOOL shaped, const struct window_rects *rects );
 extern BOOL ANDROID_CreateWindowSurface( HWND hwnd, BOOL layered, const RECT *surface_rect, struct window_surface **surface );
-extern void ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, HWND owner_hint, UINT swp_flags, BOOL fullscreen,
+extern void ANDROID_WindowPosChanged( HWND hwnd, HWND insert_after, HWND owner_hint, UINT swp_flags,
                                       const struct window_rects *new_rects, struct window_surface *surface );
+extern ANativeWindow *get_client_window( HWND hwnd );
+extern BOOL has_client_surface( HWND hwnd );
 
 /* unixlib interface */
 
