@@ -334,31 +334,22 @@ static NTSTATUS WINAPI macdrv_app_icon(void *arg, ULONG size)
         icon_bits = LockResource(icon_res_data);
         if (icon_bits)
         {
-            static const BYTE png_magic[] = { 0x89, 0x50, 0x4e, 0x47 };
+            HICON icon;
 
             entry->width = width;
             entry->height = height;
-            entry->size = icon_dir->idEntries[i].dwBytesInRes;
 
-            if (!memcmp(icon_bits, png_magic, sizeof(png_magic)))
+            /* dwBytesInRes from the icon_dir entry is wrong in some apps; use
+               SizeofResource instead. */
+            icon = CreateIconFromResourceEx(icon_bits, SizeofResource(NULL, res_info),
+                                            TRUE, 0x00030000, width, height, 0);
+            if (icon)
             {
-                entry->png = (UINT_PTR)icon_bits;
-                entry->icon = 0;
+                entry->icon = HandleToUlong(icon);
                 count++;
             }
             else
-            {
-                HICON icon = CreateIconFromResourceEx(icon_bits, icon_dir->idEntries[i].dwBytesInRes,
-                                                      TRUE, 0x00030000, width, height, 0);
-                if (icon)
-                {
-                    entry->icon = HandleToUlong(icon);
-                    entry->png = 0;
-                    count++;
-                }
-                else
-                    WARN("failed to create icon %d from resource with ID %hd\n", i, icon_dir->idEntries[i].nID);
-            }
+                WARN("failed to create icon %d from resource with ID %hd\n", i, icon_dir->idEntries[i].nID);
         }
         else
             WARN("failed to lock RT_ICON resource %d with ID %hd\n", i, icon_dir->idEntries[i].nID);
