@@ -28,6 +28,5 @@ BOOL WINAPI DllMain( HINSTANCE instance, DWORD reason, void *reserved )
     if (reason != DLL_PROCESS_ATTACH) return TRUE;
 
     DisableThreadLibraryCalls( instance );
-    if (__wine_init_unix_call()) return FALSE;
-    return !WINE_UNIX_CALL( 0, NULL );
+    return !__wine_init_unix_call();
 }
