@@ -45,9 +45,11 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define FACTENGINE_E_NONOTIFICATIONCALLBACK	0x8ac70004
+#define FACTENGINE_E_INVALIDUSAGE		0x8ac70006
 #define FACTENGINE_E_INVALIDDATA		0x8ac70007
 #define FACTENGINE_E_INSTANCELIMITFAILTOPLAY	0x8ac70008
 #define FACTENGINE_E_INVALIDVARIABLEINDEX	0x8ac7000a
+#define FACTENGINE_E_INVALIDCUEINDEX		0x8ac7000c
 
 /* Type Declarations */
 
