@@ -458,6 +458,17 @@ static void parse_command_line( int argc, WCHAR *argv[] )
                 usage();
             } else
                 opts.sei.lpDirectory = argv[++i];
+
+            if (opts.sei.lpDirectory[0] == '\"')
+            {
+                int len = wcslen(opts.sei.lpDirectory);
+                if (len > 1 && opts.sei.lpDirectory[len - 1] == '\"')
+                {
+                    WCHAR* lpDirectory = wcsdup(opts.sei.lpDirectory);
+                    lpDirectory[len - 1] = 0;
+                    opts.sei.lpDirectory = lpDirectory + 1;
+                }
+            }
         }
         else if (is_option(argv[i], L"/b"))
             opts.creation_flags &= ~CREATE_NEW_CONSOLE;
