@@ -61,6 +61,8 @@ static WCHAR g_drv_keyW[256] = L"Software\\Wine\\Drivers\\";
 
 static UINT num_render_devs, num_capture_devs;
 static struct DeviceInfo *render_devs, *capture_devs;
+static UINT num_midi_devs;
+static WCHAR **midi_devs;
 
 static const struct
 {
@@ -178,6 +180,31 @@ static BOOL load_devices(IMMDeviceEnumerator *devenum, EDataFlow dataflow,
     return TRUE;
 }
 
+static BOOL load_midi_devices(UINT *ndevs, WCHAR ***out)
+{
+    UINT i;
+
+    *ndevs = midiOutGetNumDevs();
+
+    if(*ndevs > 0){
+        *out = malloc(sizeof(WCHAR *) * (*ndevs));
+        if(!*out)
+            return FALSE;
+
+        for(i = 0; i < *ndevs; ++i){
+            MIDIOUTCAPSW caps;
+            if (MMSYSERR_NOERROR != midiOutGetDevCapsW(i, &caps, sizeof(caps))){
+                (*out)[i] = NULL;
+                continue;
+            }
+            (*out)[i] = StrDupW(caps.szPname);
+        }
+    }else
+        *out = NULL;
+
+    return TRUE;
+}
+
 static BOOL get_driver_name(IMMDeviceEnumerator *devenum, PROPVARIANT *pv)
 {
     IMMDevice *device;
@@ -214,6 +241,7 @@ static void initAudioDlg (HWND hDlg)
     WCHAR display_str[256], format_str[256], sysdefault_str[256], disabled_str[64];
     IMMDeviceEnumerator *devenum;
     BOOL have_driver = FALSE;
+    WCHAR *reg_midi_dev;
     HRESULT hr;
     UINT i;
     LVCOLUMNW lvcol;
@@ -248,6 +276,8 @@ static void initAudioDlg (HWND hDlg)
         IMMDeviceEnumerator_Release(devenum);
     }
 
+    load_midi_devices(&num_midi_devs, &midi_devs);
+
     SendDlgItemMessageW(hDlg, IDC_AUDIOOUT_DEVICE, CB_ADDSTRING,
             0, (LPARAM)sysdefault_str);
     SendDlgItemMessageW(hDlg, IDC_AUDIOOUT_DEVICE, CB_SETCURSEL, 0, 0);
@@ -261,6 +291,9 @@ static void initAudioDlg (HWND hDlg)
     SendDlgItemMessageW(hDlg, IDC_VOICEIN_DEVICE, CB_ADDSTRING,
             0, (LPARAM)sysdefault_str);
     SendDlgItemMessageW(hDlg, IDC_VOICEIN_DEVICE, CB_SETCURSEL, 0, 0);
+    SendDlgItemMessageW(hDlg, IDC_MIDI_DEVICE, CB_ADDSTRING,
+            0, (LPARAM)sysdefault_str);
+    SendDlgItemMessageW(hDlg, IDC_MIDI_DEVICE, CB_SETCURSEL, 0, 0);
 
     i = 0;
     while (speaker_configs[i].text_id != 0) {
@@ -369,6 +402,21 @@ static void initAudioDlg (HWND hDlg)
     }else
         swprintf(display_str, ARRAY_SIZE(display_str), format_str, disabled_str);
 
+    reg_midi_dev = get_reg_key(HKEY_CURRENT_USER,
+            L"Software\\Microsoft\\Windows\\CurrentVersion\\Multimedia\\MIDIMap", L"szPname", NULL);
+    for(i = 0; i < num_midi_devs; ++i){
+        if(!midi_devs[i])
+            continue;
+
+        SendDlgItemMessageW(hDlg, IDC_MIDI_DEVICE, CB_ADDSTRING,
+                0, (LPARAM)midi_devs[i]);
+        SendDlgItemMessageW(hDlg, IDC_MIDI_DEVICE, CB_SETITEMDATA,
+                i + 1, (LPARAM)midi_devs[i]);
+        if(reg_midi_dev && !wcscmp(midi_devs[i], reg_midi_dev))
+            SendDlgItemMessageW(hDlg, IDC_MIDI_DEVICE, CB_SETCURSEL, i + 1, 0);
+    }
+    free(reg_midi_dev);
+
     SetDlgItemTextW(hDlg, IDC_AUDIO_DRIVER, display_str);
 }
 
@@ -388,6 +436,23 @@ static void set_reg_device(HWND hDlg, int dlgitem, const WCHAR *key_name)
         set_reg_key(HKEY_CURRENT_USER, g_drv_keyW, key_name, info->id);
 }
 
+static void set_reg_midi_device(HWND hDlg, int dlgitem)
+{
+    UINT idx;
+    WCHAR *id;
+
+    idx = SendDlgItemMessageW(hDlg, dlgitem, CB_GETCURSEL, 0, 0);
+
+    id = (WCHAR *)SendDlgItemMessageW(hDlg, dlgitem, CB_GETITEMDATA, idx, 0);
+
+    if(!id || id == (void*)CB_ERR)
+        set_reg_key(HKEY_CURRENT_USER,
+                L"Software\\Microsoft\\Windows\\CurrentVersion\\Multimedia\\MIDIMap", L"szPname", NULL);
+    else
+        set_reg_key(HKEY_CURRENT_USER,
+                L"Software\\Microsoft\\Windows\\CurrentVersion\\Multimedia\\MIDIMap", L"szPname", id);
+}
+
 static void test_sound(void)
 {
     if(!PlaySoundW(MAKEINTRESOURCEW(IDW_TESTSOUND), NULL, SND_RESOURCE | SND_ASYNC)){
@@ -501,6 +566,12 @@ AudioDlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
                   SendMessageW(GetParent(hDlg), PSM_CHANGED, 0, 0);
               }
               break;
+          case IDC_MIDI_DEVICE:
+              if(HIWORD(wParam) == CBN_SELCHANGE){
+                  set_reg_midi_device(hDlg, IDC_MIDI_DEVICE);
+                  SendMessageW(GetParent(hDlg), PSM_CHANGED, 0, 0);
+              }
+              break;
           case IDC_SPEAKERCONFIG_SPEAKERS:
               if(HIWORD(wParam) == CBN_SELCHANGE){
                   UINT dev, idx;
