@@ -30,11 +30,9 @@
 #define WIN32_LEAN_AND_MEAN
 
 #include <assert.h>
-#include <stdio.h>
 #include <limits.h>
 #include <windows.h>
 #include <winreg.h>
-#include <wine/unicode.h>
 #include <wine/debug.h>
 #include <wine/list.h>
 
@@ -62,14 +60,12 @@ void set_window_title(HWND dialog)
     if (current_app)
     {
         WCHAR apptitle[256];
-        LoadStringW (GetModuleHandleW(NULL), IDS_WINECFG_TITLE_APP, apptitle,
-            sizeof(apptitle)/sizeof(apptitle[0]));
-        wsprintfW (newtitle, apptitle, current_app);
+        LoadStringW(GetModuleHandleW(NULL), IDS_WINECFG_TITLE_APP, apptitle, ARRAY_SIZE(apptitle));
+        swprintf(newtitle, ARRAY_SIZE(newtitle), apptitle, current_app);
     }
     else
     {
-        LoadStringW (GetModuleHandleW(NULL), IDS_WINECFG_TITLE, newtitle,
-            sizeof(newtitle)/sizeof(newtitle[0]));
+        LoadStringW(GetModuleHandleW(NULL), IDS_WINECFG_TITLE, newtitle, ARRAY_SIZE(newtitle));
     }
 
     WINE_TRACE("setting title to %s\n", wine_dbgstr_w (newtitle));
@@ -83,11 +79,11 @@ WCHAR* load_string (UINT id)
     int len;
     WCHAR* newStr;
 
-    LoadStringW (GetModuleHandleW(NULL), id, buf, sizeof(buf)/sizeof(buf[0]));
+    LoadStringW(GetModuleHandleW(NULL), id, buf, ARRAY_SIZE(buf));
 
-    len = lstrlenW (buf);
-    newStr = HeapAlloc (GetProcessHeap(), 0, (len + 1) * sizeof (WCHAR));
-    memcpy (newStr, buf, len * sizeof (WCHAR));
+    len = wcslen(buf);
+    newStr = malloc((len + 1) * sizeof(WCHAR));
+    memcpy(newStr, buf, len * sizeof(WCHAR));
     newStr[len] = 0;
     return newStr;
 }
@@ -119,11 +115,11 @@ static WCHAR *get_config_key (HKEY root, const WCHAR *subkey, const WCHAR *name,
         if (res == ERROR_FILE_NOT_FOUND)
         {
             WINE_TRACE("Section key not present - using default\n");
-            return def ? strdupW(def) : NULL;
+            return wcsdup(def);
         }
         else
         {
-            WINE_ERR("RegOpenKey failed on wine config key (res=%d)\n", res);
+            WINE_ERR("RegOpenKey failed on wine config key (res=%ld)\n", res);
         }
         goto end;
     }
@@ -132,15 +128,15 @@ static WCHAR *get_config_key (HKEY root, const WCHAR *subkey, const WCHAR *name,
     if (res == ERROR_FILE_NOT_FOUND)
     {
         WINE_TRACE("Value not present - using default\n");
-        buffer = def ? strdupW(def) : NULL;
+        buffer = wcsdup(def);
 	goto end;
     } else if (res != ERROR_SUCCESS)
     {
-        WINE_ERR("Couldn't query value's length (res=%d)\n", res);
+        WINE_ERR("Couldn't query value's length (res=%ld)\n", res);
         goto end;
     }
 
-    buffer = HeapAlloc(GetProcessHeap(), 0, len + sizeof(WCHAR));
+    buffer = malloc(len + sizeof(WCHAR));
 
     RegQueryValueExW(hSubKey, name, NULL, NULL, (LPBYTE) buffer, &len);
 
@@ -167,7 +163,7 @@ static int set_config_key(HKEY root, const WCHAR *subkey, REGSAM access, const W
     DWORD res = 1;
     HKEY key = NULL;
 
-    WINE_TRACE("subkey=%s: name=%s, value=%p, type=%d\n", wine_dbgstr_w(subkey),
+    WINE_TRACE("subkey=%s: name=%s, value=%p, type=%ld\n", wine_dbgstr_w(subkey),
                wine_dbgstr_w(name), value, type);
 
     assert( subkey != NULL );
@@ -192,7 +188,7 @@ static int set_config_key(HKEY root, const WCHAR *subkey, REGSAM access, const W
 end:
     if (key && key != root) RegCloseKey(key);
     if (res != 0)
-        WINE_ERR("Unable to set configuration key %s in section %s, res=%d\n",
+        WINE_ERR("Unable to set configuration key %s in section %s, res=%ld\n",
                  wine_dbgstr_w(name), wine_dbgstr_w(subkey), res);
     return res;
 }
@@ -217,7 +213,7 @@ struct setting
     HKEY root;    /* the key on which path is rooted */
     WCHAR *path;   /* path in the registry rooted at root  */
     WCHAR *name;   /* name of the registry value. if null, this means delete the key  */
-    WCHAR *value;  /* contents of the registry value. if null, this means delete the value  */
+    void  *value;  /* contents of the registry value. if null, this means delete the value  */
     DWORD type;   /* type of registry value. REG_SZ or REG_DWORD for now */
 };
 
@@ -231,13 +227,13 @@ static void free_setting(struct setting *setting)
     WINE_TRACE("destroying %p: %s\n", setting,
                wine_dbgstr_w(setting->path));
 
-    HeapFree(GetProcessHeap(), 0, setting->path);
-    HeapFree(GetProcessHeap(), 0, setting->name);
-    HeapFree(GetProcessHeap(), 0, setting->value);
+    free(setting->path);
+    free(setting->name);
+    free(setting->value);
 
     list_remove(&setting->entry);
 
-    HeapFree(GetProcessHeap(), 0, setting);
+    free(setting);
 }
 
 /**
@@ -246,9 +242,9 @@ static void free_setting(struct setting *setting)
  * default will be used.
  *
  * If already in the list, the contents as given there will be
- * returned. You are expected to HeapFree the result.
+ * returned. You are expected to free the result.
  */
-WCHAR *get_reg_keyW(HKEY root, const WCHAR *path, const WCHAR *name, const WCHAR *def)
+WCHAR *get_reg_key(HKEY root, const WCHAR *path, const WCHAR *name, const WCHAR *def)
 {
     struct list *cursor;
     struct setting *s;
@@ -270,7 +266,7 @@ WCHAR *get_reg_keyW(HKEY root, const WCHAR *path, const WCHAR *name, const WCHAR
         WINE_TRACE("found %s:%s in settings list, returning %s\n",
                    wine_dbgstr_w(path), wine_dbgstr_w(name),
                    wine_dbgstr_w(s->value));
-        return s->value ? strdupW(s->value) : NULL;
+        return wcsdup(s->value);
     }
 
     /* no, so get from the registry */
@@ -281,43 +277,6 @@ WCHAR *get_reg_keyW(HKEY root, const WCHAR *path, const WCHAR *name, const WCHAR
     return val;
 }
 
-char *get_reg_key(HKEY root, const char *path, const char *name, const char *def)
-{
-    WCHAR *wpath, *wname, *wdef = NULL, *wRet = NULL;
-    char *szRet = NULL;
-    int len;
-
-    WINE_TRACE("path=%s, name=%s, def=%s\n", path, name, def);
-
-    wpath = HeapAlloc(GetProcessHeap(), 0, (strlen(path)+1)*sizeof(WCHAR));
-    wname = HeapAlloc(GetProcessHeap(), 0, (strlen(name)+1)*sizeof(WCHAR));
-
-    MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, strlen(path)+1);
-    MultiByteToWideChar(CP_ACP, 0, name, -1, wname, strlen(name)+1);
-
-    if (def)
-    {
-        wdef = HeapAlloc(GetProcessHeap(), 0, (strlen(def)+1)*sizeof(WCHAR));
-        MultiByteToWideChar(CP_ACP, 0, def, -1, wdef, strlen(def)+1);
-    }
-
-    wRet = get_reg_keyW(root, wpath, wname, wdef);
-
-    len = WideCharToMultiByte(CP_ACP, 0, wRet, -1, NULL, 0, NULL, NULL);
-    if (len)
-    {
-        szRet = HeapAlloc(GetProcessHeap(), 0, len);
-        WideCharToMultiByte(CP_ACP, 0, wRet, -1, szRet, len, NULL, NULL);
-    }
-
-    HeapFree(GetProcessHeap(), 0, wpath);
-    HeapFree(GetProcessHeap(), 0, wname);
-    HeapFree(GetProcessHeap(), 0, wdef);
-    HeapFree(GetProcessHeap(), 0, wRet);
-
-    return szRet;
-}
-
 /**
  * Used to set a registry key.
  *
@@ -356,18 +315,18 @@ static void set_reg_key_ex(HKEY root, const WCHAR *path, const WCHAR *name, cons
         if (!s->name && !name) return;
 
         /* do we want to undelete this key? */
-        if (!s->name && name) s->name = strdupW(name);
+        if (!s->name && name) s->name = wcsdup(name);
 
         /* yes, we have already set it, so just replace the content and return  */
-        HeapFree(GetProcessHeap(), 0, s->value);
+        free(s->value);
         s->type = type;
         switch (type)
         {
             case REG_SZ:
-                s->value = value ? strdupW(value) : NULL;
+                s->value = wcsdup(value);
                 break;
             case REG_DWORD:
-                s->value = HeapAlloc(GetProcessHeap(), 0, sizeof(DWORD));
+                s->value = malloc(sizeof(DWORD));
                 memcpy( s->value, value, sizeof(DWORD) );
                 break;
         }
@@ -386,18 +345,18 @@ static void set_reg_key_ex(HKEY root, const WCHAR *path, const WCHAR *name, cons
     }
 
     /* otherwise add a new setting for it  */
-    s = HeapAlloc(GetProcessHeap(), 0, sizeof(struct setting));
+    s = malloc(sizeof(struct setting));
     s->root  = root;
-    s->path  = strdupW(path);
-    s->name  = name  ? strdupW(name)  : NULL;
+    s->path  = wcsdup(path);
+    s->name  = wcsdup(name);
     s->type  = type;
     switch (type)
     {
         case REG_SZ:
-            s->value = value ? strdupW(value) : NULL;
+            s->value = wcsdup(value);
             break;
         case REG_DWORD:
-            s->value = HeapAlloc(GetProcessHeap(), 0, sizeof(DWORD));
+            s->value = malloc(sizeof(DWORD));
             memcpy( s->value, value, sizeof(DWORD) );
             break;
     }
@@ -405,54 +364,12 @@ static void set_reg_key_ex(HKEY root, const WCHAR *path, const WCHAR *name, cons
     list_add_tail(&settings, &s->entry);
 }
 
-void set_reg_key(HKEY root, const char *path, const char *name, const char *value)
-{
-    WCHAR *wpath, *wname = NULL, *wvalue = NULL;
-
-    wpath = HeapAlloc(GetProcessHeap(), 0, (strlen(path)+1)*sizeof(WCHAR));
-    MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, strlen(path)+1);
-
-    if (name)
-    {
-        wname = HeapAlloc(GetProcessHeap(), 0, (strlen(name)+1)*sizeof(WCHAR));
-        MultiByteToWideChar(CP_ACP, 0, name, -1, wname, strlen(name)+1);
-    }
-
-    if (value)
-    {
-        wvalue = HeapAlloc(GetProcessHeap(), 0, (strlen(value)+1)*sizeof(WCHAR));
-        MultiByteToWideChar(CP_ACP, 0, value, -1, wvalue, strlen(value)+1);
-    }
-
-    set_reg_key_ex(root, wpath, wname, wvalue, REG_SZ);
-
-    HeapFree(GetProcessHeap(), 0, wpath);
-    HeapFree(GetProcessHeap(), 0, wname);
-    HeapFree(GetProcessHeap(), 0, wvalue);
-}
-
-void set_reg_key_dword(HKEY root, const char *path, const char *name, DWORD value)
-{
-    WCHAR *wpath, *wname;
-
-    wpath = HeapAlloc(GetProcessHeap(), 0, (strlen(path)+1)*sizeof(WCHAR));
-    wname = HeapAlloc(GetProcessHeap(), 0, (strlen(name)+1)*sizeof(WCHAR));
-
-    MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, strlen(path)+1);
-    MultiByteToWideChar(CP_ACP, 0, name, -1, wname, strlen(name)+1);
-
-    set_reg_key_ex(root, wpath, wname, &value, REG_DWORD);
-
-    HeapFree(GetProcessHeap(), 0, wpath);
-    HeapFree(GetProcessHeap(), 0, wname);
-}
-
-void set_reg_keyW(HKEY root, const WCHAR *path, const WCHAR *name, const WCHAR *value)
+void set_reg_key(HKEY root, const WCHAR *path, const WCHAR *name, const WCHAR *value)
 {
     set_reg_key_ex(root, path, name, value, REG_SZ);
 }
 
-void set_reg_key_dwordW(HKEY root, const WCHAR *path, const WCHAR *name, DWORD value)
+void set_reg_key_dword(HKEY root, const WCHAR *path, const WCHAR *name, DWORD value)
 {
     set_reg_key_ex(root, path, name, &value, REG_DWORD);
 }
@@ -461,10 +378,10 @@ void set_reg_key_dwordW(HKEY root, const WCHAR *path, const WCHAR *name, DWORD v
  * enumerates the value names at the given path, taking into account
  * the changes in the settings list.
  *
- * you are expected to HeapFree each element of the array, which is null
+ * you are expected to free each element of the array, which is null
  * terminated, as well as the array itself.
  */
-static WCHAR **enumerate_valuesW(HKEY root, WCHAR *path)
+WCHAR **enumerate_values(HKEY root, const WCHAR *path)
 {
     HKEY key;
     DWORD res, i = 0, valueslen = 0;
@@ -477,7 +394,7 @@ static WCHAR **enumerate_valuesW(HKEY root, WCHAR *path)
         while (TRUE)
         {
             WCHAR name[1024];
-            DWORD namesize = sizeof(name)/sizeof(name[0]);
+            DWORD namesize = ARRAY_SIZE(name);
             BOOL removed = FALSE;
 
             /* find out the needed size, allocate a buffer, read the value  */
@@ -508,17 +425,16 @@ static WCHAR **enumerate_valuesW(HKEY root, WCHAR *path)
             }
 
             /* grow the array if necessary, add buffer to it, iterate  */
-            if (values) values = HeapReAlloc(GetProcessHeap(), 0, values, sizeof(WCHAR*) * (valueslen + 1));
-            else values = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR*));
+            values = realloc(values, sizeof(WCHAR*) * (valueslen + 1));
 
-            values[valueslen++] = strdupW(name);
-            WINE_TRACE("valueslen is now %d\n", valueslen);
+            values[valueslen++] = wcsdup(name);
+            WINE_TRACE("valueslen is now %ld\n", valueslen);
             i++;
         }
     }
     else
     {
-        WINE_WARN("failed opening registry key %s, res=0x%x\n",
+        WINE_WARN("failed opening registry key %s, res=0x%lx\n",
                   wine_dbgstr_w(path), res);
     }
 
@@ -548,16 +464,15 @@ static WCHAR **enumerate_valuesW(HKEY root, WCHAR *path)
         WINE_TRACE("%s in list but not registry\n", wine_dbgstr_w(setting->name));
 
         /* otherwise it's been set by the user but isn't in the registry */
-        if (values) values = HeapReAlloc(GetProcessHeap(), 0, values, sizeof(WCHAR*) * (valueslen + 1));
-        else values = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR*));
+        values = realloc(values, sizeof(WCHAR*) * (valueslen + 1));
 
-        values[valueslen++] = strdupW(setting->name);
+        values[valueslen++] = wcsdup(setting->name);
     }
 
     WINE_TRACE("adding null terminator\n");
     if (values)
     {
-        values = HeapReAlloc(GetProcessHeap(), 0, values, sizeof(WCHAR*) * (valueslen + 1));
+        values = realloc(values, sizeof(WCHAR*) * (valueslen + 1));
         values[valueslen] = NULL;
     }
 
@@ -566,78 +481,29 @@ static WCHAR **enumerate_valuesW(HKEY root, WCHAR *path)
     return values;
 }
 
-char **enumerate_values(HKEY root, char *path)
-{
-    WCHAR *wpath;
-    WCHAR **wret;
-    char **ret=NULL;
-    int i=0, len=0, size;
-
-    wpath = HeapAlloc(GetProcessHeap(), 0, (strlen(path)+1)*sizeof(WCHAR));
-    MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, strlen(path)+1);
-
-    wret = enumerate_valuesW(root, wpath);
-
-    if (wret)
-    {
-        for(len=0; wret[len]; len++);
-        ret = HeapAlloc(GetProcessHeap(), 0, (len+1)*sizeof(char*));
-
-        /* convert WCHAR ** to char ** and HeapFree each WCHAR * element on our way */
-        for (i=0; i<len; i++)
-        {
-            size = WideCharToMultiByte(CP_ACP, 0, wret[i], -1, NULL, 0, NULL, NULL);
-            if(size)
-            {
-                ret[i] = HeapAlloc(GetProcessHeap(), 0, size);
-                WideCharToMultiByte(CP_ACP, 0, wret[i], -1, ret[i], size, NULL, NULL);
-                HeapFree(GetProcessHeap(), 0, wret[i]);
-            }
-        }
-        ret[len] = NULL;
-    }
-
-    HeapFree(GetProcessHeap(), 0, wpath);
-    HeapFree(GetProcessHeap(), 0, wret);
-
-    return ret;
-}
-
 /**
  * returns true if the given key/value pair exists in the registry or
  * has been written to.
  */
-BOOL reg_key_exists(HKEY root, const char *path, const char *name)
+BOOL reg_key_exists(HKEY root, const WCHAR *path, const WCHAR *name)
 {
-    char *val = get_reg_key(root, path, name, NULL);
+    WCHAR *val = get_reg_key(root, path, name, NULL);
 
-    if (val)
-    {
-        HeapFree(GetProcessHeap(), 0, val);
-        return TRUE;
-    }
-
-    return FALSE;
+    free(val);
+    return val != NULL;
 }
 
 static void process_setting(struct setting *s)
 {
-    static const WCHAR softwareW[] = {'S','o','f','t','w','a','r','e','\\'};
     HKEY key;
     BOOL needs_wow64 = (is_win64 && s->root == HKEY_LOCAL_MACHINE && s->path &&
-                        !strncmpiW( s->path, softwareW, sizeof(softwareW)/sizeof(WCHAR) ));
+                        !wcsnicmp(s->path, L"Software\\", wcslen(L"Software\\")));
 
     if (s->value)
     {
-	WINE_TRACE("Setting %s:%s to '%s'\n", wine_dbgstr_w(s->path),
-                   wine_dbgstr_w(s->name), wine_dbgstr_w(s->value));
         set_config_key(s->root, s->path, MAXIMUM_ALLOWED, s->name, s->value, s->type);
         if (needs_wow64)
-        {
-            WINE_TRACE("Setting 32-bit %s:%s to '%s'\n", wine_dbgstr_w(s->path),
-                       wine_dbgstr_w(s->name), wine_dbgstr_w(s->value));
             set_config_key(s->root, s->path, MAXIMUM_ALLOWED | KEY_WOW64_32KEY, s->name, s->value, s->type);
-        }
     }
     else
     {
@@ -689,38 +555,17 @@ void apply(void)
 WCHAR* current_app = NULL; /* the app we are currently editing, or NULL if editing global */
 
 /* returns a registry key path suitable for passing to addTransaction  */
-char *keypath(const char *section)
+WCHAR *keypath(const WCHAR *section)
 {
-    static char *result = NULL;
-
-    HeapFree(GetProcessHeap(), 0, result);
-
-    if (current_app)
-    {
-        result = HeapAlloc(GetProcessHeap(), 0, strlen("AppDefaults\\") + lstrlenW(current_app)*2 + 2 /* \\ */ + strlen(section) + 1 /* terminator */);
-        wsprintfA(result, "AppDefaults\\%ls", current_app);
-        if (section[0]) sprintf( result + strlen(result), "\\%s", section );
-    }
-    else
-    {
-        result = strdupA(section);
-    }
-
-    return result;
-}
-
-WCHAR *keypathW(const WCHAR *section)
-{
-    static const WCHAR appdefaultsW[] = {'A','p','p','D','e','f','a','u','l','t','s','\\',0};
     static WCHAR *result = NULL;
 
-    HeapFree(GetProcessHeap(), 0, result);
+    free(result);
 
     if (current_app)
     {
-        DWORD len = sizeof(appdefaultsW) + (lstrlenW(current_app) + lstrlenW(section) + 1) * sizeof(WCHAR);
-        result = HeapAlloc(GetProcessHeap(), 0, len );
-        lstrcpyW( result, appdefaultsW );
+        DWORD len = sizeof(L"AppDefaults\\") + (lstrlenW(current_app) + lstrlenW(section) + 1) * sizeof(WCHAR);
+        result = malloc(len);
+        lstrcpyW( result, L"AppDefaults\\" );
         lstrcatW( result, current_app );
         if (section[0])
         {
@@ -731,7 +576,7 @@ WCHAR *keypathW(const WCHAR *section)
     }
     else
     {
-        result = strdupW(section);
+        result = wcsdup(section);
     }
 
     return result;
@@ -753,10 +598,10 @@ void PRINTERROR(void)
 
 BOOL initialize(HINSTANCE hInstance)
 {
-    DWORD res = RegCreateKeyA(HKEY_CURRENT_USER, WINE_KEY_ROOT, &config_key);
+    DWORD res = RegCreateKeyW(HKEY_CURRENT_USER, WINE_KEY_ROOT, &config_key);
 
     if (res != ERROR_SUCCESS) {
-	WINE_ERR("RegOpenKey failed on wine config key (%d)\n", res);
+	WINE_ERR("RegOpenKey failed on wine config key (%ld)\n", res);
         return TRUE;
     }
 
