@@ -30,7 +30,6 @@
 #include "winbase.h"
 #include "winedump.h"
 #include "cvconst.h"
-#include "wine/mscvpdb.h"
 
 #define PSTRING(adr, ofs) \
     ((const struct p_string*)((const char*)(adr) + (ofs)))
@@ -53,11 +52,12 @@ struct full_value
     } v;
 };
 
-static int full_numeric_leaf(struct full_value* fv, const unsigned short int* leaf)
+static int full_numeric_leaf(struct full_value *fv, const unsigned char *leaf)
 {
-    unsigned short int type = *leaf++;
+    unsigned short int type = *(const unsigned short *)leaf;
     int length = 2;
 
+    leaf += length;
     fv->type = fv_integer;
     if (type < LF_NUMERIC)
     {
@@ -79,7 +79,7 @@ static int full_numeric_leaf(struct full_value* fv, const unsigned short int* le
 
         case LF_USHORT:
             length += 2;
-            fv->v.i = *leaf;
+            fv->v.i = *(const unsigned short*)leaf;
             break;
 
         case LF_LONG:
@@ -185,7 +185,7 @@ static const char* full_value_string(const struct full_value* fv)
     return tmp;
 }
 
-static int numeric_leaf(int* value, const unsigned short int* leaf)
+static int numeric_leaf(int* value, const unsigned char* leaf)
 {
     struct full_value fv;
     int len = full_numeric_leaf(&fv, leaf);
@@ -462,7 +462,7 @@ static const char* get_callconv(unsigned cc)
     case CV_CALL_RESERVED:      callconv = "reserved"; break;
     default:
         {
-            static char tmp[16];
+            static char tmp[20];
             sprintf(tmp, "callconv=%x", cc);
             callconv = tmp;
         }
@@ -519,24 +519,24 @@ static void do_field(const unsigned char* start, const unsigned char* end)
         switch (fieldtype->generic.id)
         {
         case LF_ENUMERATE_V1:
-            leaf_len = full_numeric_leaf(&full_value, &fieldtype->enumerate_v1.value);
-            pstr = PSTRING(&fieldtype->enumerate_v1.value, leaf_len);
+            leaf_len = full_numeric_leaf(&full_value, fieldtype->enumerate_v1.data);
+            pstr = (const struct p_string*)&fieldtype->enumerate_v1.data[leaf_len];
             printf("\t\tEnumerate V1: '%s' value:%s\n",
                    p_string(pstr), full_value_string(&full_value));
             ptr += 2 + 2 + leaf_len + 1 + pstr->namelen;
             break;
 
         case LF_ENUMERATE_V3:
-            leaf_len = full_numeric_leaf(&full_value, &fieldtype->enumerate_v3.value);
-            cstr = (const char*)&fieldtype->enumerate_v3.value + leaf_len;
+            leaf_len = full_numeric_leaf(&full_value, fieldtype->enumerate_v3.data);
+            cstr = (const char*)&fieldtype->enumerate_v3.data[leaf_len];
             printf("\t\tEnumerate V3: '%s' value:%s\n",
                    cstr, full_value_string(&full_value));
             ptr += 2 + 2 + leaf_len + strlen(cstr) + 1;
             break;
 
         case LF_MEMBER_V1:
-            leaf_len = numeric_leaf(&value, &fieldtype->member_v1.offset);
-            pstr = PSTRING(&fieldtype->member_v1.offset, leaf_len);
+            leaf_len = numeric_leaf(&value, fieldtype->member_v1.data);
+            pstr = (const struct p_string *)&fieldtype->member_v1.data[leaf_len];
             printf("\t\tMember V1: '%s' type:%x attr:%s @%d\n",
                    p_string(pstr), fieldtype->member_v1.type,
                    get_attr(fieldtype->member_v1.attribute), value);
@@ -544,8 +544,8 @@ static void do_field(const unsigned char* start, const unsigned char* end)
             break;
 
         case LF_MEMBER_V2:
-            leaf_len = numeric_leaf(&value, &fieldtype->member_v2.offset);
-            pstr = PSTRING(&fieldtype->member_v2.offset, leaf_len);
+            leaf_len = numeric_leaf(&value, fieldtype->member_v2.data);
+            pstr = (const struct p_string *)&fieldtype->member_v2.data[leaf_len];
             printf("\t\tMember V2: '%s' type:%x attr:%s @%d\n",
                    p_string(pstr), fieldtype->member_v2.type,
                    get_attr(fieldtype->member_v2.attribute), value);
@@ -553,10 +553,10 @@ static void do_field(const unsigned char* start, const unsigned char* end)
             break;
 
         case LF_MEMBER_V3:
-            leaf_len = numeric_leaf(&value, &fieldtype->member_v3.offset);
-            cstr = (const char*)&fieldtype->member_v3.offset + leaf_len;
+            leaf_len = numeric_leaf(&value, fieldtype->member_v3.data);
+            cstr = (const char*)&fieldtype->member_v3.data[leaf_len];
             printf("\t\tMember V3: '%s' type:%x attr:%s @%d\n",
-                   cstr, fieldtype->member_v3.type, 
+                   cstr, fieldtype->member_v3.type,
                    get_attr(fieldtype->member_v3.attribute), value);
             ptr += 2 + 2 + 4 + leaf_len + strlen(cstr) + 1;
             break;
@@ -676,61 +676,61 @@ static void do_field(const unsigned char* start, const unsigned char* end)
             printf("\t\tFriend function V1: '%s' type:%x\n",
                    p_string(&fieldtype->friendfcn_v1.p_name),
                    fieldtype->friendfcn_v1.type);
-            ptr += 2 + 2 + (1 + fieldtype->stmember_v2.p_name.namelen);
+            ptr += 2 + 2 + (1 + fieldtype->friendfcn_v1.p_name.namelen);
             break;
 
         case LF_FRIENDFCN_V2:
             printf("\t\tFriend function V2: '%s' type:%x\n",
                    p_string(&fieldtype->friendfcn_v2.p_name),
                    fieldtype->friendfcn_v2.type);
-            ptr += 2 + 2 + 4 + (1 + fieldtype->stmember_v2.p_name.namelen);
+            ptr += 2 + 2 + 4 + (1 + fieldtype->friendfcn_v2.p_name.namelen);
             break;
 
         case LF_FRIENDFCN_V3:
             printf("\t\tFriend function V3: '%s' type:%x\n",
                    fieldtype->friendfcn_v3.name,
                    fieldtype->friendfcn_v3.type);
-            ptr += 2 + 2 + 4 + (strlen(fieldtype->stmember_v3.name) + 1);
+            ptr += 2 + 2 + 4 + (strlen(fieldtype->friendfcn_v3.name) + 1);
             break;
 
         case LF_BCLASS_V1:
-            leaf_len = numeric_leaf(&value, &fieldtype->bclass_v1.offset);
+            leaf_len = numeric_leaf(&value, fieldtype->bclass_v1.data);
             printf("\t\tBase class V1: type:%x attr:%s @%d\n",
-                   fieldtype->bclass_v1.type, 
+                   fieldtype->bclass_v1.type,
                    get_attr(fieldtype->bclass_v1.attribute), value);
             ptr += 2 + 2 + 2 + leaf_len;
             break;
 
         case LF_BCLASS_V2:
-            leaf_len = numeric_leaf(&value, &fieldtype->bclass_v2.offset);
+            leaf_len = numeric_leaf(&value, fieldtype->bclass_v2.data);
             printf("\t\tBase class V2: type:%x attr:%s @%d\n",
-                   fieldtype->bclass_v2.type, 
+                   fieldtype->bclass_v2.type,
                    get_attr(fieldtype->bclass_v2.attribute), value);
             ptr += 2 + 2 + 4 + leaf_len;
             break;
 
         case LF_VBCLASS_V1:
         case LF_IVBCLASS_V1:
-            leaf_len = numeric_leaf(&value, &fieldtype->vbclass_v1.vbpoff);
+            leaf_len = numeric_leaf(&value, fieldtype->vbclass_v1.data);
             printf("\t\t%sirtual base class V1: type:%x (ptr:%x) attr:%s vbpoff:%d ",
-                   (fieldtype->generic.id == LF_VBCLASS_V2) ? "V" : "Indirect v",
+                   (fieldtype->generic.id == LF_VBCLASS_V1) ? "V" : "Indirect v",
                    fieldtype->vbclass_v1.btype, fieldtype->vbclass_v1.vbtype,
                    get_attr(fieldtype->vbclass_v1.attribute), value);
             ptr += 2 + 2 + 2 + 2 + leaf_len;
-            leaf_len = numeric_leaf(&value, (const unsigned short*)ptr);
+            leaf_len = numeric_leaf(&value, ptr);
             printf("vboff:%d\n", value);
             ptr += leaf_len;
             break;
 
         case LF_VBCLASS_V2:
         case LF_IVBCLASS_V2:
-            leaf_len = numeric_leaf(&value, &fieldtype->vbclass_v1.vbpoff);
+            leaf_len = numeric_leaf(&value, fieldtype->vbclass_v2.data);
             printf("\t\t%sirtual base class V2: type:%x (ptr:%x) attr:%s vbpoff:%d ",
                    (fieldtype->generic.id == LF_VBCLASS_V2) ? "V" : "Indirect v",
                    fieldtype->vbclass_v2.btype, fieldtype->vbclass_v2.vbtype,
                    get_attr(fieldtype->vbclass_v2.attribute), value);
             ptr += 2 + 2 + 4 + 4 + leaf_len;
-            leaf_len = numeric_leaf(&value, (const unsigned short*)ptr);
+            leaf_len = numeric_leaf(&value, ptr);
             printf("vboff:%d\n", value);
             ptr += leaf_len;
             break;
@@ -827,20 +827,20 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
                curr_type, type->pointer_v2.datatype);
         break;
     case LF_ARRAY_V1:
-        leaf_len = numeric_leaf(&value, &type->array_v1.arrlen);
+        leaf_len = numeric_leaf(&value, type->array_v1.data);
         printf("\t%x => Array V1-'%s'[%u type:%x] type:%x\n",
-               curr_type, p_string(PSTRING(&type->array_v1.arrlen, leaf_len)),
+               curr_type, p_string((const struct p_string *)&type->array_v1.data[leaf_len]),
                value, type->array_v1.idxtype, type->array_v1.elemtype);
         break;
     case LF_ARRAY_V2:
-        leaf_len = numeric_leaf(&value, &type->array_v2.arrlen);
+        leaf_len = numeric_leaf(&value, type->array_v2.data);
         printf("\t%x => Array V2-'%s'[%u type:%x] type:%x\n",
-               curr_type, p_string(PSTRING(&type->array_v2.arrlen, leaf_len)),
+               curr_type, p_string((const struct p_string *)&type->array_v2.data[leaf_len]),
                value, type->array_v2.idxtype, type->array_v2.elemtype);
         break;
     case LF_ARRAY_V3:
-        leaf_len = numeric_leaf(&value, &type->array_v3.arrlen);
-        str = (const char*)&type->array_v3.arrlen + leaf_len;
+        leaf_len = numeric_leaf(&value, type->array_v3.data);
+        str = (const char*)&type->array_v3.data[leaf_len];
         printf("\t%x => Array V3-'%s'[%u type:%x] type:%x\n",
                curr_type, str, value,
                type->array_v3.idxtype, type->array_v3.elemtype);
@@ -871,10 +871,10 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
 
     case LF_STRUCTURE_V1:
     case LF_CLASS_V1:
-        leaf_len = numeric_leaf(&value, &type->struct_v1.structlen);
+        leaf_len = numeric_leaf(&value, type->struct_v1.data);
         printf("\t%x => %s V1 '%s' elts:%u property:%s fieldlist-type:%x derived-type:%x vshape:%x size:%u\n",
                curr_type, type->generic.id == LF_CLASS_V1 ? "Class" : "Struct",
-               p_string(PSTRING(&type->struct_v1.structlen, leaf_len)),
+               p_string((const struct p_string *)&type->struct_v1.data[leaf_len]),
                type->struct_v1.n_element, get_property(type->struct_v1.property),
                type->struct_v1.fieldlist, type->struct_v1.derived,
                type->struct_v1.vshape, value);
@@ -882,11 +882,11 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
 
     case LF_STRUCTURE_V2:
     case LF_CLASS_V2:
-        leaf_len = numeric_leaf(&value, &type->struct_v2.structlen);
+        leaf_len = numeric_leaf(&value, type->struct_v2.data);
         printf("\t%x => %s V2 '%s' elts:%u property:%s\n"
                "                fieldlist-type:%x derived-type:%x vshape:%x size:%u\n",
                curr_type, type->generic.id == LF_CLASS_V2 ? "Class" : "Struct",
-               p_string(PSTRING(&type->struct_v2.structlen, leaf_len)),
+               p_string((const struct p_string *)&type->struct_v2.data[leaf_len]),
                type->struct_v2.n_element, get_property(type->struct_v2.property),
                type->struct_v2.fieldlist, type->struct_v2.derived,
                type->struct_v2.vshape, value);
@@ -894,37 +894,37 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
 
     case LF_STRUCTURE_V3:
     case LF_CLASS_V3:
-        leaf_len = numeric_leaf(&value, &type->struct_v3.structlen);
-        str = (const char*)&type->struct_v3.structlen + leaf_len;
+        leaf_len = numeric_leaf(&value, type->struct_v3.data);
+        str = (const char*)&type->struct_v3.data[leaf_len];
         printf("\t%x => %s V3 '%s' elts:%u property:%s\n"
                "                fieldlist-type:%x derived-type:%x vshape:%x size:%u\n",
                curr_type, type->generic.id == LF_CLASS_V3 ? "Class" : "Struct",
                str, type->struct_v3.n_element, get_property(type->struct_v3.property),
                type->struct_v3.fieldlist, type->struct_v3.derived,
                type->struct_v3.vshape, value);
-        if (type->union_v3.property.has_decorated_name)
+        if (type->struct_v3.property.has_decorated_name)
             printf("\t\tDecorated name:%s\n", str + strlen(str) + 1);
         break;
 
     case LF_UNION_V1:
-        leaf_len = numeric_leaf(&value, &type->union_v1.un_len);
+        leaf_len = numeric_leaf(&value, type->union_v1.data);
         printf("\t%x => Union V1 '%s' count:%u property:%s fieldlist-type:%x size:%u\n",
-               curr_type, p_string(PSTRING(&type->union_v1.un_len, leaf_len)),
+               curr_type, p_string((const struct p_string *)&type->union_v1.data[leaf_len]),
                type->union_v1.count, get_property(type->union_v1.property),
                type->union_v1.fieldlist, value);
         break;
 
     case LF_UNION_V2:
-        leaf_len = numeric_leaf(&value, &type->union_v2.un_len);
+        leaf_len = numeric_leaf(&value, type->union_v2.data);
         printf("\t%x => Union V2 '%s' count:%u property:%s fieldlist-type:%x size:%u\n",
-               curr_type, p_string(PSTRING(&type->union_v2.un_len, leaf_len)),
+               curr_type, p_string((const struct p_string *)&type->union_v2.data[leaf_len]),
                type->union_v2.count, get_property(type->union_v2.property),
                type->union_v2.fieldlist, value);
         break;
 
     case LF_UNION_V3:
-        leaf_len = numeric_leaf(&value, &type->union_v3.un_len);
-        str = (const char*)&type->union_v3.un_len + leaf_len;
+        leaf_len = numeric_leaf(&value, type->union_v3.data);
+        str = (const char*)&type->union_v3.data[leaf_len];
         printf("\t%x => Union V3 '%s' count:%u property:%s fieldlist-type:%x size:%u\n",
                curr_type, str, type->union_v3.count,
                get_property(type->union_v3.property),
@@ -958,7 +958,7 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
                type->enumeration_v3.fieldlist,
                type->enumeration_v3.count,
                get_property(type->enumeration_v3.property));
-        if (type->union_v3.property.has_decorated_name)
+        if (type->enumeration_v3.property.has_decorated_name)
             printf("\t\tDecorated name:%s\n", type->enumeration_v3.name + strlen(type->enumeration_v3.name) + 1);
         break;
 
@@ -990,7 +990,7 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
     case LF_PROCEDURE_V2:
         printf("\t%x => Procedure V2 ret_type:%x callconv:%s attr:%s (#%u args_type:%x)\n",
                curr_type, type->procedure_v2.rvtype,
-               get_callconv(type->procedure_v2.callconv), get_funcattr(type->procedure_v1.funcattr),
+               get_callconv(type->procedure_v2.callconv), get_funcattr(type->procedure_v2.funcattr),
                type->procedure_v2.params, type->procedure_v2.arglist);
         break;
 
@@ -1009,13 +1009,19 @@ static void codeview_dump_one_type(unsigned curr_type, const union codeview_type
         break;
 
     case LF_MODIFIER_V1:
-        printf("\t%x => Modifier V1 type:%x modif:%x\n",
-               curr_type, type->modifier_v1.type, type->modifier_v1.attribute);
+        printf("\t%x => Modifier V1 type:%x modif:%x%s%s%s\n",
+               curr_type, type->modifier_v1.type, type->modifier_v1.attribute,
+               (type->modifier_v1.attribute & 1) ? "-const" : "",
+               (type->modifier_v1.attribute & 2) ? "-volatile" : "",
+               (type->modifier_v1.attribute & 4) ? "-unaligned" : "");
         break;
 
     case LF_MODIFIER_V2:
-        printf("\t%x => Modifier V2 type:%x modif:%x\n",
-               curr_type, type->modifier_v2.type, type->modifier_v2.attribute);
+        printf("\t%x => Modifier V2 type:%x modif:%x%s%s%s\n",
+               curr_type, type->modifier_v2.type, type->modifier_v2.attribute,
+               (type->modifier_v2.attribute & 1) ? "-const" : "",
+               (type->modifier_v2.attribute & 2) ? "-volatile" : "",
+               (type->modifier_v2.attribute & 4) ? "-unaligned" : "");
         break;
 
     case LF_METHODLIST_V1:
@@ -1180,7 +1186,7 @@ BOOL codeview_dump_types_from_offsets(const void* table, const DWORD* offsets, u
     return TRUE;
 }
 
-BOOL codeview_dump_types_from_block(const void* table, unsigned long len)
+BOOL codeview_dump_types_from_block(const void* table, unsigned long len, unsigned int from, unsigned int to)
 {
     unsigned int        curr_type = 0x1000;
     const unsigned char*ptr = table;
@@ -1189,7 +1195,8 @@ BOOL codeview_dump_types_from_block(const void* table, unsigned long len)
     {
         const union codeview_type* type = (const union codeview_type*)ptr;
 
-        codeview_dump_one_type(curr_type, type);
+        if (from <= curr_type && curr_type < to)
+            codeview_dump_one_type(curr_type, type);
         curr_type++;
         ptr += type->generic.len + 2;
     }
@@ -1285,7 +1292,7 @@ static void dump_binannot(const unsigned char* ba, const char* last, unsigned in
         case BA_OP_ChangeCodeOffsetAndLineOffset:
             {
                 unsigned p1 = binannot_uncompress(&ba);
-                printf("%*s  | ChangeCodeOffsetAndLineOffset %u %u (0x%x)\n", indent, "", p1 & 0xf, binannot_getsigned(p1 >> 4), p1);
+                printf("%*s  | ChangeCodeOffsetAndLineOffset %u %d (0x%x)\n", indent, "", p1 & 0xf, binannot_getsigned(p1 >> 4), p1);
             }
             break;
         case BA_OP_ChangeCodeLengthAndCodeOffset:
@@ -1356,6 +1363,9 @@ BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long
     int          length;
     struct symbol_dumper sd;
 
+    if (start == sizeof(unsigned))
+        printf("        [header: %x]\n", *((unsigned*)root));
+
     init_symbol_dumper(&sd);
     /*
      * Loop over the different types of records and whenever we
@@ -1429,11 +1439,22 @@ BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long
 	case S_DATAREF:
 	case S_PROCREF:
 	case S_LPROCREF:
-            printf("%sref V3 '%s' %04x:%08x name:%08x\n",
-                   sym->generic.id == S_DATAREF ? "Data" :
-                                      (sym->generic.id == S_PROCREF ? "Proc" : "Lproc"),
-                   get_symbol_str(sym->refsym2_v3.name),
-                   sym->refsym2_v3.imod, sym->refsym2_v3.ibSym, sym->refsym2_v3.sumName);
+	case S_TOKENREF:
+        case S_ANNOTATIONREF:
+            {
+                const char *kind;
+                switch (sym->generic.id)
+                {
+                case S_DATAREF:       kind = "Data";       break;
+                case S_PROCREF:       kind = "Proc";       break;
+                case S_LPROCREF:      kind = "LProc";      break;
+                case S_TOKENREF:      kind = "Token";      break;
+                case S_ANNOTATIONREF: kind = "Annotation"; break;
+                default:              kind = "----";       break;
+                }
+                printf("%sref V3 '%s' %04x:%08x name:%08x\n", kind, get_symbol_str(sym->refsym2_v3.name),
+                       sym->refsym2_v3.imod, sym->refsym2_v3.ibSym, sym->refsym2_v3.sumName);
+            }
 	    break;
 
         /*
@@ -1699,9 +1720,9 @@ BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long
                 int             vlen;
                 struct full_value fv;
 
-                vlen = full_numeric_leaf(&fv, &sym->constant_v2.cvalue);
+                vlen = full_numeric_leaf(&fv, sym->constant_v2.data);
                 printf("Constant V2 '%s' = %s type:%x\n",
-                       p_string(PSTRING(&sym->constant_v2.cvalue, vlen)),
+                       p_string((const struct p_string *)&sym->constant_v2.data[vlen]),
                        full_value_string(&fv), sym->constant_v2.type);
             }
             break;
@@ -1711,9 +1732,9 @@ BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long
                 int             vlen;
                 struct full_value fv;
 
-                vlen = full_numeric_leaf(&fv, &sym->constant_v3.cvalue);
+                vlen = full_numeric_leaf(&fv, sym->constant_v3.data);
                 printf("Constant V3 '%s' =  %s type:%x\n",
-                       (const char*)&sym->constant_v3.cvalue + vlen,
+                       (const char*)&sym->constant_v3.data[vlen],
                        full_value_string(&fv), sym->constant_v3.type);
             }
             break;
@@ -1889,6 +1910,7 @@ BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long
         case S_INLINEES:
             {
                 unsigned i, ninvoc;
+                const cv_typ_t *functions;
                 const unsigned* invoc;
                 const char* tag;
 
@@ -1896,12 +1918,15 @@ BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long
                 else if (sym->generic.id == S_CALLEES) tag = "Callees";
                 else tag = "Inlinees";
                 printf("%s V3 count:%u\n", tag, sym->function_list_v3.count);
-                invoc = (const unsigned*)&sym->function_list_v3.funcs[sym->function_list_v3.count];
+                functions = (const cv_typ_t *)&sym->function_list_v3.data;
+                invoc = (const unsigned*)&functions[sym->function_list_v3.count];
                 ninvoc = (const unsigned*)get_last(sym) - invoc;
+                if (ninvoc < sym->function_list_v3.count) ninvoc = sym->function_list_v3.count;
 
-                for (i = 0; i < sym->function_list_v3.count; ++i)
+                for (i = 0; i < ninvoc; ++i)
                     printf("%*s| func:%x invoc:%u\n",
-                           indent, "", sym->function_list_v3.funcs[i], i < ninvoc ? invoc[i] : 0);
+                           indent, "", functions[i], invoc[i]);
+                if (i < sym->function_list_v3.count) printf("Number of entries exceed symbol serialized size\n");
             }
             break;
 
@@ -1954,6 +1979,48 @@ BOOL codeview_dump_symbols(const void* root, unsigned long start, unsigned long
                    sym->pogoinfo_v3.numInstrs, sym->pogoinfo_v3.staInstLive);
             break;
 
+        case S_GMANPROC:
+        case S_LMANPROC:
+            printf("%s Managed Procedure V3: '%s' (%04x:%08x#%x) attr:%x\n",
+                   sym->generic.id == S_GMANPROC ? "Global" : "Local",
+                   sym->managed_proc_v3.name,
+                   sym->managed_proc_v3.sect, sym->managed_proc_v3.off, sym->managed_proc_v3.proc_len,
+                   sym->managed_proc_v3.flags);
+            printf("%*s\\- Debug: start=%08x end=%08x\n",
+                   indent, "", sym->managed_proc_v3.debug_start, sym->managed_proc_v3.debug_end);
+            printf("%*s\\- parent:<%x> end:<%x> next<%x>\n",
+                   indent, "", sym->managed_proc_v3.pparent, sym->managed_proc_v3.pend, sym->managed_proc_v3.pnext);
+            printf("%*s\\- token:%x retReg:%x\n",
+                   indent, "", sym->managed_proc_v3.token, sym->managed_proc_v3.ret_reg);
+            push_symbol_dumper(&sd, sym, sym->managed_proc_v3.pend);
+            break;
+
+        case S_MANSLOT:
+            printf("Managed slot V3: '%s' type:%x attr:%s slot:%u\n",
+                   sym->managed_slot_v3.name, sym->managed_slot_v3.typeid,
+                   get_varflags(sym->managed_slot_v3.attr), sym->managed_slot_v3.islot);
+            break;
+
+        case S_OEM:
+            printf("OEM symbol V3 guid=%s type=%x\n",
+                   get_guid_str(&sym->oem_v3.idOEM), sym->oem_v3.typeid);
+            {
+                const unsigned int *from = (const void*)sym->oem_v3.rgl;
+                const unsigned int *last = (unsigned int*)((unsigned char*)sym + 2 + sym->generic.len);
+                printf("%*s\\- rgl: [", indent, "");
+                for (; from < last; from++) printf("%08x%s", *from, (from + 1) < last ? " " : "");
+                printf("]\n");
+            }
+            break;
+
+        case S_ARMSWITCHTABLE:
+            printf("ARM switch table type=%u base=%04x:%08x branch=%04x:%08x table=%04x:%08x entries=#%u\n",
+                   sym->armswitchtable.switch_type,
+                   sym->armswitchtable.base_section, sym->armswitchtable.base_offset,
+                   sym->armswitchtable.branch_section, sym->armswitchtable.branch_offset,
+                   sym->armswitchtable.table_section, sym->armswitchtable.table_offset,
+                   sym->armswitchtable.number_entries);
+            break;
         default:
             printf("\n\t\t>>> Unsupported symbol-id %x sz=%d\n", sym->generic.id, sym->generic.len + 2);
             dump_data((const void*)sym, sym->generic.len + 2, "  ");
@@ -2017,7 +2084,7 @@ void codeview_dump_linetab(const char* linetab, BOOL pascal_str, const char* pfx
     }
 }
 
-void codeview_dump_linetab2(const char* linetab, DWORD size, const char* strimage, DWORD strsize, const char* pfx)
+void codeview_dump_linetab2(const char* linetab, DWORD size, const PDB_STRING_TABLE* strimage, const char* pfx)
 {
     unsigned    i;
     const struct CV_DebugSSubsectionHeader_t*     hdr;
@@ -2094,7 +2161,7 @@ void codeview_dump_linetab2(const char* linetab, DWORD size, const char* strimag
                 const char* meth[] = {"None", "MD5", "SHA1", "SHA256"};
                 printf("%s  %d] name=%s size=%u method=%s checksum=[",
                        pfx, (unsigned)((const char*)chksms - (const char*)(hdr + 1)),
-                       strimage ? strimage + chksms->strOffset : "--none--",
+                       pdb_get_string_table_entry(strimage, chksms->strOffset),
                        chksms->size, chksms->method < ARRAY_SIZE(meth) ? meth[chksms->method] : "<<unknown>>");
                 for (i = 0; i < chksms->size; ++i) printf("%02x", chksms->checksum[i]);
                 printf("]\n");
